--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
--          ADC 16bits, 4 channels
--          Vref = internal (pin REF=REFIN=GND)
--          Full scale [0V;Vref]
--          Vin = Vref * (D / 65536)
--          ADC 1LSb = Vref/65536
--          Straight binary eigital encodin
--          
--          Conversion time: 3.2us (time between converion = 5us)
--
--          38.4MHz max SPI clock (26ns cycle)
--          polarity = 0 (polarity=0: SCK low on idle ; polarity=1: SCK high on idle )
--          phase    = 0 (for polarity 0 phase=0: MOSI changes on SCK falling; phase=1: rising)
--                       (for polarity 1 phase=0: MOSI changes on SCK rising;  phase=1: falling)
--
--          send 14bits : 14  13   12   11   10  9   8   7  6   5   4   2   1   0
--                        CFG INCC INCC INCC INx INx INx BW REF REF REF SEQ SEQ RB
--
--          Modes: with or without busy indicator
--              used - read/write after conversion (RAC) => respect tCYC between accesses (5us)
--                   - read/write during conversion (RDC)
--                   - read/write spanning conversion (RSC)
--
--
--          CFG:  update the configuration 
--                     0 :   keep previous
--                used 1 : update previous
--          INCC: input channel configuration
--                     0   0   X : bipolar diff pairs ref to Vref/2
--                     0   1   0 : bipolar ref to COM
--                used 0   1   1 : Temperature
--                     0   0   X : unipolar diff pairs ref to GND
--                     1   1   0 : unipolar ref to COM
--                used 1   1   1 : unipolar ref to GND
--          INx: input channel selection
--                used X   0   0 : IN0
--                used X   0   1 : IN1
--                used X   1   0 : IN2
--                used X   1   1 : IN3
--          BW:  bandwidth selection
--                     0 : 1/4 bandwidth (limit the noise)
--                used 1 : full bandwidth
--          REF: Reference/buffer selection
--                used 0   0   0 : internal REF 2.5V   + temperature
--                     0   0   1 : internal REF 4.096V + temperature
--                     0   1   0 : external REF + temperature 
--                     0   1   1 : external REF + temperature + internal buffer
--                     1   0   0 : reserved
--                     1   0   1 : reserved
--                     1   1   0 : external REF
--                     1   1   1 : external REF + internal buffer
--          SEQ: Channel sequencer
--                used 0   0  : disable
--                     0   1  : update configuration during sequence
--                     1   0  : scan IN0 to IN3 (set in INx) then temperature
--                     1   1  : scan IN0 to IN3 (set in INx) 
--          RB: Read back the CFG register
--                     0 : read back the current CFG at the end
--                used 1 : no read-back
--
--          Temperature: offset 283mV at 25°C
--                       slope 1mV per °C
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity adc_4ch_ad7682 is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS               : integer  := 1;     -- 1=true / 0=false 
    --main clock frequency
    g_CLOCK_FREQ              : positive := 100E6; -- Main clk freq in Hz
    -- Refresh period    
    g_WAIT_GAP_BEFORE_REFRESH : integer  := 0      -- clock cycles between the next conversion         
  );
  port(
    --clk & rst
    rst_ir                    : in  std_logic; 
    clk_ik                    : in  std_logic; 
    cen_ie                    : in  std_logic; 
    --FMC enable              
    enable_i                  : in  std_logic; 
    --ADC CTRL                
    ADC3_LV_volt_valid_o      : out std_logic;
    ADC3_LV_volt_ob16         : out std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o     : out std_logic;
    ADC3_HV1_curr_ob16        : out std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o     : out std_logic;
    ADC3_HV2_curr_ob16        : out std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o      : out std_logic;
    ADC3_LV_curr_ob16         : out std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o     : out std_logic;
    ADC3_4ch_temp_ob16        : out std_logic_vector(15 downto 0); 
    --SPI output              
    SPI_sync_no               : out std_logic;
    SPI_clk_o                 : out std_logic; 
    SPI_mosi_o                : out std_logic;
    SPI_miso_o                : in  std_logic
  );
end entity;


------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of adc_4ch_ad7682 is
  
  function ceil_div(a, b : integer) return integer is
  begin
      if ( b /= 0 ) then
        return (a + b-1)/b; --rounded up
      else
        return a;
      end if;
  end ceil_div;
   
  function log2(x : positive) return natural is
     constant log2_max : natural  := natural'high;
     variable log2_x   : natural  := 0;
     variable x_approx : positive := 1;
  begin
     for log2_x in 0 to log2_max loop
        if ( x_approx < x ) then
           x_approx := 2*x_approx;
        else
           return log2_x;
        end if;
     end loop;
     return log2_max;
  end log2;
  
  --SPI
  constant c_SPI_FREQ_HZ         : positive := 10E6;                                  -- 10MHz (38MHz max)
  constant c_SPI_SCK_RATE        : positive := ceil_div(g_CLOCK_FREQ, c_SPI_FREQ_HZ); -- 100MHz/10MHz ~=100 => 100ns
  constant c_SPI_SCK_RATE_LENGTH : positive := log2(c_SPI_SCK_RATE)+1;                -- number of bit of the clock division counter
  constant c_SPI_SLAVE_NB        : positive := 1;                                     -- slave selection (hot bit)
  constant c_CMB_SPI_DATA_LEN    : positive := 16;                                    -- SPI frame length in bits

  --CMB
  signal cmb_frame               : std_logic;
  signal cmb_addr_b              : std_logic_vector(c_SPI_SLAVE_NB-1 downto 0);
  signal cmb_enable_b            : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_wr                  : std_logic;
  signal cmb_wr_data_b           : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd                  : std_logic;
  signal cmb_rd_data_b           : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd_valid            : std_logic;
  signal cmb_busy                : std_logic;
  
  --Timer
  signal timer_1us               : std_logic;

begin 

  ------------------------------------
  -- Timer 1us (100MHz=>1MHz)
  ------------------------------------
  i_timer_1us : entity work.timer
  generic map(
    g_IN_EN_PULSE_FREQ => g_CLOCK_FREQ,  -- integer -- Enable Input Frequency 
    g_TIMER_PULSE_FREQ => 1E6            -- integer -- Pulse Frequency 
  )
  port map(
    clk_ik             => clk_ik,        -- in  std_logic; -- input Clock at G_IN_CLOCK_FREQ
    rst_ir             => rst_ir,        -- in  std_logic; -- input Reset
    cen_ie             => cen_ie,        -- in  std_logic; -- input clock enable
    clr_timer_i        => cmb_busy,      -- in  std_logic; -- input clear
    timer_en_ie        => '1',           -- in  std_logic; -- timer enable (can be an higher rate timer)
    timer_pulse_o      => timer_1us      -- out std_logic  -- Pulse generated at frequency G_TIMER_PULSE_FREQ
  );
  

 
  --------------------------------------------------------------------------------------------------
  -- SPI protocol
  --------------------------------------------------------------------------------------------------
  i_adc_4ch_ad7682_spi_protocol: entity work.adc_4ch_ad7682_spi_protocol
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS               => g_SYNTHESIS,               -- integer  := 1;   -- 1=true / 0=false
    -- Refresh period    
    g_WAIT_GAP_BEFORE_REFRESH => g_WAIT_GAP_BEFORE_REFRESH, -- integer  := 0;   -- clock cycles between the next conversion         
    -- Local slave bus
    g_CMB_SPI_ADDR_LEN        => c_SPI_SLAVE_NB,            -- positive := 1;   -- slave selection (hot bit)
    g_CMB_SPI_DATA_LEN        => c_CMB_SPI_DATA_LEN         -- positive := 8;   -- SPI frame length in bits
  )
  port map(
    -- Clock and reset
    rst_ir                    => rst_ir,                    -- in  std_logic;
    clk_ik                    => clk_ik,                    -- in  std_logic;
    cen_ie                    => cen_ie,                    -- in  std_logic;
    --enable
    enable_i                  => enable_i,                  -- in  std_logic; 
    --timer 
    timer_1us_i               => timer_1us,                 -- in  std_logic; 
    --ADC CTRL
    ADC3_LV_volt_valid_o      => ADC3_LV_volt_valid_o,      -- out std_logic;
    ADC3_LV_volt_ob16         => ADC3_LV_volt_ob16,         -- out std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o     => ADC3_HV1_curr_valid_o,     -- out std_logic;
    ADC3_HV1_curr_ob16        => ADC3_HV1_curr_ob16,        -- out std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o     => ADC3_HV2_curr_valid_o,     -- out std_logic;
    ADC3_HV2_curr_ob16        => ADC3_HV2_curr_ob16,        -- out std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o      => ADC3_LV_curr_valid_o,      -- out std_logic;
    ADC3_LV_curr_ob16         => ADC3_LV_curr_ob16,         -- out std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o     => ADC3_4ch_temp_valid_o,     -- out std_logic;
    ADC3_4ch_temp_ob16        => ADC3_4ch_temp_ob16,        -- out std_logic_vector(15 downto 0); 
    -- CMB SPI output
    cmb_frame_o               => cmb_frame,                 -- out std_logic;
    cmb_addr_ob               => cmb_addr_b,                -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ob             => cmb_enable_b,              -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_o                  => cmb_wr ,                   -- out std_logic;
    cmb_wr_data_ob            => cmb_wr_data_b,             -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_o                  => cmb_rd,                    -- out std_logic;
    cmb_rd_data_ib            => cmb_rd_data_b,             -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_i            => cmb_rd_valid,              -- in  std_logic;
    cmb_busy_i                => cmb_busy                   -- in  std_logic;
  );



  --------------------------------------------------------------------------------------------------
  -- SPI PHY master
  --------------------------------------------------------------------------------------------------
  i_spi_phy_master: entity work.spi_phy_master
  generic map(
    -- Local slave bus
    g_CMB_ADDR_LEN_SS_NUM => c_SPI_SLAVE_NB,       -- positive := 1; -- slave selection (hot bit)
    g_CMB_DATA_LEN        => c_CMB_SPI_DATA_LEN,   -- positive := 8; -- SPI frame length in bits
    -- SPI bus                            
    g_SPI_SCK_RATE_LENGTH => c_SPI_SCK_RATE_LENGTH -- positive := 8  -- clock period division
  )
  port map(
    -- Local slave bus
    rst_ir                => rst_ir,               -- in  std_logic;
    clk_ik                => clk_ik,               -- in  std_logic;
    cen_ie                => cen_ie,               -- in  std_logic;
    -- CMB input                                   
    cmb_frame_i           => cmb_frame,            -- in  std_logic;
    cmb_addr_ib           => cmb_addr_b,           -- in  std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ib         => cmb_enable_b,         -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_i              => cmb_wr,               -- in  std_logic;
    cmb_wr_data_ib        => cmb_wr_data_b,        -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_i              => cmb_rd,               -- in  std_logic;
    cmb_rd_data_ob        => cmb_rd_data_b,        -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_o        => cmb_rd_valid,         -- out std_logic;
    cmb_busy_o            => cmb_busy,             -- out std_logic;
    -- SPI control                                 
    spi_sck_phase_i       => '0',                  -- in  std_logic;
    spi_sck_polarity_i    => '0',                  -- in  std_logic;
    spi_sck_rate_ib       => std_logic_vector(to_unsigned(
                             c_SPI_SCK_RATE, c_SPI_SCK_RATE_LENGTH)), -- in  std_logic_vector(g_SPI_SCK_RATE_LENGTH-1 downto 0); --clock division factor
    -- SPI bus                                     
    spi_cs_on(0)          => SPI_sync_no,          -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0);
    spi_sck_ok            => SPI_clk_o,            -- out std_logic;
    spi_mosi_o            => SPI_mosi_o,           -- out std_logic;
    spi_mosi_oe_n         => open,                 -- out std_logic; --for tri-state outputs
    spi_miso_i            => SPI_miso_o            -- in  std_logic
  );

end architecture;