--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
--          ADC 24bits, 6.6SPS
--          Vref = 2.5V (IC13 REF3125AQDBZRQ1)
--          Full scale [-0.5Vref;+0.5Vref] = [-1.25V;+1.25V]
--          Internal clock (Pin F0=GND)
--
--          data output is straight binary : pin COM=GND
--
--          Vin = Vref * (D / 16777216)
--          ADC 1LSb = 2500/16777216 = 0.149uV 
--          ADC full scale = [upper_limit ; lower_limit] = [0.162V ; 2.336V]
--          HV gain R2/R1+1=18000/10000+1=2.8 
--          HV resolution: 1LSb = ADC_lsb * |HVin|  * 2 / (upper_limit - lower_limit) 
--                              = 0.149   * |-1750| * 2 / (2.223       - 0.0492     )
--                              = 239.9mV
--          HV full scale = [-1750V ; 0V]
--          
--          4MHz max SPI clock (250ns cycle)
--          polarity = 1 (polarity=0: SCK low on idle ; polarity=1: SCK high on idle )
--          phase    = 1 (for polarity 0 phase=0: MOSI changes on SCK falling; phase=1: rising)
--                       (for polarity 1 phase=0: MOSI changes on SCK rising;  phase=1: falling)
--          send 32bits : 31    30  29  28  27  26  25  24  ... 5  | 4   3   2   1   0
--          used 27bits : 26    25  24  23  22  21  20  19  ... 0  |
--              MISO-SDO  EOC_n DMY SIG D23 D22 D21 D20 D19 ... D0 | SUB SUB SUB SUB SUB
--              MOSI-SDI  EN    X   X   X   IM  FOA FOB SPD ... X  | X   X   X   X   X
--
--          EOC_n = High during the conversion
--          DMY   = Dummy always low
--          SIG   = Sign (High=positive, Low=negative)
--
--          If SIG=1 and D23=1 => overflow  (Vin > +0.5Vref)
--          If SIG=0 and D23=0 => underflow (Vin < -0.5Vref)
--
--          Modes:  EN  IM  FOA FOB SPD 
--                  0   X   X   X   X  : keep previsou mode
--             used 1   0   0   0   0  : External Input, 50Hz and 60Hz Rejection, Autocalibration    => CONV in 147.0ms
--                  1   0   0   0   1  : External Input, 50Hz and 60Hz Rejection, 2x Speed           => CONV in  73.6ms
--                  1   0   0   1   0  : External Input, 50Hz Rejection, Autocalibration             => CONV in 160.0ms
--                  1   0   0   1   1  : External Input, 50Hz Rejection, 2x Speed                    => CONV in  80.0ms
--                  1   0   1   0   0  : External Input, 60Hz Rejection, Autocalibration             => CONV in 133.0ms
--                  1   0   1   0   1  : External Input, 60Hz Rejection, 2x Speed                    => CONV in  67.0ms
--                  1   0   1   1   0  : reserved
--                  1   0   1   1   1  : reserved
--             used 1   1   0   0   0  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
--                  1   1   0   0   1  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
--                  1   1   0   1   0  : Temperature Input, 50Hz Rejection, Autocalibration
--                  1   1   0   1   1  : Temperature Input, 50Hz Rejection, Autocalibration
--                  1   1   1   0   0  : Temperature Input, 60Hz Rejection, Autocalibration
--                  1   1   1   0   1  : Temperature Input, 60Hz Rejection, Autocalibration
--                  1   1   1   1   0  : reserved
--                  1   1   1   1   1  : reserved
--
--        EN  = Enable
--        IM  = Internal Temperature
--        FOA = No 50Hz rejection
--        FOB = No 60Hz rejection
--        SPD = Speed mode, double output rate (disable the offset auto-calibration)
--
--        Temperature: PTAT (proportional to absolute temperature) 
--                     offset 420mV at 27°C
--                     slope 420/(27+273)=1.40mV/°C or 0.714°C/mV
--                     T=0.714*mV-273     (B=1.4*(+27)/420=0.09)
--
--                     Tc = Rsdo/R0sdo*(T0+273)-273
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity adc_hv_ltc2484 is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS               : integer  := 1;     -- 1=true / 0=false 
    --main clock frequency
    g_CLOCK_FREQ              : positive := 100E6; -- Main clk freq in Hz
    -- Refresh period    
    g_WAIT_GAP_BEFORE_REFRESH : integer  := 0      -- clock cycles between the next conversion         
  );
  port(
    --clk & rst
    rst_ir          : in  std_logic; 
    clk_ik          : in  std_logic; 
    cen_ie          : in  std_logic; 
    --FMC enable
    enable_i        : in  std_logic; 
    --ADC CTRL
    ch_ob25         : out std_logic_vector(24 downto 0); 
    ch_valid_o      : out std_logic; 
    --Temperature request (give the last value and trigger a new conversion)
    temp_rd_i       : in  std_logic;
    temp_ob25       : out std_logic_vector(24 downto 0); 
    temp_rd_valid_o : out std_logic; 
    temp_busy_o     : out std_logic; 
    --SPI output    
    SPI_sync_no     : out std_logic;
    SPI_clk_o       : out std_logic; 
    SPI_mosi_o      : out std_logic;
    SPI_miso_o      : in  std_logic
  );
end entity;


------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of adc_hv_ltc2484 is
  
  function ceil_div(a, b : integer) return integer is
  begin
      if ( b /= 0 ) then
        return (a + b-1)/b; --rounded up
      else
        return a;
      end if;
  end ceil_div;
   
  function log2(x : positive) return natural is
     constant log2_max : natural  := natural'high;
     variable log2_x   : natural  := 0;
     variable x_approx : positive := 1;
  begin
     for log2_x in 0 to log2_max loop
        if ( x_approx < x ) then
           x_approx := 2*x_approx;
        else
           return log2_x;
        end if;
     end loop;
     return log2_max;
  end log2;
  
  --SPI
  constant c_SPI_FREQ_HZ         : positive := 4E6;                                   -- 4MHz max
  constant c_SPI_SCK_RATE        : positive := ceil_div(g_CLOCK_FREQ, c_SPI_FREQ_HZ); -- 100MHz/4MHz ~=25 => 250ns
  constant c_SPI_SCK_RATE_LENGTH : positive := log2(c_SPI_SCK_RATE)+1;                -- number of bit of the clock division counter
  constant c_SPI_SLAVE_NB        : positive := 1;                                     -- slave selection (hot bit)
  constant c_CMB_SPI_DATA_LEN    : positive := 27;                                    -- SPI frame length in bits

  --CMB
  signal cmb_frame               : std_logic;
  signal cmb_addr_b              : std_logic_vector(c_SPI_SLAVE_NB-1 downto 0);
  signal cmb_enable_b            : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_wr                  : std_logic;
  signal cmb_wr_data_b           : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd                  : std_logic;
  signal cmb_rd_data_b           : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd_valid            : std_logic;
  signal cmb_busy                : std_logic;
  
  --Timer
  signal timer_1us               : std_logic;

begin 

  ------------------------------------
  -- Timer 1us (100MHz=>1MHz)
  ------------------------------------
  i_timer_1us : entity work.timer
  generic map(
    g_IN_EN_PULSE_FREQ => g_CLOCK_FREQ,  -- integer -- Enable Input Frequency
    g_TIMER_PULSE_FREQ => 1E6            -- integer -- Pulse Frequency
  )
  port map(
    clk_ik             => clk_ik,        -- in  std_logic; -- input Clock at G_IN_CLOCK_FREQ
    rst_ir             => rst_ir,        -- in  std_logic; -- input Reset
    cen_ie             => cen_ie,        -- in  std_logic; -- input clock enable
    clr_timer_i        => cmb_busy,      -- in  std_logic; -- input clear
    timer_en_ie        => '1',           -- in  std_logic; -- timer enable (can be an higher rate timer)
    timer_pulse_o      => timer_1us      -- out std_logic  -- Pulse generated at frequency G_TIMER_PULSE_FREQ
  );

  --------------------------------------------------------------------------------------------------
  -- SPI protocol
  --------------------------------------------------------------------------------------------------
  i_adc_hv_ltc2484_spi_protocol: entity work.adc_hv_ltc2484_spi_protocol
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS               => g_SYNTHESIS,               -- integer  := 1;     -- 1=true / 0=false
    -- Refresh period    
    g_WAIT_GAP_BEFORE_REFRESH => g_WAIT_GAP_BEFORE_REFRESH, -- integer := 0;      -- clock cycles between the next conversion         
    -- Local slave bus
    g_CMB_SPI_ADDR_LEN        => c_SPI_SLAVE_NB,            -- positive := 1;     -- slave selection (hot bit)
    g_CMB_SPI_DATA_LEN        => c_CMB_SPI_DATA_LEN         -- positive := 8;     -- SPI frame length in bits
  )
  port map(
    -- Clock and reset
    rst_ir                    => rst_ir,                    -- in  std_logic;
    clk_ik                    => clk_ik,                    -- in  std_logic;
    cen_ie                    => cen_ie,                    -- in  std_logic;
    --enable
    enable_i                  => enable_i,                  -- in  std_logic; 
    --timer 
    timer_1us_i               => timer_1us,                 -- in  std_logic; 
    --ADC CTRL
    ch_ob25                   => ch_ob25,                   -- out std_logic_vector(23 downto 0); 
    ch_valid_o                => ch_valid_o,                -- out std_logic; 
    --Temperature request (give the last value and trigger a new conversion)
    temp_rd_i                 => temp_rd_i,                 -- in  std_logic;
    temp_ob25                 => temp_ob25,                 -- out std_logic_vector(23 downto 0); 
    temp_rd_valid_o           => temp_rd_valid_o,           -- out std_logic; 
    temp_busy_o               => temp_busy_o,               -- out std_logic; 
    -- CMB SPI output
    cmb_frame_o               => cmb_frame,                 -- out std_logic;
    cmb_addr_ob               => cmb_addr_b,                -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ob             => cmb_enable_b,              -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_o                  => cmb_wr ,                   -- out std_logic;
    cmb_wr_data_ob            => cmb_wr_data_b,             -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_o                  => cmb_rd,                    -- out std_logic;
    cmb_rd_data_ib            => cmb_rd_data_b,             -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_i            => cmb_rd_valid,              -- in  std_logic;
    cmb_busy_i                => cmb_busy                   -- in  std_logic;
  );



  --------------------------------------------------------------------------------------------------
  -- SPI PHY master
  --------------------------------------------------------------------------------------------------
  i_spi_phy_master: entity work.spi_phy_master
  generic map(
    -- Local slave bus
    g_CMB_ADDR_LEN_SS_NUM => c_SPI_SLAVE_NB,       -- positive := 1; -- slave selection (hot bit)
    g_CMB_DATA_LEN        => c_CMB_SPI_DATA_LEN,   -- positive := 8; -- SPI frame length in bits
    -- SPI bus                            
    g_SPI_SCK_RATE_LENGTH => c_SPI_SCK_RATE_LENGTH -- positive := 8  -- clock period division
  )
  port map(
    -- Local slave bus
    rst_ir                => rst_ir,               -- in  std_logic;
    clk_ik                => clk_ik,               -- in  std_logic;
    cen_ie                => cen_ie,               -- in  std_logic;
    -- CMB input                                   
    cmb_frame_i           => cmb_frame,            -- in  std_logic;
    cmb_addr_ib           => cmb_addr_b,           -- in  std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ib         => cmb_enable_b,         -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_i              => cmb_wr,               -- in  std_logic;
    cmb_wr_data_ib        => cmb_wr_data_b,        -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_i              => cmb_rd,               -- in  std_logic;
    cmb_rd_data_ob        => cmb_rd_data_b,        -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_o        => cmb_rd_valid,         -- out std_logic;
    cmb_busy_o            => cmb_busy,             -- out std_logic;
    -- SPI control                                 
    spi_sck_phase_i       => '1',                  -- in  std_logic;
    spi_sck_polarity_i    => '1',                  -- in  std_logic;
    spi_sck_rate_ib       => std_logic_vector(to_unsigned(
                             c_SPI_SCK_RATE, c_SPI_SCK_RATE_LENGTH)), -- in  std_logic_vector(g_SPI_SCK_RATE_LENGTH-1 downto 0); --clock division factor
    -- SPI bus                                     
    spi_cs_on(0)          => SPI_sync_no,          -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0);
    spi_sck_ok            => SPI_clk_o,            -- out std_logic;
    spi_mosi_o            => SPI_mosi_o,           -- out std_logic;
    spi_mosi_oe_n         => open,                 -- out std_logic; --for tri-state outputs
    spi_miso_i            => SPI_miso_o            -- in  std_logic
  );


end architecture;