--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
--          ADC 24bits, 6.6SPS
--          Vref = 2.5V (IC13 REF3125AQDBZRQ1)
--          Full scale [-0.5Vref;+0.5Vref] = [-1.25V;+1.25V]
--          Internal clock (Pin F0=GND)
--
--          Vin = Vref * (D / 16777216)
--          ADC 1LSb = 2500/16777216 = 0.149uV 
--          ADC full scale = [upper_limit ; lower_limit] = [0.162V ; 2.336V]
--          HV gain R2/R1+1=18000/10000+1=2.8 
--          HV resolution: 1LSb = ADC_lsb * |HVin|  * 2 / (upper_limit - lower_limit) 
--                              = 0.149   * |-1750| * 2 / (2.223       - 0.0492     )
--                              = 239.9mV
--          HV full scale = [-1750V ; 0V]
--          
--          4MHz max SPI clock (250ns cycle)
--          polarity = 1 (polarity=0: SCK low on idle ; polarity=1: SCK high on idle )
--          phase    = 1 (for polarity 0 phase=0: MOSI changes on SCK falling; phase=1: rising)
--                       (for polarity 1 phase=0: MOSI changes on SCK rising;  phase=1: falling)
--          send 32bits : 31    30  29  28  27  26  25  24  ... 5  | 4   3   2   1   0
--          used 27bits : 26    25  24  23  22  21  20  19  ... 0  |
--              MISO-SDO  EOC_n DMY SIG D23 D22 D21 D20 D19 ... D0 | SUB SUB SUB SUB SUB
--              MOSI-SDI  EN    X   X   X   IM  FOA FOB SPD ... X  | X   X   X   X   X
--
--          EOC_n = High during the conversion
--          DMY   = Dummy always low
--          SIG   = Sign (High=positive, Low=negative)
--
--          If SIG=1 and D23=1 => overflow  (Vin > +0.5Vref)
--          If SIG=0 and D23=0 => underflow (Vin < -0.5Vref)
--
--          Modes:  EN  IM  FOA FOB SPD 
--                  0   X   X   X   X  : keep previsou mode
--             used 1   0   0   0   0  : External Input, 50Hz and 60Hz Rejection, Autocalibration    => CONV in 147.0ms
--                  1   0   0   0   1  : External Input, 50Hz and 60Hz Rejection, 2x Speed           => CONV in  73.6ms
--                  1   0   0   1   0  : External Input, 50Hz Rejection, Autocalibration             => CONV in 160.0ms
--                  1   0   0   1   1  : External Input, 50Hz Rejection, 2x Speed                    => CONV in  80.0ms
--                  1   0   1   0   0  : External Input, 60Hz Rejection, Autocalibration             => CONV in 133.0ms
--                  1   0   1   0   1  : External Input, 60Hz Rejection, 2x Speed                    => CONV in  67.0ms
--                  1   0   1   1   0  : reserved
--                  1   0   1   1   1  : reserved
--             used 1   1   0   0   0  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
--                  1   1   0   0   1  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
--                  1   1   0   1   0  : Temperature Input, 50Hz Rejection, Autocalibration
--                  1   1   0   1   1  : Temperature Input, 50Hz Rejection, Autocalibration
--                  1   1   1   0   0  : Temperature Input, 60Hz Rejection, Autocalibration
--                  1   1   1   0   1  : Temperature Input, 60Hz Rejection, Autocalibration
--                  1   1   1   1   0  : reserved
--                  1   1   1   1   1  : reserved
--
--        EN  = Enable
--        IM  = Internal Temperature
--        FOA = No 50Hz rejection
--        FOB = No 60Hz rejection
--        SPD = Speed mode, double output rate (disable the offset auto-calibration)
--
--        Temperature: PTAT (proportional to absolute temperature) 
--                     offset 420mV at 27°C
--                     slope 420/(27+273)=1.40mV/°C
--                     Tc = Rsdo/R0sdo*(T0+273)-273
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity adc_hv_ltc2484_spi_protocol is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS               : integer  := 1;   -- 1=true / 0=false
    -- Refresh period    
    g_WAIT_GAP_BEFORE_REFRESH : integer  := 0; -- clock cycles between the next conversion         
    -- Local slave bus
    g_CMB_SPI_ADDR_LEN        : positive := 1; -- slave selection (hot bit)
    g_CMB_SPI_DATA_LEN        : positive := 24 -- SPI frame length in bits
  );
  port(
    --clock and reset
    rst_ir                    : in  std_logic; 
    clk_ik                    : in  std_logic; 
    cen_ie                    : in  std_logic; 
    --enable
    enable_i                  : in  std_logic; 
    --timer 
    timer_1us_i               : in  std_logic; 
    --ADC CTRL
    ch_ob25                   : out std_logic_vector(24 downto 0); 
    ch_valid_o                : out std_logic; 
    --Temperature request (give the last value and trigger a new conversion)
    temp_rd_i                 : in  std_logic;
    temp_ob25                 : out std_logic_vector(24 downto 0); 
    temp_rd_valid_o           : out std_logic; 
    temp_busy_o               : out std_logic; 
    -- CMB SPI output
    cmb_frame_o               : out std_logic;
    cmb_addr_ob               : out std_logic_vector(g_CMB_SPI_ADDR_LEN-1 downto 0); --slave selection
    cmb_enable_ob             : out std_logic_vector(g_CMB_SPI_DATA_LEN-1 downto 0);
    cmb_wr_o                  : out std_logic;
    cmb_wr_data_ob            : out std_logic_vector(g_CMB_SPI_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_o                  : out std_logic;
    cmb_rd_data_ib            : in  std_logic_vector(g_CMB_SPI_DATA_LEN-1 downto 0);
    cmb_rd_valid_i            : in  std_logic;
    cmb_busy_i                : in  std_logic
  );
end entity ;


------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of adc_hv_ltc2484_spi_protocol is
  
  --log2 function
  function log2(x : positive) return natural is
     constant log2_max : natural  := natural'high;
     variable log2_x   : natural  := 0;
     variable x_approx : positive := 1;
  begin
     for log2_x in 0 to log2_max loop
        if ( x_approx < x ) then
           x_approx := 2*x_approx;
        else
           return log2_x;
        end if;
     end loop;
     return log2_max;
  end log2;
  
  --get conversion time function
  function get_start_time return unsigned is
    variable v_start_time : integer := 200000;
  begin
    if g_SYNTHESIS>0 then
      v_start_time := 200000;  -- 200ms 
    else
      v_start_time := 10;      -- 10us to speed up simulation
    end if; 
    return to_unsigned(v_start_time, 32); -- conversion to unsigned
  end function;
  
  --get conversion time function
  function get_conv_time return unsigned is
    variable v_conv_time : integer := 147000;
  begin
    if g_SYNTHESIS>0 then
      v_conv_time := 147000; -- 147ms 
    else
      v_conv_time := 4;      -- 4us to speed up simulation
    end if; 
    return to_unsigned(v_conv_time, 32); -- conversion to unsigned
  end function;
  
  constant zeros   : std_logic_vector(g_CMB_SPI_DATA_LEN-8-1 downto 0) := (others=>'0');
  
  -- Channel: External Input, 50Hz and 60Hz Rejection, Autocalibration => CONV in 147.0ms
  constant c_CMD_CONV_CH      : std_logic_vector(7 downto 0) := "10000000"; -- (EN=1 XXX IM=0 FOA=0 FOB=0 SPD=0)
  
  -- Temperature: Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
  constant c_CMD_CONV_TEMP    : std_logic_vector(7 downto 0) := "10001000"; -- (EN=1 XXX IM=1 FOA=0 FOB=0 SPD=0)
  
  --Conversion duration
  constant c_START_DUR_US     : unsigned(31 downto 0) := get_start_time; -- time at startup
  constant c_CONV_DUR_US      : unsigned(31 downto 0) := get_conv_time;  -- conversion time (min CONV time 5us)
  signal conv_wait_cnt_u      : unsigned(31 downto 0);

  --refresh gap counter
  signal refresh_gap_cnt_u    : unsigned(log2(g_WAIT_GAP_BEFORE_REFRESH+1)+1 downto 0);

  -- Registers (last updated value)
  signal temp_reg_b25         : std_logic_vector(24 downto 0);

  -- CMB internal
  signal ch_busy              : std_logic;
  signal temp_req             : std_logic;
  signal temp_busy            : std_logic;
  signal temp_updated         : std_logic;
  signal cmb_wr_rd            : std_logic;
  signal cmb_wr_data_b        : std_logic_vector(g_CMB_SPI_DATA_LEN-1 downto 0);

  -- SPI cycle FSM :
  type t_state is
  (s_STARTUP,
   s_IDLE,
   s_CONV,
   s_CONV_WAIT);
  attribute syn_enum_encoding : STRING;
  attribute syn_enum_encoding of t_state : type is "one-hot";
  signal state                : t_state;
  
  --new cycle of conversion
  signal start_spi_cycle      : std_logic;
  
  --current/last conversion was for channel (not temperature)
  signal ch_last_cmd          : std_logic;
  signal ch_curr_cmd          : std_logic;


begin 

  -----------------
  -- Read TEMP internal register (last updated value)
  --    - request for an update
  --    - the current value is replied as the conversion time is longer than the vme timeout
  -----------------
  p_rd_temp: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
     temp_ob25       <= (others=>'0');
     temp_rd_valid_o <= '0';
     temp_req        <= '1'; --the next conversion after reset is the temperature

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        --temperature read request
        if temp_busy = '0' then
          if temp_rd_i = '1' then
            temp_req        <= '1';           --a request for an update is done
            temp_ob25       <= temp_reg_b25;  --the current value is replied 
            temp_rd_valid_o <= '1';
          else 
            temp_rd_valid_o <= '0';
            --clear the request when the value is updated
            if temp_updated='1' then 
              temp_req <= '0';
            end if;
            
          end if;
        end if;
      end if;
    end if;
  end process;
  
  --busy output
  temp_busy   <= not(enable_i);
  temp_busy_o <= temp_busy;

  -----------------
  -- SPI gap between refresh
  -----------------
  p_refresh_gap: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
     refresh_gap_cnt_u <= (others=>'0'); --do not wait at startup
     start_spi_cycle   <= '1';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        if state           = s_IDLE then
          if refresh_gap_cnt_u = 0 then 
            start_spi_cycle   <= '1';
          else
            start_spi_cycle   <= '0';
              refresh_gap_cnt_u <= refresh_gap_cnt_u - 1;
          end if;
        else
          start_spi_cycle     <= '0';
          if g_SYNTHESIS>0 then
            refresh_gap_cnt_u <= to_unsigned(g_WAIT_GAP_BEFORE_REFRESH, refresh_gap_cnt_u'length);
          else
            refresh_gap_cnt_u <= to_unsigned(0,refresh_gap_cnt_u'length); --to speed up simulation
          end if;
        end if;
      end if;
    end if;
  end process;
 
  ----------------------------------------------------------------------------------------------------
  -- SPI state machine
  ----------------------------------------------------------------------------------------------------
  p_spi_FSM : process(rst_ir, clk_ik)
  begin
    if rst_ir = '1' then
      cmb_wr_rd       <= '0';
      cmb_wr_data_b   <= (others=>'0');
      conv_wait_cnt_u <= c_START_DUR_US;
      state           <= s_STARTUP;
      ch_curr_cmd     <= '1';    
      ch_last_cmd     <= '1';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
      
        --disable
        if enable_i ='0' then
          cmb_wr_rd       <= '0';
          cmb_wr_data_b   <= (others=>'0');
          conv_wait_cnt_u <= c_START_DUR_US;
          state           <= s_STARTUP;
          ch_curr_cmd     <= '1';    
          ch_last_cmd     <= '1';
        else
        
          case state is
          
            -----------------
            -- wait at startup
            when s_STARTUP =>
                --go to IDLE
                if conv_wait_cnt_u = 0 then
                  conv_wait_cnt_u <= c_CONV_DUR_US;
                  state           <= s_IDLE;
                --wait at startup
                else
                  if timer_1us_i='1' then
                    conv_wait_cnt_u <= conv_wait_cnt_u - 1;
                  end if;
                end if;
          
            -----------------
            -- idle : wait for the start
            when s_IDLE =>
              cmb_wr_rd         <= '0';       
              cmb_wr_data_b     <= (others=>'0');
              if start_spi_cycle = '1' then     
                state           <= s_CONV;
              end if;
          
            -----------------
            -- conversion command: 
            when s_CONV =>
              if cmb_busy_i = '0' then
                cmb_wr_rd       <= '1';
                state           <= s_CONV_WAIT;
                --temperature conversion request
                if temp_req='1' and ch_curr_cmd='1' then 
                  ch_curr_cmd   <= '0'; 
                  cmb_wr_data_b <= c_CMD_CONV_TEMP & zeros;
                --next conv is still a channel
                else
                  ch_curr_cmd   <= '1'; 
                  cmb_wr_data_b <= c_CMD_CONV_CH & zeros;
                end if;
              end if;
              
            -----------------
            -- wait for the end of the conversion: 
            when s_CONV_WAIT =>
              if cmb_busy_i = '0' then
                --stop the cycle
                if conv_wait_cnt_u = 0 then
                  conv_wait_cnt_u <= c_CONV_DUR_US;
                  ch_last_cmd     <= ch_curr_cmd; --command pipeline
                  state           <= s_IDLE;
                --wait for the end of the conversion
                else
                  cmb_wr_rd        <= '0';
                  if timer_1us_i='1' then
                    conv_wait_cnt_u <= conv_wait_cnt_u - 1;
                  end if;
                end if;
              end if;
          
            -----------------
            -- other state           
            when others =>
              state           <= s_IDLE;
          
          end case;
        end if;
      end if;
    end if;
  end process;

  -----------------
  -- SPI read valid to registers
  -----------------
  --  cmb_rd_data_ib(27) = EOC (should always be '0')
  --  cmb_rd_data_ib(26) = Dummy (unsued)
  p_spi_command: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      ch_ob25      <= (others=>'0');
      temp_reg_b25 <= (others=>'0');
      temp_updated <= '0';
      ch_valid_o   <= '0';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        temp_updated <= '0';
        ch_valid_o   <= '0';
        if cmb_rd_valid_i='1' then
          --last command was channel conversion
          if ch_last_cmd = '1' then
            ch_ob25    <= cmb_rd_data_ib(24 downto 0); -- sign + 24 bits
            ch_valid_o <= '1';
          --last command was temperature conversion
          else
            temp_reg_b25 <= cmb_rd_data_ib(24 downto 0); -- sign + 24 bits
            temp_updated <= '1';
          end if;
        -- else
          -- temp_updated <= '0';
        end if;
      end if;
    end if;
  end process;


  -------------------
  -- CMB output
  -------------------
  cmb_frame_o    <= '0';           -- no frame
  cmb_addr_ob    <= "1";           -- 1 slave only (hot bit)
  cmb_enable_ob  <= (others=>'1'); -- all 24b enable
  cmb_wr_o       <= cmb_wr_rd;     -- RD and WR in parallel (start the new conversion)
  cmb_wr_data_ob <= cmb_wr_data_b; 
  cmb_rd_o       <= cmb_wr_rd;     -- RD and WR in parallel (read the previsous acquisition)

end architecture;