--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
--          DAC 16bits
--          Vref = 2.5V (IC13 REF3125AQDBZRQ1)
--          Vout = Vref * (D / 65536)
--          DAC 1LSb = 2500/65536=3.815uV 
--          HV gain R2/R1+1=18000/10000+1=2.8 
--          HV resolution: 1LSb = 27mV
--          HV full scale = [0V ; 1750V]
--          
--          AD5663ARMZ powers-up at 0V
--          LDAC_n unused (permanently pulled low)
--          
--          50MHz max SPI clock (20ns cycle)
--          Din must be stable on SCK falling edge
--          polarity = 0 (polarity=0: SCK low on idle ; polarity=1: SCK high on idle )
--          phase    = 1 (for polarity 0 phase=0: MOSI changes on SCK falling; phase=1: rising)
--                       (for polarity 1 phase=0: MOSI changes on SCK rising;  phase=1: falling)
--
--          send 24bits :  23 22 21 20 19 18 17 16 15  ... 0
--                         X  X  C2 C1 C0 A2 A1 A0 D15 ... D0
--
--          commands: C2 C1 C0
--                     0  0  0 : WR reg n
--                     0  0  1 : update DAC reg n
--                     0  1  0 : WR reg n, update all ch (SW LDAC_n)
--                used 0  1  1 : WR and update reg/ch n
--                     1  0  0 : power down DAC (DB5/4=00: normal, 01: power-down 1kohm, 10: PD100kohm, 11: PD tri-state, DB1=chB, DB0=chA)
--                     1  0  1 : reset (DB0=0 soft reset, DB0=1 5us power-down reset)
--                     1  1  0 : LDAC_n reg setup (0=use LDAC pin, 1=don't care, DB1=chB, DB0=chA)
--                     1  1  1 : reserved
--
--          address: C2 C1 C0
--              used  0  0  0 : DAC channel 1
--                    0  0  1 : DAC channel 2
--              used  0  1  0 : reserved
--                    0  1  1 : reserved
--                    1  0  0 : reserved
--                    1  0  1 : reserved
--                    1  1  0 : reserved
--                    1  1  1 : All DAC channels
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity dac_ad5663armz is
  generic(
    g_CLOCK_FREQ : positive := 100E6 -- Main clk freq in Hz
  );
  port(
    --clk & rst
    rst_ir       : in  std_logic; 
    clk_ik       : in  std_logic; 
    cen_ie       : in  std_logic; 
    --FMC enable
    enable_i     : in  std_logic; 
    --DAC CTRL     
    chA_wr_i     : in  std_logic; 
    chA_ib16     : in  std_logic_vector(15 downto 0); 
    chA_busy_o   : out std_logic; 
    chB_wr_i     : in  std_logic; 
    chB_ib16     : in  std_logic_vector(15 downto 0); 
    chB_busy_o   : out std_logic; 
    --SPI output  
    SPI_sync_no  : out std_logic;
    SPI_clk_o    : out std_logic; 
    SPI_mosi_o   : out std_logic
  );
end entity ;


------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of dac_ad5663armz is

  function ceil_div(a, b : integer) return integer is
  begin
      if ( b /= 0 ) then
        return (a + b-1)/b; --rounded up
      else
        return a;
      end if;
  end ceil_div;
   
  function log2(x : positive) return natural is
     constant log2_max : natural  := natural'high;
     variable log2_x   : natural  := 0;
     variable x_approx : positive := 1;
  begin
     for log2_x in 0 to log2_max loop
        if ( x_approx < x ) then
           x_approx := 2*x_approx;
        else
           return log2_x;
        end if;
     end loop;
     return log2_max;
  end log2;
  
  --SPI
  constant c_SPI_FREQ_HZ         : positive := 50E6;                                  -- 50MHz max
  constant c_SPI_SCK_RATE        : positive := ceil_div(g_CLOCK_FREQ, c_SPI_FREQ_HZ); -- 100MHz/50MHz ~=2 => 20ns
  constant c_SPI_SCK_RATE_LENGTH : positive := log2(c_SPI_SCK_RATE)+1;                -- number of bit of the clock division counter
  constant c_SPI_SLAVE_NB        : positive := 1;                                     -- slave selection (hot bit)
  constant c_CMB_SPI_DATA_LEN    : positive := 24;                                    -- SPI frame length in bits

  --CMB
  signal cmb_frame     : std_logic;
  signal cmb_addr_b    : std_logic_vector(c_SPI_SLAVE_NB-1 downto 0);
  signal cmb_enable_b  : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_wr        : std_logic;
  signal cmb_wr_data_b : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd        : std_logic;
  signal cmb_rd_data_b : std_logic_vector(c_CMB_SPI_DATA_LEN-1 downto 0);
  signal cmb_rd_valid  : std_logic;
  signal cmb_busy      : std_logic;

begin 

  --------------------------------------------------------------------------------------------------
  -- SPI protocol
  --------------------------------------------------------------------------------------------------
  i_dac_ad5663_spi_protocol: entity work.dac_ad5663_spi_protocol
  generic map(
    -- Local slave bus
    g_CMB_SPI_ADDR_LEN => c_SPI_SLAVE_NB,    -- positive := 1; -- slave selection (hot bit)
    g_CMB_SPI_DATA_LEN => c_CMB_SPI_DATA_LEN -- positive := 8; -- SPI frame length in bits
  )
  port map(
    -- Clock and reset
    rst_ir             => rst_ir,            -- in  std_logic;
    clk_ik             => clk_ik,            -- in  std_logic;
    cen_ie             => cen_ie,            -- in  std_logic;
    --enable
    enable_i           => enable_i,          -- in  std_logic; 
    -- DAC Input ctrl
    chA_wr_i           => chA_wr_i,          -- in  std_logic; 
    chA_ib16           => chA_ib16,          -- in  std_logic_vector(15 downto 0); 
    chA_busy_o         => chA_busy_o,        -- out std_logic; 
    chB_wr_i           => chB_wr_i,          -- in  std_logic; 
    chB_ib16           => chB_ib16,          -- in  std_logic_vector(15 downto 0); 
    chB_busy_o         => chB_busy_o,        -- out std_logic; 
    -- CMB SPI output                        
    cmb_frame_o        => cmb_frame,         -- out std_logic;
    cmb_addr_ob        => cmb_addr_b,        -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ob      => cmb_enable_b,      -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_o           => cmb_wr ,           -- out std_logic;
    cmb_wr_data_ob     => cmb_wr_data_b,     -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_o           => cmb_rd,            -- out std_logic;
    cmb_rd_data_ib     => cmb_rd_data_b,     -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_i     => cmb_rd_valid,      -- in  std_logic;
    cmb_busy_i         => cmb_busy           -- in  std_logic;
  );



  --------------------------------------------------------------------------------------------------
  -- SPI PHY master
  --------------------------------------------------------------------------------------------------
  i_spi_phy_master: entity work.spi_phy_master
  generic map(
    -- Local slave bus
    g_CMB_ADDR_LEN_SS_NUM => c_SPI_SLAVE_NB,       -- positive := 1; -- slave selection (hot bit)
    g_CMB_DATA_LEN        => c_CMB_SPI_DATA_LEN,   -- positive := 8; -- SPI frame length in bits
    -- SPI bus                            
    g_SPI_SCK_RATE_LENGTH => c_SPI_SCK_RATE_LENGTH -- positive := 8  -- bits for clock period division counter
  )
  port map(
    -- Local slave bus
    rst_ir                => rst_ir,               -- in  std_logic;
    clk_ik                => clk_ik,               -- in  std_logic;
    cen_ie                => cen_ie,               -- in  std_logic;
    -- CMB input                                   
    cmb_frame_i           => cmb_frame,            -- in  std_logic;
    cmb_addr_ib           => cmb_addr_b,           -- in  std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ib         => cmb_enable_b,         -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_i              => cmb_wr,               -- in  std_logic;
    cmb_wr_data_ib        => cmb_wr_data_b,        -- in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_i              => cmb_rd,               -- in  std_logic;
    cmb_rd_data_ob        => cmb_rd_data_b,        -- out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_o        => cmb_rd_valid,         -- out std_logic;
    cmb_busy_o            => cmb_busy,             -- out std_logic;
    -- SPI control                                 
    spi_sck_phase_i       => '0',                  -- in  std_logic; --idle low
    spi_sck_polarity_i    => '1',                  -- in  std_logic; --stable on falling edge
    spi_sck_rate_ib       => std_logic_vector(to_unsigned(
                             c_SPI_SCK_RATE, c_SPI_SCK_RATE_LENGTH)), -- in  std_logic_vector(g_SPI_SCK_RATE_LENGTH-1 downto 0); --clock division factor
    -- SPI bus                                     
    spi_cs_on(0)          => SPI_sync_no,          -- out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0);
    spi_sck_ok            => SPI_clk_o,            -- out std_logic;
    spi_mosi_o            => SPI_mosi_o,           -- out std_logic;
    spi_mosi_oe_n         => open,                 -- out std_logic; --for tri-state outputs
    spi_miso_i            => '0'                   -- in  std_logic
  );

end architecture;