--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity fmc_hv_2ch_wb is
  generic(
    --synthesis or simuation mode
    g_SYNTHESIS                : integer  := 1;     -- 1=true / 0=false
    --main clock frequency     
    g_CLOCK_FREQ               : positive := 100E6  -- Main clk freq in Hz
  );
  port (
    --Clock and reset
    rst_ir                     : in    std_logic; 
    clk_ik                     : in    std_logic; 
    cen_ie                     : in    std_logic; 
    -- WB
    wb_cyc_i                   : in    std_logic;
    wb_stb_i                   : in    std_logic;
    wb_adr_ib32                : in    std_logic_vector(31 downto 0);
    wb_sel_ib4                 : in    std_logic_vector(3 downto 0);
    wb_we_i                    : in    std_logic;
    wb_dat_ib32                : in    std_logic_vector(31 downto 0);
    wb_dat_ob32                : out   std_logic_vector(31 downto 0);
    wb_ack_o                   : out   std_logic;
    wb_stall_o                 : out   std_logic;
    --FMC I/F
    fmc_present_i              : in    std_logic; 
    fmc_power_good_i           : in    std_logic; 
    fmc_enable_ctrl_o          : out   std_logic;
    fmc_enable_stat_i          : in    std_logic; 
    fmc_HV1_outena_o           : out   std_logic;
    fmc_HV2_outena_o           : out   std_logic;
    fmc_LV_outena_o            : out   std_logic;
    fmc_DAC1_sync_no           : out   std_logic;
    fmc_DAC1_clk_o             : out   std_logic; 
    fmc_DAC1_din_o             : out   std_logic; 
    fmc_DAC2_sync_no           : out   std_logic; 
    fmc_DAC2_clk_o             : out   std_logic; 
    fmc_DAC2_din_o             : out   std_logic; 
    fmc_ADC1_Vfb_clk_o         : out   std_logic; 
    fmc_ADC1_Vfb_cs_o          : out   std_logic;  
    fmc_ADC1_Vfb_mosi_o        : out   std_logic; 
    fmc_ADC1_Vfb_miso_i        : in    std_logic; 
    fmc_ADC2_Vfb_clk_o         : out   std_logic; 
    fmc_ADC2_Vfb_cs_o          : out   std_logic;  
    fmc_ADC2_Vfb_mosi_o        : out   std_logic; 
    fmc_ADC2_Vfb_miso_i        : in    std_logic; 
    fmc_ADC_sck_o              : out   std_logic; 
    fmc_ADC_cnv_o              : out   std_logic; 
    fmc_ADC_din_o              : out   std_logic; 
    fmc_ADC_sdo_i              : in    std_logic;
    fmc_i2c_prom_scl_iok       : inout std_logic; 
    fmc_i2c_prom_sda_io        : inout std_logic 
  );
end entity ;




------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of fmc_hv_2ch_wb is

  function get_temp_update_us return positive is
  begin
    if g_SYNTHESIS>0 then
      return 1E6; -- 1 seconds
    else
      return 10;  -- 10 us for simulation
    end if;  
  end function;

  --Slave WishBone
  signal slv_wb_cyc                 : std_logic;     
  signal slv_wb_adr_b32             : std_logic_vector(31 downto 0);
  signal slv_wb_sel_b4              : std_logic_vector(3 downto 0); 
  signal slv_wb_we                  : std_logic; 
  signal slv_wb_dat_mosi_b32        : std_logic_vector(31 downto 0); 

  -- WB to controls
  signal ctrl_wb_stb                : std_logic; 
  signal ctrl_wb_miso_b32           : std_logic_vector(31 downto 0); 
  signal ctrl_wb_ack                : std_logic; 
  signal ctrl_wb_stall              : std_logic; 

  -- WB to status
  signal status_wb_stb              : std_logic;  
  signal status_wb_miso_b32         : std_logic_vector(31 downto 0);
  signal status_wb_ack              : std_logic; 
  signal status_wb_stall            : std_logic; 
  
  -- WB to regultaor
  signal regul_wb_stb               : std_logic;  
  signal regul_wb_miso_b32          : std_logic_vector(31 downto 0);
  signal regul_wb_ack               : std_logic; 
  signal regul_wb_stall             : std_logic; 

  -- WB to EEPROM
  signal eeprom_wb_stb              : std_logic; 
  signal eeprom_wb_miso_b32         : std_logic_vector(31 downto 0);
  signal eeprom_wb_ack              : std_logic; 
  signal eeprom_wb_stall            : std_logic; 
  
  --CTRL regs
  signal reg_wr_b32                 : std_logic_vector(31 downto 0); --Write strobe for the 32 control registers
         --FMC
  signal enable_ctrl_b32            : std_logic_vector(31 downto 0); --[0]= FMC Voltage enable
  signal regulator_ena_b32          : std_logic_vector(31 downto 0); --[2:0]= [LV, HV2, HV1] voltage regulator enable
         --HV1
  signal fmc_HV1_oe                 : std_logic_vector(31 downto 0); --[0]= HV1 FMC output enable
  signal fmc_HV1_pwm_ena_b32        : std_logic_vector(31 downto 0); --[0]= HV1 PWM enable
  signal fmc_HV1_pwm_duty_cycle_b32 : std_logic_vector(31 downto 0); --[8:0]= HV1 duty cycle 256 steps
  signal fmc_HV1_pwm_clk_div_b32    : std_logic_vector(31 downto 0); --[31:0]= HV1 PWM period (main clock divided by 256*clk_div)
  signal fmc_HV1_reg_volt_cmd_b32   : std_logic_vector(31 downto 0); --[15:0] TBC ???????????????
  signal fmc_HV1_DAC_volt_cmd_b32   : std_logic_vector(31 downto 0); --[15:0]= DAC command value
         --HV2
  signal fmc_HV2_oe                 : std_logic_vector(31 downto 0); --[0]= HV2 FMC output enable
  signal fmc_HV2_pwm_ena_b32        : std_logic_vector(31 downto 0); --[0]= HV2 PWM enable
  signal fmc_HV2_pwm_duty_cycle_b32 : std_logic_vector(31 downto 0); --[8:0]= HV2 PWM duty cycle 256 steps
  signal fmc_HV2_pwm_clk_div_b32    : std_logic_vector(31 downto 0); --[31:0]= HV2 PWM period (main clock divided by 256*clk_div)
  signal fmc_HV2_reg_volt_cmd_b32   : std_logic_vector(31 downto 0); --[15:0] TBC ???????????????
  signal fmc_HV2_DAC_volt_cmd_b32   : std_logic_vector(31 downto 0); --[15:0]= DAC command value
         --LV
  signal fmc_LV_oe                  : std_logic_vector(31 downto 0); --[0]= LV FMC output enable
  signal fmc_LV_pwm_ena_b32         : std_logic_vector(31 downto 0); --[0]= LV PWM enable
  signal fmc_LV_pwm_duty_cycle_b32  : std_logic_vector(31 downto 0); --[8:0]= LV PWM duty cycle 256 steps
  signal fmc_LV_pwm_clk_div_b32     : std_logic_vector(31 downto 0); --[31:0]= LV PWM period (main clock divided by 256*clk_div)
  signal fmc_LV_reg_volt_cmd_b32    : std_logic_vector(31 downto 0); --[15:0] TBC ???????????????
  signal fmc_LV_DAC_cmd_b32         : std_logic_vector(31 downto 0); --[15:0]= DAC command value
         --TP3
  signal fmc_TP3_DAC_cmd_b32        : std_logic_vector(31 downto 0); --[15:0]= DAC command value
  
  --STATUS regs
  signal present                    : std_logic;
  signal power_good                 : std_logic;
  signal enable_stat                : std_logic; 
  signal ADC1_temp_b32              : std_logic_vector(31 downto 0);
  signal ADC2_temp_b32              : std_logic_vector(31 downto 0);
  signal ADC3_temp_b32              : std_logic_vector(31 downto 0);

  --Temperature readout
  signal timer_1us                  : std_logic;                  --pulse at 1us
  signal temperature_timer_us       : unsigned(31 downto 0);      --number of us
  constant c_TEMP_TIMER_US_INIT     : integer := get_temp_update_us; --Number of clock cycles in 10 seconds
  
  --Regulator output to arbitration
  signal HV1_regul_cmd_wr           : std_logic; 
  signal HV1_regul_cmd_b16          : std_logic_vector(15 downto 0); 
  signal HV2_regul_cmd_wr           : std_logic; 
  signal HV2_regul_cmd_b16          : std_logic_vector(15 downto 0); 
  signal LV_regul_cmd_wr            : std_logic; 
  signal LV_regul_cmd_b16           : std_logic_vector(15 downto 0); 
  
  --FMC interface 
  signal DAC1_HV1_wr                : std_logic; 
  signal DAC1_HV1_b16               : std_logic_vector(15 downto 0); 
  signal DAC1_HV1_busy              : std_logic; 
  signal DAC1_TP3_wr                : std_logic; 
  signal DAC1_TP3_b16               : std_logic_vector(15 downto 0); 
  signal DAC1_TP3_busy              : std_logic; 
  signal DAC2_HV2_wr                : std_logic; 
  signal DAC2_HV2_b16               : std_logic_vector(15 downto 0); 
  signal DAC2_HV2_busy              : std_logic; 
  signal DAC2_LV_wr                 : std_logic; 
  signal DAC2_LV_b16                : std_logic_vector(15 downto 0); 
  signal DAC2_LV_busy               : std_logic; 
  signal ADC1_ch_b25                : std_logic_vector(24 downto 0); 
  signal ADC1_ch_valid              : std_logic; 
  signal ADC1_temp_rd               : std_logic;
  signal ADC1_temp_b25              : std_logic_vector(24 downto 0); 
  signal ADC1_temp_rd_valid         : std_logic; 
  signal ADC1_temp_busy             : std_logic; 
  signal ADC2_ch_b25                : std_logic_vector(24 downto 0); 
  signal ADC2_ch_valid              : std_logic; 
  signal ADC2_temp_rd               : std_logic;
  signal ADC2_temp_b25              : std_logic_vector(24 downto 0); 
  signal ADC2_temp_rd_valid         : std_logic; 
  signal ADC2_temp_busy             : std_logic; 
  signal LV_volt_valid              : std_logic;
  signal LV_volt_b16                : std_logic_vector(15 downto 0);
  signal HV1_curr_valid             : std_logic;
  signal HV1_curr_b16               : std_logic_vector(15 downto 0); 
  signal HV2_curr_valid             : std_logic;
  signal HV2_curr_b16               : std_logic_vector(15 downto 0); 
  signal LV_curr_valid              : std_logic;
  signal LV_curr_b16                : std_logic_vector(15 downto 0); 
  signal ADC3_4ch_temp_valid        : std_logic;
  signal ADC3_4ch_temp_b16          : std_logic_vector(15 downto 0); 
  
begin 


  ----------------------
  -- FMC Addr decoder
  ----------------------
  i_wb_addr_decoder: entity work.wb_addr_decoder
  generic map(
    g_ADDR_WIDTH     => 32,                   -- integer  := 21,
    g_DATA_WIDTH     => 32                    -- integer  := 32 
  )
  port map(
    --CLK/RST
    rst_ir            => rst_ir,              -- in    std_logic; 
    clk_ik            => clk_ik,              -- in    std_logic; 
    cen_ie            => cen_ie,              -- in    std_logic; 
    --Master WishBone
    Cyc_i             => wb_cyc_i,            -- in    std_logic; 
    Stb_i             => wb_stb_i,            -- in    std_logic; 
    Adr_ib            => wb_adr_ib32,         -- in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib            => wb_sel_ib4,          -- in    std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_i              => wb_we_i,             -- in    std_logic; 
    DatMoSi_ib        => wb_dat_ib32,         -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb       => wb_dat_ob32,         -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq            => wb_ack_o,            -- out   std_logic; 
    Stall_o           => wb_stall_o,          -- out   std_logic; 
    --Slave WishBone
    Cyc_oq            => slv_wb_cyc,          -- out   std_logic;     
    Adr_oqb           => slv_wb_adr_b32,      -- out   std_logic_vector(g_ADDR_WIDTH-1 downto 0);
    Sel_oqb           => slv_wb_sel_b4,       -- out   std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_oq             => slv_wb_we,           -- out   std_logic; 
    DatMoSi_oqb       => slv_wb_dat_mosi_b32, -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --Slave ctrl
    StbCtrlReg_oq     => ctrl_wb_stb,         -- out   std_logic; 
    DatCtrlReg_ib     => ctrl_wb_miso_b32,    -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckCtrlReg_i      => ctrl_wb_ack,         -- in    std_logic; 
    StallCtrlReg_i    => ctrl_wb_stall,       -- in    std_logic; 
    --Slave status
    StbStatReg_oq     => status_wb_stb,       -- out   std_logic;
    DatStatReg_ib     => status_wb_miso_b32,  -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckStatReg_i      => status_wb_ack,       -- in    std_logic; 
    StallStatReg_i    => status_wb_stall,     -- in    std_logic;
    --Slave regulator 
    StbRegul_oq       => regul_wb_stb,        -- out   std_logic;
    DatRegul_ib       => regul_wb_miso_b32,   -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckRegul_i        => regul_wb_ack,        -- in    std_logic; 
    StallRegul_i      => regul_wb_stall,      -- in    std_logic;
    --Slave Eeprom                            
    StbEeprom_oq      => eeprom_wb_stb,       -- out   std_logic;
    DatEeprom_ib      => eeprom_wb_miso_b32,  -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckEeprom_i       => eeprom_wb_ack,       -- in    std_logic; 
    StallEeprom_i     => eeprom_wb_stall      -- in    std_logic
  );


  ----------------------
  -- FMC CTRL Registers 
  ----------------------
  i_wb_32ctrl_reg: entity work.wb_32out_reg
  generic map(
    g_ADDR_WIDTH        => 5,           -- integer  := 5; 
    g_DATA_WIDTH        => 32,          -- integer  := 32;
    g_reg0_default      => x"00000000", -- std_logic_vector;
    g_reg0_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg1_default      => x"00000000", -- std_logic_vector;
    g_reg1_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg2_default      => x"00000000", -- std_logic_vector;
    g_reg2_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg3_default      => x"00000000", -- std_logic_vector;
    g_reg3_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg4_default      => x"00000000", -- std_logic_vector;
    g_reg4_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg5_default      => x"00000000", -- std_logic_vector;
    g_reg5_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg6_default      => x"00000000", -- std_logic_vector;
    g_reg6_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg7_default      => x"00000000", -- std_logic_vector;
    g_reg7_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg8_default      => x"00000000", -- std_logic_vector;
    g_reg8_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;
    g_reg9_default      => x"00000000", -- std_logic_vector; 
    g_reg9_autoclrmask  => x"FFFFFFFF", -- std_logic_vector;    
    g_reg10_default     => x"00000000", -- std_logic_vector;
    g_reg10_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg11_default     => x"00000000", -- std_logic_vector;
    g_reg11_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg12_default     => x"00000000", -- std_logic_vector;
    g_reg12_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg13_default     => x"00000000", -- std_logic_vector;
    g_reg13_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg14_default     => x"00000000", -- std_logic_vector;
    g_reg14_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg15_default     => x"00000000", -- std_logic_vector;
    g_reg15_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg16_default     => x"00000000", -- std_logic_vector;
    g_reg16_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg17_default     => x"00000000", -- std_logic_vector;
    g_reg17_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg18_default     => x"00000000", -- std_logic_vector;
    g_reg18_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg19_default     => x"00000000", -- std_logic_vector;
    g_reg19_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg20_default     => x"00000000", -- std_logic_vector;
    g_reg20_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg21_default     => x"00000000", -- std_logic_vector;
    g_reg21_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg22_default     => x"00000000", -- std_logic_vector;
    g_reg22_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg23_default     => x"00000000", -- std_logic_vector;
    g_reg23_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg24_default     => x"00000000", -- std_logic_vector;
    g_reg24_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg25_default     => x"00000000", -- std_logic_vector;
    g_reg25_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg26_default     => x"00000000", -- std_logic_vector;
    g_reg26_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg27_default     => x"00000000", -- std_logic_vector;
    g_reg27_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg28_default     => x"00000000", -- std_logic_vector;
    g_reg28_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg29_default     => x"00000000", -- std_logic_vector;
    g_reg29_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg30_default     => x"00000000", -- std_logic_vector;
    g_reg30_autoclrmask => x"FFFFFFFF", -- std_logic_vector;
    g_reg31_default     => x"FFFFFFFF", -- std_logic_vector; --unused (all at '1')
    g_reg31_autoclrmask => x"FFFFFFFF"  -- std_logic_vector
  )
  port map(
    --CLK/RST
    rst_ir       => rst_ir,                     -- in    std_logic; 
    clk_ik       => clk_ik,                     -- in    std_logic; 
    cen_ie       => cen_ie,                     -- in    std_logic; 
    --Master WishBone                         
    Cyc_i        => slv_wb_cyc,                 -- in    std_logic; 
    Stb_i        => ctrl_wb_stb,                -- in    std_logic; 
    Adr_ib       => slv_wb_adr_b32(4 downto 0), -- in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib       => slv_wb_sel_b4,              -- in    std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_i         => slv_wb_we,                  -- in    std_logic; 
    DatMoSi_ib   => slv_wb_dat_mosi_b32,        -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb  => ctrl_wb_miso_b32,           -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq       => ctrl_wb_ack,                -- out   std_logic; 
    Stall_o      => ctrl_wb_stall,              -- out   std_logic; 
    -- update strobe
    reg_wr_ob32  => reg_wr_b32,                 -- out   std_logic_vector(31 downto 0); 
    --Registers                                 
    reg0_ob      => enable_ctrl_b32,            -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg1_ob      => regulator_ena_b32,          -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg2_ob      => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg3_ob      => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg4_ob      => fmc_HV1_oe,                 -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg5_ob      => fmc_HV1_pwm_ena_b32,        -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg6_ob      => fmc_HV1_pwm_duty_cycle_b32, -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg7_ob      => fmc_HV1_pwm_clk_div_b32,    -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg8_ob      => fmc_HV1_reg_volt_cmd_b32,   -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg9_ob      => fmc_HV1_DAC_volt_cmd_b32,   -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg10_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg11_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg12_ob     => fmc_HV2_oe,                 -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg13_ob     => fmc_HV2_pwm_ena_b32,        -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg14_ob     => fmc_HV2_pwm_duty_cycle_b32, -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg15_ob     => fmc_HV2_pwm_clk_div_b32,    -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg16_ob     => fmc_HV2_reg_volt_cmd_b32,   -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg17_ob     => fmc_HV2_DAC_volt_cmd_b32,   -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg18_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg19_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg20_ob     => fmc_LV_oe,                  -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg21_ob     => fmc_LV_pwm_ena_b32,         -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg22_ob     => fmc_LV_pwm_duty_cycle_b32,  -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg23_ob     => fmc_LV_pwm_clk_div_b32,     -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg24_ob     => fmc_LV_reg_volt_cmd_b32,    -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg25_ob     => fmc_LV_DAC_cmd_b32,         -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg26_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg27_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg28_ob     => fmc_TP3_DAC_cmd_b32,        -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg29_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg30_ob     => open,                       -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg31_ob     => open                        -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0)
  );

  ----------------------
  -- FMC STATUS Registers 
  ----------------------
  i_wb_32status_reg: entity work.wb_32in_reg
  generic map(
    g_ADDR_WIDTH           => 5,                          -- integer  := 5; 
    g_DATA_WIDTH           => 32                          -- integer  := 32 
  )                                                
  port map(                                        
    --CLK/RST                                      
    rst_ir                 => rst_ir,                     -- in    std_logic; 
    clk_ik                 => clk_ik,                     -- in    std_logic; 
    cen_ie                 => cen_ie,                     -- in    std_logic; 
    --Master WishBone                              
    Cyc_i                  => slv_wb_cyc,                 -- in    std_logic; 
    Stb_i                  => status_wb_stb,              -- in    std_logic; 
    Adr_ib                 => slv_wb_adr_b32(4 downto 0), -- in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib                 => slv_wb_sel_b4,              -- in    std_logic_vector(g_ADDR_WIDTH/8-1 downto 0); 
    We_i                   => slv_wb_we,                  -- in    std_logic; 
    DatMoSi_ib             => slv_wb_dat_mosi_b32,        -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb            => status_wb_miso_b32,         -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq                 => status_wb_ack,              -- out   std_logic; 
    Stall_o                => status_wb_stall,            -- out   std_logic; 
    --Registers
    --0x80
    reg0_ib(31 downto  1)  => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg0_ib(0)             => present,                    -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg1_ib(31 downto  1)  => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg1_ib(0)             => power_good,                 -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg2_ib(31 downto  1)  => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg2_ib(0)             => enable_stat,                -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg3_ib                => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --0x10
    reg4_ib                => ADC1_temp_b32,              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg5_ib                => ADC2_temp_b32,              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg6_ib                => ADC3_temp_b32,              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg7_ib                => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --0x20
    reg8_ib(31 downto 25)  => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg8_ib(24 downto  0)  => ADC1_ch_b25,                -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg9_ib(31 downto 16)  => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg9_ib(15 downto  0)  => HV1_curr_b16,               -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg10_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg11_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --0x30
    reg12_ib(31 downto 25) => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg12_ib(24 downto  0) => ADC2_ch_b25,                -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg13_ib(31 downto 16) => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg13_ib(15 downto  0) => HV2_curr_b16,               -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg14_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg15_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --0x40
    reg16_ib(31 downto 16) => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg16_ib(15 downto  0) => LV_volt_b16,                -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg17_ib(31 downto 16) => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg17_ib(15 downto  0) => LV_curr_b16,                -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg18_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg19_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --0x44
    reg20_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg21_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg22_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg23_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg24_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg25_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg26_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg27_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg28_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg29_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg30_ib               => (others=>'0'),              -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    reg31_ib               => x"FFFFFFFF" --unused        -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0)
  );
     
  ----------------------
  -- FMC Regulator
  ----------------------
  i_fmc_hv_2ch_regulator: entity work.fmc_hv_2ch_regulator
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS          => 0                                      -- integer   := 1; -->1=true / 0=false
  )
  port map(
    --CLK/RST
    rst_ir               => rst_ir,                                -- in    std_logic; 
    clk_ik               => clk_ik,                                -- in    std_logic; 
    cen_ie               => cen_ie,                                -- in    std_logic; 
    --enable inputs
    HV1_regulator_ena_i  => regulator_ena_b32(0),                  -- in    std_logic; 
    HV2_regulator_ena_i  => regulator_ena_b32(1),                  -- in    std_logic; 
    LV_regulator_ena_i   => regulator_ena_b32(2),                  -- in    std_logic; 
    --Voltage commands inputs
    HV1_volt_cmd_wr_i    => reg_wr_b32(8),                         -- in    std_logic; 
    HV1_volt_cmd_ib16    => fmc_HV1_reg_volt_cmd_b32(15 downto 0), -- in    std_logic_vector(15 downto 0); 
    HV2_volt_cmd_wr_i    => reg_wr_b32(16),                        -- in    std_logic; 
    HV2_volt_cmd_ib16    => fmc_HV2_reg_volt_cmd_b32(15 downto 0), -- in    std_logic_vector(15 downto 0); 
    LV_volt_cmd_wr_i     => reg_wr_b32(24),                        -- in    std_logic; 
    LV_volt_cmd_ib16     => fmc_LV_reg_volt_cmd_b32(15 downto 0),  -- in    std_logic_vector(15 downto 0); 
    --Master WishBone (config)                         
    Cyc_i                => slv_wb_cyc,                            -- in    std_logic; 
    Stb_i                => regul_wb_stb,                          -- in    std_logic; 
    Adr_ib               => slv_wb_adr_b32,                        -- in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib               => slv_wb_sel_b4,                         -- in    std_logic_vector(g_ADDR_WIDTH/8-1 downto 0); 
    We_i                 => slv_wb_we,                             -- in    std_logic; 
    DatMoSi_ib           => slv_wb_dat_mosi_b32,                   -- in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb          => regul_wb_miso_b32,                     -- out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq               => regul_wb_ack,                          -- out   std_logic; 
    Stall_o              => regul_wb_stall,                        -- out   std_logic; 
    --HV1 
    HV1_DAC_wr_o         => HV1_regul_cmd_wr,                      -- out   std_logic; 
    HV1_DAC_ob16         => HV1_regul_cmd_b16,                     -- out   std_logic_vector(15 downto 0); 
    HV1_ADC_volt_valid_i => ADC1_ch_valid,                         -- in    std_logic;
    HV1_ADC_volt_ib25    => ADC1_ch_b25,                           -- in    std_logic_vector(24 downto 0); 
    HV1_ADC_curr_valid_i => HV1_curr_valid,                        -- in    std_logic;
    HV1_ADC_curr_ib16    => HV1_curr_b16,                          -- in    std_logic_vector(15 downto 0); 
    --HV2
    HV2_DAC_wr_o         => HV2_regul_cmd_wr,                      -- out   std_logic; 
    HV2_DAC_ob16         => HV2_regul_cmd_b16,                     -- out   std_logic_vector(15 downto 0); 
    HV2_ADC_volt_valid_i => ADC2_ch_valid,                         -- in    std_logic;
    HV2_ADC_volt_ib25    => ADC2_ch_b25,                           -- in    std_logic_vector(24 downto 0); 
    HV2_ADC_curr_valid_i => HV2_curr_valid,                        -- in    std_logic;
    HV2_ADC_curr_ib16    => HV2_curr_b16,                          -- in    std_logic_vector(15 downto 0); 
    --LV
    LV_DAC_wr_o          => LV_regul_cmd_wr,                       -- out   std_logic; 
    LV_DAC_ob16          => LV_regul_cmd_b16,                      -- out   std_logic_vector(15 downto 0); 
    LV_ADC_volt_valid_i  => LV_volt_valid,                         -- in    std_logic;
    LV_ADC_volt_ib16     => LV_volt_b16,                           -- in    std_logic_vector(15 downto 0); 
    LV_ADC_curr_valid_i  => LV_curr_valid,                         -- in    std_logic;
    LV_ADC_curr_ib16     => LV_curr_b16                            -- in    std_logic_vector(15 downto 0)
  );

  ------------------------------------
  -- Timer 1us (100MHz=>1MHz)
  ------------------------------------
  i_timer_1us : entity work.timer
  generic map(
    g_IN_EN_PULSE_FREQ => g_CLOCK_FREQ,      -- integer -- Enable Input Frequency
    g_TIMER_PULSE_FREQ => 1000000            -- integer -- Pulse Frequency
  )
  port map(
    clk_ik              => clk_ik,           -- in  std_logic; -- input Clock at G_IN_CLOCK_FREQ
    rst_ir              => rst_ir,           -- in  std_logic; -- input Reset
    cen_ie              => cen_ie,           -- in  std_logic; -- input clock enable
    clr_timer_i         => '0',              -- in  std_logic; -- input clear
    timer_en_ie         => '1',              -- in  std_logic; -- timer enable (can be an higher rate timer)
    timer_pulse_o       => timer_1us         -- out std_logic  -- Pulse generated at frequency G_TIMER_PULSE_FREQ
  );
  
  -----------------
  -- Temperature ADC1 & ADC2 periodic Readout 
  -----------------
  p_ADC_temp_readout: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
     temperature_timer_us <= to_unsigned(c_TEMP_TIMER_US_INIT,32);
     ADC1_temp_rd  <= '0';
     ADC2_temp_rd  <= '0';
     ADC1_temp_b32 <= (others=>'0');
     ADC2_temp_b32 <= (others=>'0');
     ADC3_temp_b32 <= (others=>'0');

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        
        --disable
        if enable_stat = '0' then
          temperature_timer_us <= (others=>'0');
          ADC1_temp_rd         <= '0';
          ADC2_temp_rd         <= '0';
        --enable
        else
          --ADC1 & ADC2 read request
          if temperature_timer_us = 0 then
            ADC1_temp_rd <= '1';
            ADC2_temp_rd <= '1';
            temperature_timer_us <= to_unsigned(c_TEMP_TIMER_US_INIT,32);
          else
            --ADC1 read
            if ADC1_temp_busy='0' and ADC1_temp_rd='1' then
              ADC1_temp_rd <= '0';
            end if;
            if ADC1_temp_rd_valid='1' then 
              ADC1_temp_b32 <= "0000000" & ADC1_temp_b25;
            end if;
            --ADC2 read
            if ADC2_temp_busy='0' and ADC2_temp_rd='1' then
              ADC2_temp_rd <= '0';
            end if;
            if ADC2_temp_rd_valid='1' then 
              ADC2_temp_b32 <= "0000000" & ADC2_temp_b25;
            end if;
            --Timer when both read done
            if ADC1_temp_rd='0' and ADC2_temp_rd='0' and timer_1us='1' then 
              temperature_timer_us <= temperature_timer_us-1;
            end if;
          end if;
          
          --ADC3 latch
          if ADC3_4ch_temp_valid='1' then 
            ADC3_temp_b32 <= x"0000" & ADC3_4ch_temp_b16;
          end if;
        end if;
      end if;
    end if;
  end process;
  
  -----------------
  -- Register vs Regulator arbiter
  -----------------
  p_dac_arbiter: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      DAC1_HV1_wr  <= '0';
      DAC1_HV1_b16 <= (others=>'0');
      DAC2_HV2_wr  <= '0';
      DAC2_HV2_b16 <= (others=>'0');
      DAC2_LV_wr   <= '0';
      DAC2_LV_b16  <= (others=>'0');
      DAC1_TP3_wr  <= '0';
      DAC1_TP3_b16 <= (others=>'0');

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        
        -- HV1 regulator enabled
        if regulator_ena_b32(0) = '1' then
          if DAC1_HV1_wr='1' and DAC1_HV1_busy='0' then
            DAC1_HV1_wr <= '0';
          elsif HV1_regul_cmd_wr='1' then 
            DAC1_HV1_wr <= '1';
          end if;
          DAC1_HV1_b16 <= HV1_regul_cmd_b16;
        -- HV1 commanded by registers
        else
          if DAC1_HV1_wr='1' and DAC1_HV1_busy='0' then
            DAC1_HV1_wr <= '0';
          elsif reg_wr_b32(9)='1' then 
            DAC1_HV1_wr <= '1';
          end if;
          DAC1_HV1_b16 <= fmc_HV1_DAC_volt_cmd_b32(15 downto 0);
        end if;
        
        -- HV2 regulator enabled
        if regulator_ena_b32(1) = '1' then
          if DAC2_HV2_wr='1' and DAC2_HV2_busy='0' then
            DAC2_HV2_wr <= '0';
          elsif HV2_regul_cmd_wr='1' then 
            DAC2_HV2_wr <= '1';
          end if;
          DAC2_HV2_b16 <= HV2_regul_cmd_b16;
        -- HV1 commanded by registers
        else
          if DAC2_HV2_wr='1' and DAC2_HV2_busy='0' then
            DAC2_HV2_wr <= '0';
          elsif reg_wr_b32(17)='1' then 
            DAC2_HV2_wr <= '1';
          end if;
          DAC2_HV2_b16 <= fmc_HV2_DAC_volt_cmd_b32(15 downto 0);
        end if;
        
        -- LV regulator enabled
        if regulator_ena_b32(2) = '1' then
          if DAC2_LV_wr='1' and DAC2_LV_busy='0' then
            DAC2_LV_wr <= '0';
          elsif LV_regul_cmd_wr='1' then 
            DAC2_LV_wr <= '1';
          end if;
          DAC2_LV_b16 <= LV_regul_cmd_b16;
        -- LV commanded by registers
        else
          if DAC2_LV_wr='1' and DAC2_LV_busy='0' then
            DAC2_LV_wr <= '0';
          elsif reg_wr_b32(25)='1' then 
            DAC2_LV_wr <= '1';
          end if;
          DAC2_LV_b16 <= fmc_LV_DAC_cmd_b32(15 downto 0);
        end if;
        
        --Test point 3 (no regulator)
        if DAC1_TP3_wr='1' and DAC1_TP3_busy='0' then
          DAC1_TP3_wr <= '0';
        elsif reg_wr_b32(28)='1' then 
          DAC1_TP3_wr <= '1';
        end if;
        DAC1_TP3_b16 <= fmc_TP3_DAC_cmd_b32(15 downto 0);
        
      end if;
    end if;
  end process;
  
  
  ----------------------
  -- FMC Interface
  ----------------------
  i_fmc_hv_2ch_if: entity work.fmc_hv_2ch_if
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS                => g_SYNTHESIS,                            -- integer  := 1; -->1=true / 0=false
    --main clock frequency                                                
    g_CLOCK_FREQ               => g_CLOCK_FREQ                            -- positive := 100000000 --Hz
  )                                                                       
  port map(                                                               
    --CLK/RST                                                             
    rst_ir                     => rst_ir,                                 -- in    std_logic; 
    clk_ik                     => clk_ik,                                 -- in    std_logic; 
    cen_ie                     => cen_ie,                                 -- in    std_logic; 
    --Internal CTRL/STATUS                                                         
    present_o                  => present,                                -- out   std_logic; 
    enable_ctrl_i              => enable_ctrl_b32(0),                     -- in    std_logic; 
    enable_stat_o              => enable_stat,                            -- out   std_logic; 
    power_good_o               => power_good,                             -- out   std_logic;
    --PWM HV1
    fmc_HV1_pwm_ena_i          => fmc_HV1_pwm_ena_b32(0),                 -- in    std_logic; 
    fmc_HV1_pwm_duty_cycle_ib8 => fmc_HV1_pwm_duty_cycle_b32(7 downto 0), -- in    std_logic_vector(7 downto 0);
    fmc_HV1_pwm_clk_div_ib32   => fmc_HV1_pwm_clk_div_b32,                -- in    std_logic_vector(31 downto 0);
    fmc_HV1_oe_i               => fmc_HV1_oe(0),                          -- in    std_logic; 
    --PWM HV2
    fmc_HV2_pwm_ena_i          => fmc_HV2_pwm_ena_b32(0),                 -- in    std_logic; 
    fmc_HV2_pwm_duty_cycle_ib8 => fmc_HV2_pwm_duty_cycle_b32(7 downto 0), -- in    std_logic_vector(7 downto 0);
    fmc_HV2_pwm_clk_div_ib32   => fmc_HV2_pwm_clk_div_b32,                -- in    std_logic_vector(31 downto 0);
    fmc_HV2_oe_i               => fmc_HV2_oe(0),                          -- in    std_logic; 
    --PWM HV2
    fmc_LV_pwm_ena_i           => fmc_LV_pwm_ena_b32(0),                  -- in    std_logic; 
    fmc_LV_pwm_duty_cycle_ib8  => fmc_LV_pwm_duty_cycle_b32(7 downto 0),  -- in    std_logic_vector(7 downto 0);   
    fmc_LV_pwm_clk_div_ib32    => fmc_LV_pwm_clk_div_b32,                 -- in    std_logic_vector(31 downto 0);    
    fmc_LV_oe_i                => fmc_LV_oe(0),                           -- in    std_logic; 
    --DAC1
    DAC1_HV1_wr_i              => DAC1_HV1_wr,                            -- in    std_logic; 
    DAC1_HV1_ib16              => DAC1_HV1_b16,                           -- in    std_logic_vector(15 downto 0); 
    DAC1_HV1_busy_o            => DAC1_HV1_busy,                          -- out   std_logic; 
    DAC1_TP3_wr_i              => DAC1_TP3_wr,                            -- in    std_logic; 
    DAC1_TP3_ib16              => DAC1_TP3_b16,                           -- in    std_logic_vector(15 downto 0); 
    DAC1_TP3_busy_o            => DAC1_TP3_busy,                          -- out   std_logic; 
    --DAC2                                                                
    DAC2_HV2_wr_i              => DAC2_HV2_wr,                            -- in    std_logic; 
    DAC2_HV2_ib16              => DAC2_HV2_b16,                           -- in    std_logic_vector(15 downto 0); 
    DAC2_HV2_busy_o            => DAC2_HV2_busy,                          -- out   std_logic; 
    DAC2_LV_wr_i               => DAC2_LV_wr,                             -- in    std_logic; 
    DAC2_LV_ib16               => DAC2_LV_b16,                            -- in    std_logic_vector(15 downto 0); 
    DAC2_LV_busy_o             => DAC2_LV_busy,                           -- out   std_logic; 
    --ADC1                                                                
    ADC1_ch_ob25               => ADC1_ch_b25,                            -- out   std_logic_vector(24 downto 0); 
    ADC1_ch_valid_o            => ADC1_ch_valid,                          -- out   std_logic; 
    ADC1_temp_rd_i             => ADC1_temp_rd,                           -- in    std_logic;
    ADC1_temp_ob25             => ADC1_temp_b25,                          -- out   std_logic_vector(24 downto 0); 
    ADC1_temp_rd_valid_o       => ADC1_temp_rd_valid,                     -- out   std_logic; 
    ADC1_temp_busy_o           => ADC1_temp_busy,                         -- out   std_logic; 
    --ADC2                                                                
    ADC2_ch_ob25               => ADC2_ch_b25,                            -- out   std_logic_vector(24 downto 0); 
    ADC2_ch_valid_o            => ADC2_ch_valid,                          -- out   std_logic; 
    ADC2_temp_rd_i             => ADC2_temp_rd,                           -- in    std_logic;
    ADC2_temp_ob25             => ADC2_temp_b25,                          -- out   std_logic_vector(24 downto 0); 
    ADC2_temp_rd_valid_o       => ADC2_temp_rd_valid,                     -- out   std_logic; 
    ADC2_temp_busy_o           => ADC2_temp_busy,                         -- out   std_logic; 
    --ADC3                                                                
    ADC3_LV_volt_valid_o       => LV_volt_valid,                          -- out   std_logic;
    ADC3_LV_volt_ob16          => LV_volt_b16,                            -- out   std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o      => HV1_curr_valid,                         -- out   std_logic;
    ADC3_HV1_curr_ob16         => HV1_curr_b16,                           -- out   std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o      => HV2_curr_valid,                         -- out   std_logic;
    ADC3_HV2_curr_ob16         => HV2_curr_b16,                           -- out   std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o       => LV_curr_valid,                          -- out   std_logic;
    ADC3_LV_curr_ob16          => LV_curr_b16,                            -- out   std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o      => ADC3_4ch_temp_valid,                    -- out   std_logic;
    ADC3_4ch_temp_ob16         => ADC3_4ch_temp_b16,                      -- out   std_logic_vector(15 downto 0); 
    -- EEPROM WB                                                          
    eeprom_wb_cyc_i            => slv_wb_cyc,                             -- in    std_logic;
    eeprom_wb_stb_i            => eeprom_wb_stb,                          -- in    std_logic;
    eeprom_wb_adr_ib2          => slv_wb_adr_b32(1 downto 0),             -- in    std_logic_vector(1 downto 0);
    eeprom_wb_sel_ib4          => slv_wb_sel_b4,                          -- in    std_logic_vector(1 downto 0);
    eeprom_wb_we_i             => slv_wb_we,                              -- in    std_logic;
    eeprom_wb_dat_mosi_ib32    => slv_wb_dat_mosi_b32,                    -- in    std_logic_vector(31 downto 0);
    eeprom_wb_dat_miso_ob32    => eeprom_wb_miso_b32,                     -- out   std_logic_vector(31 downto 0);
    eeprom_wb_ack_o            => eeprom_wb_ack,                          -- out   std_logic;
    eeprom_wb_stall_o          => eeprom_wb_stall,                        -- out   std_logic;
    ----------------------
    --FMC I/F                                                                
    fmc_present_i              => fmc_present_i,                          -- in    std_logic; 
    fmc_power_good_i           => fmc_power_good_i,                       -- in    std_logic;
    fmc_enable_ctrl_o          => fmc_enable_ctrl_o,                      -- out   std_logic;
    fmc_enable_stat_i          => fmc_enable_stat_i,                      -- in    std_logic; 
    --    
    fmc_HV1_outena_o           => fmc_HV1_outena_o,                       -- out   std_logic;  --tri-state output buffer
    fmc_HV2_outena_o           => fmc_HV2_outena_o,                       -- out   std_logic;  --tri-state output buffer
    fmc_LV_outena_o            => fmc_LV_outena_o,                        -- out   std_logic;  --tri-state output buffer
    --                                                                    
    fmc_DAC1_sync_no           => fmc_DAC1_sync_no,                       -- out   std_logic;
    fmc_DAC1_clk_o             => fmc_DAC1_clk_o,                         -- out   std_logic; 
    fmc_DAC1_din_o             => fmc_DAC1_din_o,                         -- out   std_logic; 
    --                                                                    
    fmc_DAC2_sync_no           => fmc_DAC2_sync_no,                       -- out   std_logic; 
    fmc_DAC2_clk_o             => fmc_DAC2_clk_o,                         -- out   std_logic; 
    fmc_DAC2_din_o             => fmc_DAC2_din_o,                         -- out   std_logic; 
    --                                                                    
    fmc_ADC1_Vfb_clk_o         => fmc_ADC1_Vfb_clk_o,                     -- out   std_logic; 
    fmc_ADC1_Vfb_cs_o          => fmc_ADC1_Vfb_cs_o,                      -- out   std_logic;  
    fmc_ADC1_Vfb_mosi_o        => fmc_ADC1_Vfb_mosi_o,                    -- out   std_logic; 
    fmc_ADC1_Vfb_miso_i        => fmc_ADC1_Vfb_miso_i,                    -- in    std_logic; 
    --                                                                    
    fmc_ADC2_Vfb_clk_o         => fmc_ADC2_Vfb_clk_o,                     -- out   std_logic; 
    fmc_ADC2_Vfb_cs_o          => fmc_ADC2_Vfb_cs_o,                      -- out   std_logic;  
    fmc_ADC2_Vfb_mosi_o        => fmc_ADC2_Vfb_mosi_o,                    -- out   std_logic; 
    fmc_ADC2_Vfb_miso_i        => fmc_ADC2_Vfb_miso_i,                    -- in    std_logic; 
    --                                                                    
    fmc_ADC_sck_o              => fmc_ADC_sck_o,                          -- out   std_logic; 
    fmc_ADC_cnv_o              => fmc_ADC_cnv_o,                          -- out   std_logic; 
    fmc_ADC_din_o              => fmc_ADC_din_o,                          -- out   std_logic; 
    fmc_ADC_sdo_i              => fmc_ADC_sdo_i,                          -- in    std_logic;
    fmc_i2c_prom_scl_iok       => fmc_i2c_prom_scl_iok,                   -- inout std_logic; 
    fmc_i2c_prom_sda_io        => fmc_i2c_prom_sda_io                     -- inout std_logic 
  );
  
  
end architecture;