--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN (BE-BI-BL)
--
-- Language: VHDL 2008
--
-- Dependencies:
--
--     none
--
-- Description:
--
--
--============================================================================================

-----------------------------------------------------------------------
-- Libraries definition
-----------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


-----------------------------------------------------------------------
-- Entity
-----------------------------------------------------------------------
entity i2c_master_wb is
  generic(
    -- synthesis or simuation mode
    g_SYNTHESIS : integer := 1;
    -- length of the I2C cycle
    g_CYCLE_LEN : integer := 1000
  );
  port(
    --clock and reset
    rst_ir      : in    std_logic; --
    clk_ik      : in    std_logic; --
    cen_ie      : in    std_logic; --
    --WB
    wb_cyc_i    : in    std_logic; --
    wb_stb_i    : in    std_logic; --
    wb_we_i     : in    std_logic; --
    wb_adr_ib2  : in    std_logic_vector(1 downto 0); --
    wb_sel_ib4  : in    std_logic_vector(3 downto 0); --
    wb_dat_ib32 : in    std_logic_vector(31 downto 0); --
    wb_dat_ob32 : out   std_logic_vector(31 downto 0); --
    wb_ack_o    : out   std_logic; --
    wb_stall_o  : out   std_logic; --
    --I2C 
    i2c_scl_io  : inout std_logic; --
    i2c_sda_io  : inout std_logic  --
  );
end entity;

-----------------------------------------------------------------------
-- Architecture
-----------------------------------------------------------------------
architecture rtl of i2c_master_wb is
  
  function get_cycle_length return integer is
  begin
    if g_SYNTHESIS>0 then
      return g_CYCLE_LEN; -- from generic
    else
      return 5;           -- to speed-up the simulation (SCL period = 10 cycles)
    end if;  
  end function;
  
  constant c_CYCLE_LEN    : integer := get_cycle_length;
  
  signal bit_cnt_u4       : unsigned(3 downto 0);
  signal cycle_cnt_u10    : unsigned(9 downto 0);
  signal StatusReg_b32    : std_logic_vector(31 downto 0);
  signal CommandReg_qb32  : std_logic_vector(31 downto 0);
  signal a_Command_b8     : std_logic_vector(7 downto 0);
  
  signal in_scl           : std_logic;
  signal in_sda           : std_logic;
  
  signal SclOe_e          : std_logic;
  signal SdaOe_e          : std_logic;

  signal ShReg_b8         : std_logic_vector(7 downto 0);
  
  signal I2CAck           : std_logic;
  
  signal wb_ack           : std_logic;
  signal wb_stall         : std_logic;

  constant s_Idle         : std_logic_vector(3 downto 0) := x"0";
  constant s_StartSda1    : std_logic_vector(3 downto 0) := x"1";
  constant s_StartScl1    : std_logic_vector(3 downto 0) := x"2";
  constant s_StartSda0    : std_logic_vector(3 downto 0) := x"3";
  constant s_StartScl0    : std_logic_vector(3 downto 0) := x"4";
  constant s_SendScl0     : std_logic_vector(3 downto 0) := x"5";
  constant s_SendScl1     : std_logic_vector(3 downto 0) := x"6";
  constant s_GetScl0      : std_logic_vector(3 downto 0) := x"7";
  constant s_GetScl1      : std_logic_vector(3 downto 0) := x"8";
  constant s_StopSda0     : std_logic_vector(3 downto 0) := x"9";
  constant s_StopScl1     : std_logic_vector(3 downto 0) := x"a";
  constant s_StopSda1     : std_logic_vector(3 downto 0) := x"b";
  
  signal State_qb4        : std_logic_vector(3 downto 0);
  signal NextState_ab4    : std_logic_vector(3 downto 0);  

  constant c_SendStartBit : std_logic_vector(7 downto 0) := x"01";
  constant c_SendByte     : std_logic_vector(7 downto 0) := x"02";
  constant c_GetByte      : std_logic_vector(7 downto 0) := x"03";
  constant c_SendStopBit  : std_logic_vector(7 downto 0) := x"04";


begin
  
  --input I2C (tristate buffer)
  in_scl <= '0' when i2c_scl_io='0' else '1'; 
  in_sda <= '0' when i2c_sda_io='0' else '1'; 

  --Command
  a_Command_b8 <= CommandReg_qb32(31 downto 24); 
  
  --FSM sequential
  p_fsm_seq: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      State_qb4 <= s_Idle;

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        State_qb4 <= NextState_ab4;
      end if;
    end if;
  end process;
  
  --FSM combinatorial
  p_fsm_comb: process(State_qb4, a_Command_b8, cycle_cnt_u10, bit_cnt_u4)
  begin
      --default
      NextState_ab4 <= State_qb4;
      
      case State_qb4 is 
        when s_Idle => 
          if    a_Command_b8 = c_SendStartBit then
            NextState_ab4 <= s_StartSda1;
          elsif a_Command_b8 = c_SendByte then
            NextState_ab4 <= s_SendScl0;
          elsif a_Command_b8 = c_GetByte then
            NextState_ab4 <= s_GetScl0;
          elsif a_Command_b8 = c_SendStopBit then
            NextState_ab4 <= s_StopSda0;
          end if;
          
        when s_StartSda1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_StartScl1;
          end if;
          
        when s_StartScl1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_StartSda0;
          end if; 
          
        when s_StartSda0 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_StartScl0;
          end if; 
          
        when s_StartScl0 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_Idle;
          end if; 
          
        when s_SendScl0 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            if bit_cnt_u4 = 9 then 
              NextState_ab4 <= s_Idle;
            else
              NextState_ab4 <= s_SendScl1;
            end if; 
          end if; 
          
        when s_SendScl1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_SendScl0;
          end if; 
          
        when s_GetScl0 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            if bit_cnt_u4 = 9 then 
              NextState_ab4 <= s_Idle;
            else
              NextState_ab4 <= s_GetScl1;
            end if; 
          end if; 
          
        when s_GetScl1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_GetScl0;
          end if; 
          
        when s_StopSda0 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_StopScl1;
          end if;   
        
        when s_StopScl1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_StopSda1;
          end if;   
          
        when s_StopSda1 => 
          if cycle_cnt_u10 = c_CYCLE_LEN-1 then
            NextState_ab4 <= s_Idle;
          end if;   
        
        when others =>
          NextState_ab4 <= s_Idle;
      
      end case ;
  end process;


  --I2C proptocol
  p_i2c_protocol: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      SclOe_e       <= '0';
      SdaOe_e       <= '0';        
      bit_cnt_u4    <= (others=>'0');
      cycle_cnt_u10 <= (others=>'0');
      I2CAck        <= '0';
      ShReg_b8      <= (others=>'0');

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then

        case State_qb4 is 
        
          when s_Idle => 
            bit_cnt_u4    <= (others=>'0');
            cycle_cnt_u10 <= (others=>'0');
            if NextState_ab4 = s_SendScl0 then
              ShReg_b8    <= CommandReg_qb32(7 downto 0);
            end if;              
            if NextState_ab4 = s_GetScl0 then
              I2CAck      <= CommandReg_qb32(0);
            end if; 
            
          when s_StartSda1 => 
            SdaOe_e         <= '0'; 
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_sda = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            
          when s_StartScl1 => 
            SclOe_e         <= '0'; 
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_scl = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            
          when s_StartSda0 => 
            SdaOe_e         <= '1'; 
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_sda = '0' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 

          when s_StartScl0 => 
            SclOe_e        <= '1'; 
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_scl = '0' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if;   

         when s_SendScl0 => 
            SclOe_e         <= '1'; 
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_scl = '0' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            if cycle_cnt_u10(8 downto 0) = (c_CYCLE_LEN/2)-1 then 
              if bit_cnt_u4(3)='1' then            
                SdaOe_e <= '0'; 
              else
                SdaOe_e <= NOT(ShReg_b8(7)); 
              end if; 
            end if; 
            
          when s_SendScl1 => 
            SclOe_e         <= '0'; 
            if NextState_ab4 /= State_qb4 then
              bit_cnt_u4    <= bit_cnt_u4 + 1;
              cycle_cnt_u10 <= (others=>'0');
              ShReg_b8      <= ShReg_b8(6 downto 0) & '0';
              I2CAck        <= in_sda; 
            elsif in_scl = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
           
          when s_GetScl0 => 
            SclOe_e          <= '1'; 
            if bit_cnt_u4 = 8 then
              SdaOe_e <= NOT(I2CAck);
            else
              SdaOe_e <= '0';
            end if;
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_scl = '0' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if;  
            
          when s_GetScl1 => 
            SclOe_e         <= '0';
            if NextState_ab4 /= State_qb4 then
              bit_cnt_u4    <= bit_cnt_u4 +1;
              cycle_cnt_u10 <= (others=>'0');
              if bit_cnt_u4 = 8 then
                ShReg_b8 <= ShReg_b8;
              else
                ShReg_b8 <= ShReg_b8(6 downto 0) & in_sda;
              end if;
            elsif in_scl = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            
          when s_StopSda0 => 
            SdaOe_e         <= '1';
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_sda = '0' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if;   
            
          when s_StopScl1 => 
            SclOe_e         <= '0';
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_scl = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            
          when s_StopSda1 => 
            SdaOe_e         <= '0';             
            if NextState_ab4 /= State_qb4 then
              cycle_cnt_u10 <= (others=>'0');
            elsif in_sda = '1' then
              cycle_cnt_u10 <= cycle_cnt_u10 + 1;
            else 
              cycle_cnt_u10 <= (others=>'0');
            end if; 
            
          when others =>
            SclOe_e       <= '0';
            SdaOe_e       <= '0';
            bit_cnt_u4    <= (others=>'0');
            cycle_cnt_u10 <= (others=>'0');
            I2CAck        <= CommandReg_qb32(0);  
            ShReg_b8      <= CommandReg_qb32(7 downto 0); 
        end case ;
      end if;
    end if;
  end process;

  
  --Registers
  p_registers: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      CommandReg_qb32 <= (others=>'0');
      wb_dat_ob32     <= (others=>'0');
      wb_ack          <= '0';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        CommandReg_qb32(31) <= '1';             --clear command MSB by default
        wb_ack <= '0';
        --WB access
        if wb_cyc_i='1' AND wb_stb_i='1' AND wb_stall='0' then
          wb_ack <= '1';
          --write access
          if wb_we_i='1' then  
            if wb_adr_ib2="01" then             --Command reg = address 1
              CommandReg_qb32 <= wb_dat_ib32;
            end if;
          --read access
          else                                  
            if    wb_adr_ib2="00" then          --Status reg = address 0
              wb_dat_ob32 <= StatusReg_b32;
            elsif wb_adr_ib2="01" then          --Command reg = address 1
              wb_dat_ob32 <= CommandReg_qb32;
            else
              wb_dat_ob32 <= x"DEAD_BEEF";
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;
  
  --Status reg 
  StatusReg_b32 <= State_qb4 & x"0"             --IDLE when "0000" (can accept the next command)
                 & "00" & in_scl & NOT(SclOe_e) --input 0or1 / output 1=Z
                 & "00" & in_sda & NOT(SdaOe_e) --input 0or1 / output 1=Z
                 & x"0" & "000" & I2CAck        --0=ack driven to 0 / 1= no ACK (tristate)
                 & ShReg_b8;                    --inout shift register
  
  --Stall output
  wb_stall   <= '0' when State_qb4=s_Idle else '1';
  wb_stall_o <= wb_stall;
  
  --Ack ouput
  wb_ack_o   <= wb_ack AND wb_cyc_i AND wb_stb_i;
  
  -- tri-state outputs
  i2c_scl_io <= '0' when SclOe_e='1' else 'Z'; --open drain: requires pull-up
  i2c_sda_io <= '0' when SdaOe_e='1' else 'Z'; --open drain: requires pull-up

end architecture;  
