--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--     
--     Pulse Width modultation:  
--
--           Not Enable    0  = pwm_o _______________  always '1'
--
--           Duty cycle:   0% = pwm_o ___|_____|_____  one pulse out of 256 cycles
--                                        _     _   
--                        25% = pwm_o ___| |___| |___ 
--                                      _   _   _   _
--                        50% = pwm_o _| |_| |_| |_|  
--                                      ___   ___   _
--                        75% = pwm_o _|   |_|   |_|  
--                                    ________________
--                       100% = pwm_o                   always '1'
--
--           Clock counter:  use to define a frequency 
--                           using a clock enable counter (with regard to the main clk frequency)
--
--                   PWM_Hz = clk_Hz / [(2^PWN_LEN-1) * CLK_CNT_LEN)
--
--           Typical examples for 8b PWM @100MHz:
--
--                   clk_div cnt  | period   | frequency
--                 ---------------------------------------
--                   10000        |   25ms   |    39.2Hz   = 100MHz / [(2^8-1) * 1e4 ) 
--                   3921         |   10ms   |    100Hz    = 100MHz / [(2^8-1) * 3921)
--                                |          |             
--                   1000         |  2.5ms   |    390Hz    = 100MHz / [(2^8-1) * 1e3 )
--                   392          |    1ms   |     1kHz    = 100MHz / [(2^8-1) * 392 )
--                                |          |             
--                   100          |  255us   |   3.9kHz    = 100MHz / [(2^8-1) * 100 ) 
--                   39           |  100us   |    10kHz    = 100MHz / [(2^8-1) * 39  )
--                                |          |             
--                   10           |   25us   |  39.2kHz    = 100MHz / [(2^8-1) * 10  ) 
--                   4            |   10us   |  ~100kHz    = 100MHz / [(2^8-1) * 4   ) 
--                                |          |             
--                   0/1          |  2.5us   | 392.2kHz    = 100MHz / [(2^8-1) * 1   ) 
--
--
--
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity pwm is
  generic(
    g_PWM_LEN         : positive :=8; --gives the resolution (8b => 255 possible ratio)
    g_CLK_DIV_CNT_LEN : positive :=1  --gives the cycle period (frequency)
  );
  port (
    --clock and reset
    rst_ir        : in  std_logic; 
    clk_ik        : in  std_logic; 
    cen_ie        : in  std_logic; 
    --input        
    enable_i      : in  std_logic;                              -- duty cycle 0%
    duty_cycle_ib : in  std_logic_vector(g_PWM_LEN-1 downto 0); -- 0= 0.4% 1/256 cycles   1= 100% 256/256 cycles
    clk_div_ib    : in  std_logic_vector(g_CLK_DIV_CNT_LEN-1 downto 0); 
    --output        
    pwm_o         : out std_logic
  );
end entity ;




------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of pwm is

  constant zeros   : std_logic_vector(g_CLK_DIV_CNT_LEN-1 downto 0) := (others=>'0');
  
  signal pwm_cnt_u : unsigned(g_PWM_LEN-1 downto 0);
  signal clk_div_u : unsigned(g_CLK_DIV_CNT_LEN-1 downto 0);

begin 

  ----------------------
  --Division of the main clock to get the proper frequency
  ----------------------
  p_clk_div: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      clk_div_u <= (others=>'0');
      
    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
      
        --clock div = 0 (clock division)
        if clk_div_ib = zeros then
          clk_div_u <= (others=>'0');
          
        else 
          if clk_div_u >= unsigned(clk_div_ib)-1 then
            clk_div_u <= (others=>'0');
          else
            clk_div_u <= clk_div_u + 1;
          end if;
        end if;
      end if;
    end if;
  end process;
  
  ----------------------
  --PWM counter
  ----------------------
  p_pwm : process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      pwm_cnt_u <= (others => '0');
      pwm_o     <= '0';
      
    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
      
        --PWM disable
        if enable_i='0' then
          pwm_o <= '0';
        
        --PWM enable
        else
        
          --div clock enable
          if clk_div_u=0 then
          
            --main counter
            if pwm_cnt_u >= (2**g_PWM_LEN)-1 then
              pwm_cnt_u <= (others => '0');
            else
              pwm_cnt_u <= pwm_cnt_u + 1;
            end if;
            
            --duty cycle
            if pwm_cnt_u <= unsigned(duty_cycle_ib) then
              pwm_o <= '1';
            else
              pwm_o <= '0';
            end if;
            
          end if;
        end if;
      end if;
    end if;
  end process;


end architecture;