--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
--     spi master core (physical layer)
--     - WR:  sends a WR command and data on the SPI to the slaves specified in the address (several destinations possible)
--     - RD:  send a RD command and replies the content of the received SPI shift register from the slave specified in address (one-hot, one source slave)
--
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity spi_phy_master is
  generic(
    -- Local slave bus
    g_CMB_ADDR_LEN_SS_NUM : positive := 1; -- slave selection (hot bit)
    g_CMB_DATA_LEN        : positive := 8; -- SPI frame length in bits
    -- SPI bus
    g_SPI_SCK_RATE_LENGTH : positive := 8  -- bits for clock period division counter
  );
  port(
    -- Clock and reset
    rst_ir                : in  std_logic;
    clk_ik                : in  std_logic;
    cen_ie                : in  std_logic;
    -- CMB input
    cmb_frame_i           : in  std_logic;
    cmb_addr_ib           : in  std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0); --slave selection
    cmb_enable_ib         : in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_wr_i              : in  std_logic;
    cmb_wr_data_ib        : in  std_logic_vector(g_CMB_DATA_LEN-1 downto 0); --shift reg content
    cmb_rd_i              : in  std_logic;
    cmb_rd_data_ob        : out std_logic_vector(g_CMB_DATA_LEN-1 downto 0);
    cmb_rd_valid_o        : out std_logic;
    cmb_busy_o            : out std_logic;
    -- SPI control
    spi_sck_phase_i       : in  std_logic;
    spi_sck_polarity_i    : in  std_logic;
    spi_sck_rate_ib       : in  std_logic_vector(g_SPI_SCK_RATE_LENGTH-1 downto 0); --clock division factor
    -- SPI bus
    spi_cs_on             : out std_logic_vector(g_CMB_ADDR_LEN_SS_NUM-1 downto 0);
    spi_sck_ok            : out std_logic;
    spi_mosi_o            : out std_logic;
    spi_mosi_oe_n         : out std_logic; --for tri-state outputs
    spi_miso_i            : in  std_logic
  );
end entity;



----------------------------------------------------------------------------------------------------
-- Architecture
----------------------------------------------------------------------------------------------------
architecture rtl of spi_phy_master is

  function or_reduced(vector : std_logic_vector) return std_logic is
    variable o : std_logic;
  begin
    o := '0';
    for k in vector'range loop
      o := o or vector(k);
    end loop;
    return o;
  end function or_reduced;


  -- Local CMB slave bus
  signal saved_cmb_frame        : std_logic;
  signal saved_cmb_addr_b       : std_logic_vector(cmb_addr_ib'length-1 downto 0);
  signal saved_cmb_wr           : std_logic;
  signal saved_cmb_rd           : std_logic;
  signal cmb_busy               : std_logic;

  -- SPI control
  signal spi_sck_rate_i         : unsigned(spi_sck_rate_ib'length downto 0);
  signal saved_spi_sck_phase    : std_logic;
  signal saved_spi_sck_polarity : std_logic;
  signal saved_spi_sck_rate     : unsigned(spi_sck_rate_i'range);

  -- SPI internal FSM :
  type TYPE_STATE is
  ( s_RESET,
    s_IDLE,
    s_SLAVE_SELECT,
    s_START_OF_TRANSFERT,
    s_WORD_TRANSFERT,
    s_END_OF_TRANSFERT,
    s_SLAVE_UNSELECT);
  signal state : TYPE_STATE;

  -- shift register
  signal shift_register : std_logic_vector(cmb_wr_data_ib'length-1 downto 0);
  signal shift_counter  : natural range 0 to g_CMB_DATA_LEN-1;

  --  - clock division signals
  signal divided_clk              : std_logic;
  signal divided_clk_rising_edge  : std_logic;
  signal divided_clk_falling_edge : std_logic;
  signal divided_clk_cnt_u        : unsigned(spi_sck_rate_ib'length-1 downto 0);
  constant c_DIVIDED_CLK_CNT_ONE  : unsigned(divided_clk_cnt_u'range) := to_unsigned(1, divided_clk_cnt_u'length);

  -- others
  signal new_word : std_logic;

begin
  
  ----------------------------------------------------------------------------------------------------
  -- CMB busy output
  ----------------------------------------------------------------------------------------------------
  cmb_busy_o <= cmb_busy;
  

  ----------------------------------------------------------------------------------------------------
  -- SPI control : cast to unsigned and
  --               if spi_sck_rate_ib = "00...00" then spi_sck_rate_i <= "100...00"
  --               if spi_sck_rate_ib = "xx...xx" then spi_sck_rate_i <= "0xx...xx"
  ----------------------------------------------------------------------------------------------------
  spi_sck_rate_i <= unsigned((not or_reduced(spi_sck_rate_ib)) & spi_sck_rate_ib);


  ----------------------------------------------------------------------------------------------------
  -- CMB slave bus
  ----------------------------------------------------------------------------------------------------
  p_rd_process: process(rst_ir, clk_ik)
  begin
    if rst_ir = '1' then
      cmb_rd_data_ob <= (others => '0');
      cmb_rd_valid_o <= '0';
    
    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        if new_word = '1' then
          cmb_rd_data_ob <= shift_register;
        end if;
        cmb_rd_valid_o <= new_word;
      end if;
    end if;
  end process;
  
  --busy when not in IDLE
  cmb_busy <= '0' when state = s_IDLE else '1';


  ----------------------------------------------------------------------------------------------------
  -- SPI state machine
  ----------------------------------------------------------------------------------------------------
  p_spi_FSM : process(rst_ir, clk_ik)
  begin
    if rst_ir = '1' then
      -- local slave bus saved inputs
      saved_cmb_addr_b       <= (others => '0');
      saved_cmb_wr           <= '0';
      saved_cmb_rd           <= '0';
      saved_cmb_frame        <= '0';
      -- spi control saved inputs
      saved_spi_sck_phase    <= '0';
      saved_spi_sck_polarity <= '0';
      saved_spi_sck_rate     <= (others => '0');
      -- spi bus
      spi_cs_on              <= (others => '1');
      spi_sck_ok             <= '0';
      spi_mosi_o             <= '0';
      spi_mosi_oe_n          <= '1';
      -- shift register
      shift_register         <= (others => '0');
      shift_counter          <= 0;
      -- controls
      new_word               <= '0';
      -- state
      state                  <= s_RESET;

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        case state is

          -----------------
          -- state idle : waits for a new data transfert order (in the same frame or in a new one) polarizes spi clock signal
          when s_IDLE =>
            new_word <= '0';
            if (cmb_wr_i = '1' OR cmb_rd_i = '1') then
              -- local slave bus saved inputs
              saved_cmb_addr_b       <= cmb_addr_ib;
              saved_cmb_wr           <= cmb_wr_i;
              saved_cmb_rd           <= cmb_rd_i;
              saved_cmb_frame        <= cmb_frame_i;
              -- spi control saved inputs
              saved_spi_sck_phase    <= spi_sck_polarity_i XOR spi_sck_phase_i;
              saved_spi_sck_polarity <= spi_sck_polarity_i;
              saved_spi_sck_rate     <= spi_sck_rate_i;
              -- spi bus
              if ( saved_cmb_frame = '0' ) then
                spi_sck_ok           <= spi_sck_polarity_i;
              end if;
              -- shift register
              shift_register         <= cmb_wr_data_ib AND cmb_enable_ib;
              shift_counter          <= 0;
              -- state
              if saved_cmb_frame = '0' then
                state <= s_SLAVE_SELECT;
              else
                state <= s_WORD_TRANSFERT;
              end if;
            end if;

          -----------------
          -- state slave select: selects the slave addressed on divided clk_ik rising edge
          when s_SLAVE_SELECT =>
            if divided_clk_rising_edge = '1' then
              -- spi bus
              spi_cs_on <= NOT(saved_cmb_addr_b(spi_cs_on'length-1 downto 0));
              -- state
              state     <= s_START_OF_TRANSFERT;
            end if;

          -----------------
          -- state start of transfert: outputs the first spi data on divided clk_ik rising edge generates spi clock signal on divided clk_ik rising edge
          when s_START_OF_TRANSFERT =>
            if ( divided_clk_rising_edge = '1' ) then
              -- spi bus
              spi_sck_ok    <= saved_spi_sck_phase;
              spi_mosi_o    <= shift_register(shift_register'high) or (not saved_cmb_wr);
              spi_mosi_oe_n <= not saved_cmb_wr;
              -- state
              state         <= s_WORD_TRANSFERT;
            end if;

          -----------------
          -- state word transfert: inputs spi data on divided clk_ik falling edge outputs spi data on divided clk_ik rising edge generates spi clock signal on divided clk_ik edges
          when s_WORD_TRANSFERT =>
            if divided_clk_rising_edge = '1' then
              -- spi bus
              spi_sck_ok        <= saved_spi_sck_phase;
              spi_mosi_o        <= shift_register(shift_register'high) or (not saved_cmb_wr);
              spi_mosi_oe_n     <= not saved_cmb_wr;
            elsif ( divided_clk_falling_edge = '1' ) then
              -- spi bus
              spi_sck_ok        <= not saved_spi_sck_phase;
              -- shift register
              shift_register <= shift_register(shift_register'high-1 downto 0) & spi_miso_i;
              if ( shift_counter = g_CMB_DATA_LEN-1 ) then
                shift_counter <= 0;
              else
                shift_counter <= shift_counter + 1;
              end if;
              -- state
              if ( shift_counter = g_CMB_DATA_LEN-1 ) then
                new_word <= saved_cmb_rd;
                if ( saved_cmb_frame = '0' ) then
                  state <= s_END_OF_TRANSFERT;
                else
                  state <= s_IDLE;
                end if;
              end if;
            end if;

          -----------------
          -- state end of transfert: polarizes spi clock signal on divided clk_ik rising edge
          when s_END_OF_TRANSFERT =>
            new_word <= '0';
            if ( divided_clk_rising_edge = '1' ) then
              -- spi bus
              spi_sck_ok    <= saved_spi_sck_polarity;
              spi_mosi_o <= '0';
              spi_mosi_oe_n <= '1';
              -- state
              state      <= s_SLAVE_UNSELECT;
            end if;

          -----------------
          -- state slave unselect: unselects slave on divided clk_ik rising edge
          when s_SLAVE_UNSELECT =>
            if ( divided_clk_rising_edge = '1' ) then
              -- spi bus
              spi_cs_on  <= (others => '1');
              -- state
              state     <= s_IDLE;
            end if;

          -----------------
          -- state reset
          when others =>
            state <= s_IDLE;

        end case;
      end if;
    end if;
  end process;


  ----------------------------------------------------------------------------------------------------
  -- Clock Division
  --   generate divided_clk_falling_edge and divided_clk_rising_edge pulses
  ----------------------------------------------------------------------------------------------------
  p_divided_clk: process(rst_ir, clk_ik)
  begin
    if rst_ir = '1' then
      divided_clk              <= '1';
      divided_clk_rising_edge  <= '0';
      divided_clk_falling_edge <= '0';
      divided_clk_cnt_u        <= c_DIVIDED_CLK_CNT_ONE;

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then

        -- no SPI transfert (state = s_IDLE) and no waiting for a new cmb_frame_i data (saved_cmb_frame = '0') :
        --  resets all divided_clk signals
        if ( state = s_IDLE and saved_cmb_frame = '0' ) then
          divided_clk              <= '1';
          divided_clk_rising_edge  <= '0';
          divided_clk_falling_edge <= '0';
          divided_clk_cnt_u      <= c_DIVIDED_CLK_CNT_ONE;

        -- SPI transfert (state /= s_IDLE) :
        --  divides clk_ik
        -- waiting for a new cmb_frame_i data (state = s_IDLE) :
        --  continues to divide clk_ik and breaks as soon as the end of divided_clk_cnt_u is reached (divided_clk_rising_edge = '1')
        elsif state /= s_IDLE or divided_clk_rising_edge = '0' then

          -- divided_clk_cnt_u = saved_spi_sck_rate/2 or saved_spi_sck_rate :
          --  generates divided_clk_falling_edge or divided_clk_rising_edge pulses
          if ( divided_clk_cnt_u = saved_spi_sck_rate(saved_spi_sck_rate'left downto 1)
            OR divided_clk_cnt_u = saved_spi_sck_rate(saved_spi_sck_rate'left-1 downto 0) )
          then
            divided_clk              <= not divided_clk;
            divided_clk_falling_edge <= divided_clk;
            divided_clk_rising_edge  <= not divided_clk;
          else
            divided_clk_rising_edge  <= '0';
            divided_clk_falling_edge <= '0';
          end if;

          -- divided_clk_cnt_u for clk_ik division
          if divided_clk_cnt_u = saved_spi_sck_rate(saved_spi_sck_rate'left-1 downto 0) then
            divided_clk_cnt_u <= c_DIVIDED_CLK_CNT_ONE;
          else
            divided_clk_cnt_u <= divided_clk_cnt_u + 1;
          end if;
        end if;
      end if;
    end if;
  end process;

end architecture;
