--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN (BE-BI-BL)
--
-- File Name: timer.vhd
--
-- File versions history:
--
--     DATE          VERSION      AUTHOR             DESCRIPTION
--     08/12/2017    v_1.0        Mathieu Saccani    Timer Generation
--
-- Language: VHDL 2008
--
-- Dependencies:
--
--     None
--
-- Targeted devices:
--
--     - Vendor:  All
--     - Model:   All
--
-- Description:
--
--     The timer output is raised each g_IN_CLOCK_FREQ/g_TIMER_PULSE_FREQ.
--   
--                __________
--     reset IN             \__________________________________________________________________________
--                   ___     ___     ___     ___     ___     ___     ___     ___     ___     ___     __
--     clock IN   __|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|   |___|
--                                                   _______                         ________
--     timer OUT   _________________________________|       |_______________________|        |_____
--
--============================================================================================
--############################################################################################
--============================================================================================




----------------------------------------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;




----------------------------------------------------------------------------------------------------
-- Entity
----------------------------------------------------------------------------------------------------
entity timer is
   generic(
      g_IN_EN_PULSE_FREQ : INTEGER          := 40E6  ; --! Enable Input Frequency in Hz (default : 40MHz)
      g_TIMER_PULSE_FREQ : INTEGER          := 1E3     --! Pulse Frequency in Hz (default : 1kHz)
   );
   port(
      clk_ik           : in     std_logic; --! input Clock at G_IN_CLOCK_FREQ
      rst_ir           : in     std_logic; --! input Reset
      cen_ie           : in     std_logic; --! input clock enable
      clr_timer_i      : in     std_logic; --! input clear
      timer_en_ie      : in     std_logic; --! timer enable (can be an higher rate timer)
      timer_pulse_o    : out    std_logic  --! Pulse generated at frequency G_TIMER_PULSE_FREQ
   );
end entity;




----------------------------------------------------------------------------------------------------
-- Architecture
----------------------------------------------------------------------------------------------------

architecture rtl of timer is


  constant c_MAX_TIMER  : integer := (g_IN_EN_PULSE_FREQ/g_TIMER_PULSE_FREQ);  --Max value of the timer counter
  
  signal cnt_timer      : integer range 0 to c_MAX_TIMER-1 := c_MAX_TIMER-1;
  signal top_timer      : std_logic;

  
begin


----------------------------------------------------------------------
-- Timer
----------------------------------------------------------------------
p_timer : process (rst_ir, clk_ik)
  begin
    if ( rst_ir = '1' ) then
      cnt_timer   <= c_MAX_TIMER-1 ;
      top_timer   <= '0';
    elsif ( rising_edge(clk_ik) ) then
      top_timer  <= '0';
      
      if ( cen_ie='1' ) then                -- clock enable
      
        if (clr_timer_i='1') then           -- clear of counter
          cnt_timer  <= c_MAX_TIMER-1;
            if (cnt_timer = 0) then
              top_timer  <= '1';
            else
              top_timer  <= '0';
            end if;
            
        else
          if ( timer_en_ie='1' ) then       -- timer enable
          
            if (cnt_timer = 0) then         -- end of counter 
              top_timer  <= '1';
              cnt_timer  <= c_MAX_TIMER-1;
            else                            -- timer counting
              top_timer  <= '0';
              cnt_timer  <= cnt_timer - 1 ;
            end if;
          end if;
        end if;
      end if;
    end if;
  end process;
 
--output assignment
timer_pulse_o <= top_timer;

end architecture;