--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN 
--
-- Language: VHDL 2008
--
-- Description:
--
------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


------------------------------------------------------------------------------------------------------
-- Entity
------------------------------------------------------------------------------------------------------
entity wb_addr_decoder is
 generic(
    g_ADDR_WIDTH     : integer  := 21; 
    g_DATA_WIDTH     : integer  := 32 
  );
  port (
    --CLK/RST
    rst_ir            : in    std_logic; 
    clk_ik            : in    std_logic; 
    cen_ie            : in    std_logic; 
    --Master WishBone
    Cyc_i             : in    std_logic; 
    Stb_i             : in    std_logic; 
    Adr_ib            : in    std_logic_vector(g_ADDR_WIDTH-1 downto 0); 
    Sel_ib            : in    std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_i              : in    std_logic; 
    DatMoSi_ib        : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    DatMiSo_oqb       : out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    Ack_oq            : out   std_logic; 
    Stall_o           : out   std_logic; 
    --Slave WishBone
    Cyc_oq            : out   std_logic;     
    Adr_oqb           : out   std_logic_vector(g_ADDR_WIDTH-1 downto 0);
    Sel_oqb           : out   std_logic_vector(g_DATA_WIDTH/8-1 downto 0); 
    We_oq             : out   std_logic; 
    DatMoSi_oqb       : out   std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    --Slave ctrl
    StbCtrlReg_oq     : out   std_logic; 
    DatCtrlReg_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckCtrlReg_i      : in    std_logic; 
    StallCtrlReg_i    : in    std_logic; 
    --Slave status
    StbStatReg_oq     : out   std_logic;
    DatStatReg_ib     : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckStatReg_i      : in    std_logic; 
    StallStatReg_i    : in    std_logic;   
    --Slave regultaor
    StbRegul_oq       : out   std_logic;
    DatRegul_ib       : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckRegul_i        : in    std_logic; 
    StallRegul_i      : in    std_logic;     
    --Slave eeprom
    StbEeprom_oq      : out   std_logic;
    DatEeprom_ib      : in    std_logic_vector(g_DATA_WIDTH-1 downto 0); 
    AckEeprom_i       : in    std_logic; 
    StallEeprom_i     : in    std_logic
  );
end entity ;




------------------------------------------------------------------------------------------------------
-- Architecture
------------------------------------------------------------------------------------------------------
architecture rtl of wb_addr_decoder is
  
  -- SPI cycle FSM :
  type t_sel_enum is (
    c_sel_ctrl,
    c_sel_status,
    c_sel_regulator,
    c_sel_eeprom,
    c_sel_nothing
  );
  attribute syn_enum_encoding : STRING;
  attribute syn_enum_encoding of t_sel_enum : type is "one-hot";
  signal selection            : t_sel_enum;
  
  signal Stb   : std_logic;
  signal Cyc_q : std_logic;
  signal Ack_q : std_logic;

begin 
  
  ---------------------------
  --Address decoding
  ---------------------------
  selection <= c_sel_ctrl      when unsigned(Adr_ib) >= x"00000" AND unsigned(Adr_ib) < x"00020" --0x20 = 32 CTRL      registers  --VME_addr 0x000
          else c_sel_status    when unsigned(Adr_ib) >= x"00020" AND unsigned(Adr_ib) < x"00040" --0x20 = 32 STATUS    registers  --VME_addr 0x080
          else c_sel_regulator when unsigned(Adr_ib) >= x"00040" AND unsigned(Adr_ib) < x"00060" --0x20 = 32 REGULATOR registers  --VME_addr 0x100
          else c_sel_eeprom    when unsigned(Adr_ib) >= x"00060" AND unsigned(Adr_ib) < x"00080" --0x2  = 2  EEPROM    registers  --VME_addr 0x180
          else c_sel_nothing; 
  
  --internal strobe
  Stb <= Stb_i and Cyc_i and not(Ack_q);
  
  ---------------------------
  --Wishbone routing to slaves
  ---------------------------
  p_wb_routing: process(clk_ik, rst_ir)
  begin
    if rst_ir = '1' then
      Cyc_q        <= '0';
      We_oq         <= '0';
      Adr_oqb       <= (others=>'0');
      Sel_oqb       <= (others=>'0');
      DatMoSi_oqb   <= (others=>'0');
      --
      Ack_q         <= '0';
      DatMiSo_oqb   <= (others=>'0');
      StbCtrlReg_oq <= '0';
      StbStatReg_oq <= '0';
      StbRegul_oq   <= '0';
      StbEeprom_oq  <= '0';

    elsif rising_edge(clk_ik) then
      if cen_ie = '1' then
        ---------------------------
        -- Forwarding (pipeline)
        ---------------------------
        Cyc_q         <= Cyc_i and not(Ack_q);
        We_oq         <= We_i;
        Adr_oqb       <= Adr_ib;
        Sel_oqb       <= Sel_ib;
        DatMoSi_oqb   <= DatMoSi_ib;
        --default
        Ack_q         <= '0';
        DatMiSo_oqb   <= (others=>'0');
        StbCtrlReg_oq <= '0';
        StbStatReg_oq <= '0';
        StbEeprom_oq  <= '0';
        
        --Routing according to the selection
        case selection is
          
          when c_sel_ctrl => 
            StbCtrlReg_oq <= Stb and not(AckCtrlReg_i);
            DatMiSo_oqb   <= DatCtrlReg_ib;
            Ack_q         <= AckCtrlReg_i;
            
          when c_sel_status =>
            StbStatReg_oq <= Stb and not(AckStatReg_i);
            DatMiSo_oqb   <= DatStatReg_ib;
            Ack_q         <= AckStatReg_i;
            
          when c_sel_regulator => 
            StbRegul_oq   <= Stb and not(AckRegul_i);
            DatMiSo_oqb   <= DatRegul_ib;
            Ack_q         <= AckRegul_i;
            
          when c_sel_eeprom => 
            StbEeprom_oq  <= Stb and not(AckEeprom_i);
            DatMiSo_oqb   <= DatEeprom_ib;
            Ack_q         <= AckEeprom_i;
            
          when c_sel_nothing => 
            DatMiSo_oqb   <= x"DEADBEEF";
            Ack_q         <= Stb;
            
          when others => 
            null;
        end case;
      end if;
    end if;
  end process;
  
  --Cyc & Ack outputs
  Ack_oq <= Ack_q AND Stb_i;
  Cyc_oq <= Cyc_q AND not(Ack_q);
    
  ---------------------------
  -- Stall management
  ---------------------------
  Stall_o <=   StallCtrlReg_i   when selection = c_sel_ctrl 
          else StallStatReg_i   when selection = c_sel_status
          else StallRegul_i     when selection = c_sel_regulator
          else StallEeprom_i    when selection = c_sel_eeprom
          else '0'              when selection = c_sel_nothing
          else '0'; 
          
  --Another simplified version:       
  --Stall_o <= not(Ack_oq) AND Stb_i AND Cyc_i;
  
end architecture;