--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN (BE-BI-BL)
--
-- Language: VHDL 2008
--
-- Dependencies:
--
--     none
--
-- Description:
--
--     ADC AD7682 : - 16-Bit, 
--                  - 4-Channels, 
--                  - internal temperature, 
--                  - 250 kSPS PulSAR ADC
--
--     https://www.analog.com/media/en/technical-documentation/data-sheets/ad7682_7689.pdf
--
--============================================================================================




-----------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;




--===================================================================
-- Entity
--===================================================================
entity adc_ad7682 is
  generic (
    G_TIMING_CHECK_EN : std_logic := '0'
  );
  port (
    --Analog input
    Vin0_ib16 : in  std_logic_vector(15 downto 0); --
    Vin1_ib16 : in  std_logic_vector(15 downto 0); --
    Vin2_ib16 : in  std_logic_vector(15 downto 0); --
    Vin3_ib16 : in  std_logic_vector(15 downto 0); --
    Temp_ib16 : in  std_logic_vector(15 downto 0); --
    --SPI bus
    SCK       : in  std_logic; -- Serial clock input (SCK)
    DIN       : in  std_logic; -- Serial data input (SDI-MOSI)
    SDO       : out std_logic; -- Serial data output (SDO-MISO)
    CVN       : in  std_logic  -- Chip select input (CVN)
  );
end entity;




--============================================================================
-- architecture 
--============================================================================
architecture behavioral OF adc_ad7682 is

  constant tSCKL              :  time  := 12 ns; -- SCK low time                        
  constant tSCKH              :  time  := 12 ns; -- SCK high time                     
  constant tHSDO              :  time  :=  5 ns; -- SDO remains valid after SCK falling  
  constant tDSDO              :  time  := 48 ns; -- SDO valid delay after SCK falling  
  constant tEN                :  time  := 45 ns; -- CVN low to SDO MSB valid         
  constant tCNVH              :  time  := 10 ns; -- CVN min high time           
  constant tDIS               :  time  := 50 ns; -- CVN high (or last SCK falling) to SDO high impedance 
  constant tCLSCK             :  time  := 10 ns; -- CVN low to SCK rising           
  constant tSDIN              :  time  :=  5 ns; -- SCK rising to DIN setup time    
  constant tHDIN              :  time  :=  5 ns; -- SCK rising to DIN hold time   
      
  signal SCK_th               :   std_logic; 
  signal SCK_LH               :   std_logic;
   
  -- lengths
  constant REG_ADDR_LENGTH    : positive  := 7;
  constant REG_DATA_LENGTH    : positive  := 16;
  
  -- number of registers
  constant C_ADC_REGISTERS_NB : positive  := 5;
  
  type tab_register_type is array (integer range <>) of std_logic_vector(REG_DATA_LENGTH-1 downto 0);
  signal reg_b16              : tab_register_type(0 to C_ADC_REGISTERS_NB-1);

  -- number of variation before beeing stable (3 time +1 then 3 times -2)
  constant C_UPDATE_LOOP_NB   : positive  := 3;
  
  --number of dummy accesses during bit (do not increment data)
  --4 in BIT and 11=9+2 first refresh steps (see .do)
  constant C_DUMMY_NB         : positive  := 15;
  
  type conf_reg_type is array(2 downto 0) of std_logic_vector(13 downto 0);
  signal conf_reg_temp_b14    : conf_reg_type :=("00" & x"000",
                                                 "00" & x"000",
                                                 "00" & x"000");
                                          
  signal SDO_int              : std_logic := 'Z'; --to avoid 'U' at startup
   
  constant c_TEMP_IND         : integer := C_ADC_REGISTERS_NB-1;
  
  --startup time
  signal start                : std_logic;
   
begin


--------------------
--Reset
--------------------
p_startup : process
  begin
    start <='0';
    wait for 5 us; --settling time
    start <='1';
    wait;
end process;
    

-----------------------------------------------------------
-- process for Clock Creation
-- Create a Delayed Clock signal by tH for internal timing
-----------------------------------------------------------
  p_SCK_THDELAY: process (SCK)
  begin
   SCK_tH <= SCK after tHDIN;
  end process;
  
  p_SCK_LHDELAY: process (SCK)
  begin
   SCK_LH <= SCK after tHDIN + 1 ns;
  end process;
  
  
   
    
----------------------------------------------------------
-- Main Control Path
----------------------------------------------------------
  p_main_process : process (CVN, SCK, SCK_TH, SCK_LH, DIN)
    type bus_cycle_type is ( RESET,
                             STAND_BY,
                             ACQ_CONV
                            );
    variable bus_cycle_state : bus_cycle_type;
    variable address_in      : std_logic_vector(REG_ADDR_LENGTH-1 downto 0);
    variable address         : integer;
    variable data_in         : std_logic_vector(13 downto 0);
    variable data_out        : std_logic_vector(2*REG_DATA_LENGTH-1 downto 0);
    variable data_cnt        : integer range 0 to 2*REG_DATA_LENGTH;
    variable addr_cnt        : integer range 0 to 14;
    variable temp_int        : integer   := 0;
    variable startup         : integer   := 3; --3 first conversions are giving nothing
    variable channel         : integer range 0 to C_ADC_REGISTERS_NB-1;
  begin
    
    --*************************
    -- Start Operations on falling edge of CVN, Reset all states on rising edge
    if start='0' then     
      bus_cycle_state := RESET;
      SDO_int         <= 'Z' after tDIS;
      data_cnt        := 0;
      addr_cnt        := 0;
      address_in      := (others=>'0');
      data_in         := (others=>'0'); 
      data_out        := (others=>'0'); --for debug
      temp_int        := 0;
      startup         := 3; --3 first conversions are giving nothing
      channel         := 0;
      reg_b16(0)      <= Vin0_ib16;
      reg_b16(1)      <= Vin1_ib16;
      reg_b16(2)      <= Vin2_ib16;
      reg_b16(3)      <= Vin3_ib16;
      reg_b16(4)      <= Temp_ib16;

    elsif rising_edge(CVN) then
    
      bus_cycle_state := STAND_BY;     
      SDO_int         <= 'Z' after tDIS;
      data_cnt        := 0;
      addr_cnt        := 0;
      
      --pipeline of CFG
      conf_reg_temp_b14(1)<= conf_reg_temp_b14(0); -- pipeline n-1
      conf_reg_temp_b14(2)<= conf_reg_temp_b14(1); -- pipeline n-2
      
      --the 3 first conversions gives nothing
      if startup>0 then
        startup := startup-1;
      end if;
            
      --*************************
      -- new input latch
      reg_b16(0) <= Vin0_ib16;
      reg_b16(1) <= Vin1_ib16;
      reg_b16(2) <= Vin2_ib16;
      reg_b16(3) <= Vin3_ib16;
      reg_b16(4) <= Temp_ib16;

    else
      --*************************
      -- channel determination at the end of the frame
      --temperature sensor
      if conf_reg_temp_b14(2)(12 downto 10) = "011" then
        channel := c_TEMP_IND;
      -- voltage 0 to C_ADC_REGISTERS_NB-1
      else
        channel := to_integer(unsigned(conf_reg_temp_b14(2)(9 downto 7)));
      end if;
    
      case bus_cycle_state is
      
        when RESET => 
          if falling_edge(CVN) then
            SDO_int <= 'Z' after tEN;
          elsif rising_edge(CVN) then
            bus_cycle_state := ACQ_CONV;
          end if;
      
        when STAND_BY => 
          if falling_edge(CVN) then
            addr_cnt        := 0;
            data_cnt        := 0;
            bus_cycle_state := ACQ_CONV;
            --preload the first data
            if startup=0 then
              SDO_int <= reg_b16(channel)(REG_DATA_LENGTH-1) after tEN;
            else
              SDO_int <= 'Z';
            end if;
          end if;
             
         when ACQ_CONV =>
           
           --receive conf_reg
           if rising_edge(SCK) then 
             if (addr_cnt < 14) then
               data_in(13-addr_cnt) := DIN;
               addr_cnt  := addr_cnt + 1; 
             else
               conf_reg_temp_b14(0) <= data_in;
             end if;
           end if;
           
           --emit data
           if falling_edge(SCK) then 
             
             -- data counter
             if data_cnt <= 2*REG_DATA_LENGTH-1 then
               data_cnt := data_cnt + 1;
             else
               data_cnt := 0;
             end if;
             
             if data_cnt < 2*REG_DATA_LENGTH-1 then
               --data ouput
               data_out := reg_b16(channel) & conf_reg_temp_b14(2) & "00";
               if startup=0 then
                 SDO_int <= data_out(2*REG_DATA_LENGTH-data_cnt-1) after tDSDO; 
               else
                 SDO_int <= 'Z';
               end if;
             end if;
             
           --reload the CFG
           elsif CVN='0' and data_cnt=REG_DATA_LENGTH-1 then 
             data_out := reg_b16(channel) & conf_reg_temp_b14(2) & "00";
             SDO_int <= data_out(2*REG_DATA_LENGTH-data_cnt-1) after tDSDO; 
           
           --last data
           elsif CVN='0' and data_cnt=2*REG_DATA_LENGTH-1 then 
              SDO_int <= SDO_int after tDSDO; 
           end if;
          
         when others =>
           null;
      end case;
    end if;  
  end process;
   
   
 --outputs
 SDO <= SDO_int;
 
 
 
   
--------------------------------------------------------------------------------------------------------------
-- process for Timing Checks
-- The following timings are checked Tss, Tsu, Th, Tcss, Tcsh, Thd, Tcd
-------------------------------------------------------------------------------------------------------------- 
  p_timing_Checks: process  (SCK, CVN, DIN)
   variable RisingTIME_CS, FallingTIME_CS : time;
   variable RisingTIME, FallingTIME : time;
   variable TestTime, TestTime2, TestNOW: time;
   variable start_test : std_logic :='0';
  begin
  
    --Test timing only if time is not 0ps
    if G_TIMING_CHECK_EN='1' then
      if start_test = '0' then
        TestNOW        := NOW;
        RisingTIME_CS  := NOW;
        FallingTIME_CS := NOW;
        RisingTIME     := NOW;
        FallingTIME    := NOW;
        if (TestNOW > 100 ps) then  --superior to the propagation times in FPGA buffers (not to get X at input)
          start_test := '1'; 
        end if;
    
      else
        if SCK'EVENT and SCK = '1' then
        
          TestTime := CVN'LAST_EVENT;
          TestTime := DIN'LAST_EVENT;
          TestNOW  := NOW;
       
          --SCK low time
          assert (NOW-FallingTIME >= tSCKL)
          report "ERROR: SCK low time duration"
          severity ERROR;
       
          --CVN setup time (to SCK)
          assert (CVN'LAST_EVENT >= tCLSCK)
          report "ERROR: CHIP SELECT SETUP time VIOLATION"
          severity ERROR;
          
          --CVN to first data out MSB
          assert (CVN'LAST_EVENT >= tEN)
          report "ERROR: CHIP SELECT Low to first data out is to short"
          severity ERROR;
          
          --DIN setup time
          assert (DIN'LAST_EVENT >= tSDIN)
          report "ERROR: SERIAL INPUT SETUP time VIOLATION"
          severity ERROR;
        elsif (SCK'EVENT and SCK = '0') then
          
          --SCK high time
          assert (NOW-RisingTIME >= tSCKH)
          report "ERROR: SCK high time duration"
          severity ERROR;
        
        end if;
        
        if (SCK'EVENT and SCK = '0') then
          FallingTIME := NOW;
        end if;
        
        if (SCK'EVENT and SCK = '1') then
          RisingTIME := NOW;
        end if;
        
        if (CVN'EVENT and CVN = '1') then
          RisingTIME_CS := NOW;
        end if;
        
        if (CVN'EVENT and CVN = '0') then
          FallingTIME_CS := NOW;
         
          -- CVN high duration
          assert (FallingTIME_CS-RisingTIME_CS >= tCNVH)
          report "ERROR: CHIP SELECT min high time duration < tCNVH"
          severity ERROR;
        end if;
      end if; 
    end if;
  end process;
   
end architecture;
