--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN (BE-BI-BL)
--
-- Language: VHDL 2008
--
-- Dependencies:
--
--     none
--
-- Description:
--
--     ADC LTC2484: - 24-Bit, 
--                  - 1 Channel, 
--                  - internal temperature, 
--                  - 50/60kSPS Delta-Sigma
--
--     https://www.analog.com/media/en/technical-documentation/data-sheets/2484ff.pdf
--
--============================================================================================




-----------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;




--===================================================================
-- Entity
--===================================================================
entity adc_ltc2484 is
  generic (
    G_TIMING_CHECK_EN : std_logic := '0'
  );
  port (
    --Analog input
    Vin_ib25  : in  std_logic_vector(24 downto 0); -- sign bit
    Temp_ib24 : in  std_logic_vector(23 downto 0); --
    --SPI bus
    SCK       : in  std_logic; -- Serial clock input (SCK)
    SDI       : in  std_logic; -- Serial data input (SDI-MOSI)
    SDO       : out std_logic; -- Serial data output (SDO-MISO)
    CS_N      : in  std_logic  -- Chip select input (CS_N)
  );
end entity;




--============================================================================
-- architecture 
--============================================================================
architecture behavioral OF adc_ltc2484 is

  constant tSCKL              :  time  := 12 ns; -- SCK low time                        
  constant tSCKH              :  time  := 12 ns; -- SCK high time                     
  constant tHSDO              :  time  :=  5 ns; -- SDO remains valid after SCK falling  
  constant tDSDO              :  time  := 48 ns; -- SDO valid delay after SCK falling  
  constant tEN                :  time  := 45 ns; -- CS_N low to SDO MSB valid         
  constant tCNVH              :  time  := 10 ns; -- CS_N min high time           
  constant tDIS               :  time  := 50 ns; -- CS_N high (or last SCK falling) to SDO high impedance 
  constant tCLSCK             :  time  := 10 ns; -- CS_N low to SCK rising           
  constant tSDIN              :  time  :=  5 ns; -- SCK rising to SDI setup time    
  constant tHDIN              :  time  :=  5 ns; -- SCK rising to SDI hold time   
      
  signal SCK_th               :   std_logic; 
  signal SCK_LH               :   std_logic;
  signal SDO_int              : std_logic := 'Z'; --to avoid 'U' at startup
  
  -- lengths
  constant REG_ADDR_LENGTH    : positive  := 7;
  constant REG_DATA_LENGTH    : positive  := 25; --24b + sign
  
  -- number of registers
  constant C_ADC_REGISTERS_NB : positive  := 2;
  
  type tab_register_type is array (integer range <>) of std_logic_vector(REG_DATA_LENGTH-1 downto 0);
  signal reg_b25              : tab_register_type(0 to C_ADC_REGISTERS_NB-1);
  
  --output bits
  constant dmy                : std_logic := '0';
  signal end_conv_n           : std_logic := '1'; --End of Conversion

  --config
  signal conf_reg_b8          : std_logic_vector(7 downto 0);
  signal EN                   : std_logic; -- Enable
  signal IM                   : std_logic; -- Internal Temperature
  signal FOA                  : std_logic; -- No 50Hz rejection
  signal FOB                  : std_logic; -- No 60Hz rejection
  signal SPD                  : std_logic; -- Speed mode, double output rate (disable the offset auto-calibration)
   
  constant c_TEMP_IND         : integer := C_ADC_REGISTERS_NB-1;
  
  --startup time
  signal start             : std_logic;
   
begin


--------------------
--Reset
--------------------
p_startup : process
  begin
    start <='0';
    wait for 5 us; --settling time
    start <='1';
    wait;
end process;
    

-----------------------------------------------------------
-- process for Clock Creation
-- Create a Delayed Clock signal by tH for internal timing
-----------------------------------------------------------
  p_SCK_THDELAY: process (SCK)
  begin
   SCK_tH <= SCK after tHDIN;
  end process;
  
  p_SCK_LHDELAY: process (SCK)
  begin
   SCK_LH <= SCK after tHDIN + 1 ns;
  end process;
  
  
  --Reduced time for simulation (67 to 147ms => 5us)
  p_conversion_time: process
  begin
    wait for 5 us;
    end_conv_n <= '0';
    wait until rising_edge(CS_N);
    end_conv_n <= '1';
  end process;
   
    
----------------------------------------------------------
-- Main Control Path
----------------------------------------------------------
  p_main_process : process (CS_N, SCK, SCK_TH, SCK_LH, SDI)
    type bus_cycle_type is ( RESET,
                             STAND_BY,
                             ACQ_CONV
                            );
    variable bus_cycle_state : bus_cycle_type;
    variable address_in      : std_logic_vector(REG_ADDR_LENGTH-1 downto 0);
    variable address         : integer;
    variable data_in         : std_logic_vector(7 downto 0);
    variable data_out        : std_logic_vector(31 downto 0);
    variable data_cnt        : integer range 0 to 32-1;
    variable addr_cnt        : integer range 0 to 8; --8 bits of config
    variable temp_int        : integer   := 0;
    variable channel         : integer range 0 to C_ADC_REGISTERS_NB-1;
  begin
    
    --*************************
    -- Start Operations on falling edge of CS_N, Reset all states on rising edge
    if start='0' then
      
      conf_reg_b8     <= "10000000"; 
      EN              <= '1'; --voltage by default
      IM              <= '0'; --voltage by default
      FOA             <= '0'; --unused in this model
      FOB             <= '0'; --unused in this model
      SPD             <= '0'; --unused in this model
      
      bus_cycle_state := RESET;
      SDO_int         <= 'Z' after tDIS;
      data_cnt        := 0;
      addr_cnt        := 0;
      address_in      := (others=>'0');
      data_in         := (others=>'0'); 
      data_out        := (others=>'0'); --for debug
      temp_int        :=0;
      
      --Latch
      reg_b25(0) <= Vin_ib25;
      reg_b25(1) <= '0' & Temp_ib24;

    elsif rising_edge(CS_N) or rising_edge(start) then
    
      bus_cycle_state := ACQ_CONV;    
      SDO_int          <= 'Z' after tDIS;
      data_cnt        := 0;
      addr_cnt        := 0;
      
      --Pipeline
      EN   <= conf_reg_b8(7); -- Enable
      IM   <= conf_reg_b8(3); -- Internal Temperature
      FOA  <= conf_reg_b8(2); -- No 50Hz rejection
      FOB  <= conf_reg_b8(1); -- No 60Hz rejection
      SPD  <= conf_reg_b8(0); -- Speed mode, double output rate (disable the offset auto-calibration)

      --*************************
      -- new input latch
      reg_b25(0) <= Vin_ib25;
      reg_b25(1) <= '0' & Temp_ib24;

    else
      --*************************
      -- EN   EN  = Enable
      -- IM   IM  = Internal Temperature
      -- FOA  FOA = No 50Hz rejection
      -- FOB  FOB = No 60Hz rejection
      -- SPD  SPD = Speed mode, double output rate (disable the offset auto-calibration)
      --
      -- Modes:  EN  IM  FOA FOB SPD 
      --          0   X   X   X   X  : keep previous mode
      -- used     1   0   0   0   0  : External Input, 50Hz and 60Hz Rejection, Autocalibration    => CONV in 147.0ms
      --          1   0   0   0   1  : External Input, 50Hz and 60Hz Rejection, 2x Speed           => CONV in  73.6ms
      --          1   0   0   1   0  : External Input, 50Hz Rejection, Autocalibration             => CONV in 160.0ms
      --          1   0   0   1   1  : External Input, 50Hz Rejection, 2x Speed                    => CONV in  80.0ms
      --          1   0   1   0   0  : External Input, 60Hz Rejection, Autocalibration             => CONV in 133.0ms
      --          1   0   1   0   1  : External Input, 60Hz Rejection, 2x Speed                    => CONV in  67.0ms
      --          1   0   1   1   0  : reserved
      --          1   0   1   1   1  : reserved
      -- used     1   1   0   0   0  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
      --          1   1   0   0   1  : Temperature Input, 50Hz and 60Hz Rejection, Autocalibration
      --          1   1   0   1   0  : Temperature Input, 50Hz Rejection, Autocalibration
      --          1   1   0   1   1  : Temperature Input, 50Hz Rejection, Autocalibration
      --          1   1   1   0   0  : Temperature Input, 60Hz Rejection, Autocalibration
      --          1   1   1   0   1  : Temperature Input, 60Hz Rejection, Autocalibration
      --          1   1   1   1   0  : reserved
      --          1   1   1   1   1  : reserved
      
      if EN = '0' then
        channel := channel;    --keep previous mode
      else
        --/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\
        --  FOA, FOB and SPD not used in this model
        --/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\/!\
        if IM='0' then
          channel := 0;         --input voltage
        else
          channel := c_TEMP_IND; --temperature
        end if;
      end if;
    
      case bus_cycle_state is
      
        when RESET => 
          if falling_edge(CS_N) then
            SDO_int <= 'Z' after tEN;
            addr_cnt        := 0;
            data_cnt        := 0;
            bus_cycle_state := ACQ_CONV;
            SDO_int <= end_conv_n after tEN; --preload the first data
          elsif rising_edge(CS_N) then
            bus_cycle_state := ACQ_CONV;
          end if;

         when ACQ_CONV =>
           
           --receive configuration
           if rising_edge(SCK) then 
             if (addr_cnt < 8) then
               data_in(7-addr_cnt) := SDI;
               addr_cnt  := addr_cnt + 1; 
             else
               conf_reg_b8 <= data_in(7 downto 0); 
             end if;
           end if;
           
           --emit data
           if falling_edge(SCK) then 
             
             --data ouput   31      | 30  | 29-5 (sign+24b)  | 4-0
             data_out := end_conv_n & dmy & reg_b25(channel) & "00000";
             if data_cnt < 32-1 then
               SDO_int <= data_out(32-data_cnt-1) after tDSDO; 
             end if;
             
             -- data counter
             if data_cnt <= 32-1 then
               data_cnt := data_cnt + 1;
             else
               data_cnt := 0;
             end if;
           
           --last data
           elsif CS_N='0' and data_cnt=32-1 then 
              SDO_int <= SDO_int after tDSDO; 
           end if;
          
         when others =>
           null;
           
      end case;
    end if;  
  end process;
   
   
 --outputs
 SDO <= SDO_int;
 
 
 
   
--------------------------------------------------------------------------------------------------------------
-- process for Timing Checks
-- The following timings are checked Tss, Tsu, Th, Tcss, Tcsh, Thd, Tcd
-------------------------------------------------------------------------------------------------------------- 
  p_timing_Checks: process  (SCK, CS_N, SDI)
   variable RisingTIME_CS, FallingTIME_CS : time;
   variable RisingTIME, FallingTIME : time;
   variable TestTime, TestTime2, TestNOW: time;
   variable start_test : std_logic :='0';
  begin
  
    --Test timing only if time is not 0ps
    if G_TIMING_CHECK_EN='1' then
      if start_test = '0' then
        TestNOW        := NOW;
        RisingTIME_CS  := NOW;
        FallingTIME_CS := NOW;
        RisingTIME     := NOW;
        FallingTIME    := NOW;
        if (TestNOW > 100 ps) then  --superior to the propagation times in FPGA buffers (not to get X at input)
          start_test := '1'; 
        end if;
    
      else
        if SCK'EVENT and SCK = '1' then
        
          TestTime := CS_N'LAST_EVENT;
          TestTime := SDI'LAST_EVENT;
          TestNOW  := NOW;
       
          --SCK low time
          assert (NOW-FallingTIME >= tSCKL)
          report "ERROR: SCK low time duration"
          severity ERROR;
       
          --CS_N setup time (to SCK)
          assert (CS_N'LAST_EVENT >= tCLSCK)
          report "ERROR: CHIP SELECT SETUP time VIOLATION"
          severity ERROR;
          
          --CS_N to first data out MSB
          assert (CS_N'LAST_EVENT >= tEN)
          report "ERROR: CHIP SELECT Low to first data out is to short"
          severity ERROR;
          
          --SDI setup time
          assert (SDI'LAST_EVENT >= tSDIN)
          report "ERROR: SERIAL INPUT SETUP time VIOLATION"
          severity ERROR;
        elsif (SCK'EVENT and SCK = '0') then
          
          --SCK high time
          assert (NOW-RisingTIME >= tSCKH)
          report "ERROR: SCK high time duration"
          severity ERROR;
        
        end if;
        
        if (SCK'EVENT and SCK = '0') then
          FallingTIME := NOW;
        end if;
        
        if (SCK'EVENT and SCK = '1') then
          RisingTIME := NOW;
        end if;
        
        if (CS_N'EVENT and CS_N = '1') then
          RisingTIME_CS := NOW;
        end if;
        
        if (CS_N'EVENT and CS_N = '0') then
          FallingTIME_CS := NOW;
         
          -- CS_N high duration
          assert (FallingTIME_CS-RisingTIME_CS >= tCNVH)
          report "ERROR: CHIP SELECT min high time duration < tCNVH"
          severity ERROR;
        end if;
      end if; 
    end if;
  end process;
   
end architecture;
