--============================================================================================
--##################################   Module Information   ##################################
--============================================================================================
--
-- Company: CERN (BE-BI-BL)
--
--
-- Language: VHDL 2008
--
-- Description:
--
--        https://www.analog.com/media/en/technical-documentation/data-sheets/ad5663.pdf
--
--============================================================================================
--############################################################################################
--============================================================================================


----------------------------------------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

----------------------------------------------------------------------------------------------------
-- Entity
----------------------------------------------------------------------------------------------------
entity DAC_AD5663 is
port(
  --DAC AD5660
  sclk_ik     : in  std_logic;
  din_i       : in  std_logic; 
  sync_in     : in  std_logic; 
  --triggers        
  ldac_in     : in  std_logic; 
  clr_in      : in  std_logic; 
  -- analog voltage outputs     
  vout_A_ob16 : out std_logic_vector(15 downto 0);  -- : 0xFFFF=3.3V and 0x0000=0V
  vout_B_ob16 : out std_logic_vector(15 downto 0)   -- : 0xFFFF=3.3V and 0x0000=0V
  );
end entity;


----------------------------------------------------------------------------------------------------
-- Architecture
----------------------------------------------------------------------------------------------------
architecture model of DAC_AD5663 is
    
  --startup value : DAC output powers up to 0 V (AD5663)
  constant c_START_VAL     : integer range 0 to 2**16-1 := 0;
  
  --startup time
  signal start             : std_logic;

  --24bits shift register
  signal spi_shift_b24 : std_logic_vector(23 downto 0) := "000000" & "00" & std_logic_vector(to_unsigned(c_START_VAL,16));
  signal shift_cnt     : integer  range 0 to 23 := 0;
  
  --fields
  signal input_reg_b24 : std_logic_vector(23 downto 0);
  alias  command_b3    : std_logic_vector( 2 downto 0) is input_reg_b24(21 downto 19);
  alias  address_b3    : std_logic_vector( 2 downto 0) is input_reg_b24(18 downto 16);
  alias  data_bits_b16 : std_logic_vector(15 downto 0) is input_reg_b24(15 downto 0);
  
  --dac register
  signal dac_reg_A_b16 : std_logic_vector(15 downto 0);
  signal dac_reg_B_b16 : std_logic_vector(15 downto 0);
  
  --LDA mode
  signal LDAC_reg_A    : std_logic;
  signal LDAC_reg_B    : std_logic;
  
  --outputs
  signal vout_A_b16    : std_logic_vector(15 downto 0);
  signal vout_B_b16    : std_logic_vector(15 downto 0);
  
  --data ready to be decoded
  signal data_ready     : std_logic; 
  signal data_ready_dly : std_logic; 
   
begin
    
--------------------
--Reset
--------------------
p_startup : process
  begin
    start <='0';
    wait for 5 us; --settling time
    start <='1';
    wait;
end process;
    
--------------------
--SPI data sampling
--------------------
p_spi_input : process(sclk_ik, sync_in, start, clr_in)
  begin
  --rst
  if start = '0' then 
    spi_shift_b24    <= (others=>'0');
    input_reg_b24    <= (others=>'0');
    shift_cnt        <= 23;
    data_ready       <= '0';
  --clear
  elsif falling_edge(clr_in) then 
    spi_shift_b24    <= (others=>'0');
    input_reg_b24    <= (others=>'0');
    shift_cnt        <= 23;
    data_ready       <= '0';
  --SYNC
  elsif sync_in='1' then       
    spi_shift_b24    <= (others=>'0');
    shift_cnt        <= 23;
    data_ready       <= '0';
  --shift data in
  elsif falling_edge(sclk_ik) then       --sample on falling edge
    spi_shift_b24    <= spi_shift_b24(22 downto 0) & din_i;
    if shift_cnt=0 then
      shift_cnt      <= 23;
    else
      shift_cnt      <= shift_cnt-1;
    end if;
    data_ready       <= '0';
    --latch the register
    elsif rising_edge(sclk_ik) then    
      if shift_cnt=23 then
        input_reg_b24 <= spi_shift_b24;
        data_ready    <= '1';
      end if;
  end if;
end process;

--delay
data_ready_dly <= transport data_ready after 1 ns;

--------------------
--command_b3 decoding
--------------------
-- | c2 | c1 | c0 |
-- ----------------
-- |  0 |  0 |  0 | write input reg n
-- |  0 |  0 |  1 | update DAC reg n
-- |  0 |  1 |  0 | write input reg n & update all (sowftwre LDAC)
-- |  0 |  1 |  1 | write input reg n & update DAC reg n
-- |  1 |  0 |  0 | power-down DAC
-- |  1 |  0 |  1 | soft reset
-- |  1 |  1 |  0 | LDA register setup
-- |  1 |  1 |  1 | reserved
p_command : process(start, shift_cnt, clr_in, data_ready_dly)
begin
  if start = '0' or  falling_edge(clr_in) then 
    dac_reg_A_b16 <= (others=>'0');
    dac_reg_B_b16 <= (others=>'0');
    LDAC_reg_A    <= '0';
    LDAC_reg_B    <= '0';
    vout_A_b16    <= (others=>'0');
    vout_B_b16    <= (others=>'0');
    
  elsif rising_edge(data_ready_dly) then 
  
    case command_b3 is 
      
      -----------------
      --write input reg n
      when "000" => 
        --addr=0 DAC A
        if    address_b3="000" then
          dac_reg_A_b16 <= data_bits_b16;
        --addr=1 DAC B
        elsif address_b3="001" then
          dac_reg_B_b16 <= data_bits_b16;
        --addr=7 DAC A & B
        elsif address_b3="111" then
          dac_reg_A_b16 <= data_bits_b16;
          dac_reg_B_b16 <= data_bits_b16;
        end if;
        
      -----------------
      --update DAC reg n
      when "001" => 
        --addr=0 DAC A
        if    address_b3="000" then
          vout_A_b16 <= dac_reg_A_b16;
        --addr=1 DAC B
        elsif address_b3="001" then
          vout_B_b16 <= dac_reg_B_b16;
        --addr=7 DAC A & B
        elsif address_b3="111" then
          vout_A_b16 <= dac_reg_A_b16;
          vout_B_b16 <= dac_reg_B_b16;
        end if;
      
      -----------------
      --write input reg n & update all (sowftwre LDAC)
      when "010" => 
        --addr=0 DAC A
        if    address_b3="000" then
          dac_reg_A_b16 <= data_bits_b16; --A
          vout_A_b16    <= data_bits_b16;
          vout_B_b16    <= dac_reg_B_b16; --B
        --addr=1 DAC B
        elsif address_b3="001" then
          vout_A_b16    <= dac_reg_A_b16; --A
          dac_reg_B_b16 <= data_bits_b16; --B
          vout_B_b16    <= data_bits_b16; 
        --addr=7 DAC A & B
        elsif address_b3="111" then
          dac_reg_A_b16 <= data_bits_b16; --A
          vout_A_b16    <= data_bits_b16;
          dac_reg_B_b16 <= data_bits_b16; --B
          vout_B_b16    <= data_bits_b16; 
        end if;

      -----------------
      --write input reg n & update DAC reg n
      when "011" => 
        --addr=0 DAC A
        if    address_b3="000" then
          dac_reg_A_b16 <= data_bits_b16; --A
          vout_A_b16    <= data_bits_b16;
        --addr=1 DAC B
        elsif address_b3="001" then
          dac_reg_B_b16 <= data_bits_b16; --B
          vout_B_b16    <= data_bits_b16;
        --addr=7 DAC A & B
        elsif address_b3="111" then
          dac_reg_A_b16 <= data_bits_b16; --A
          vout_A_b16    <= data_bits_b16;
          dac_reg_B_b16 <= data_bits_b16; --B
          vout_B_b16    <= data_bits_b16; 
        end if;
      
      -----------------
      --power-down DAC (5/4=mode, 1/0=channel)
      when "100" => 
        --normal operation
        if    data_bits_b16(5 downto 4)="00" then
          vout_A_b16 <= vout_A_b16;
          vout_B_b16 <= vout_B_b16;
        --1kohm to GND or 100kohm to GND
        elsif data_bits_b16(5 downto 4)="01" or data_bits_b16(5 downto 4)="10" then
          if data_bits_b16(0)='0' then
            vout_A_b16 <= (others=>'0');
          else
            vout_A_b16 <= vout_A_b16;
          end if;
          if data_bits_b16(1)='0' then
            vout_B_b16 <= (others=>'0');
          else
            vout_B_b16 <= vout_B_b16;
          end if;
        --tristate
        elsif data_bits_b16(5 downto 4)="11" then
          if data_bits_b16(0)='0' then
            vout_A_b16 <= (others=>'Z');
          else
            vout_A_b16 <= vout_A_b16;
          end if;
          if data_bits_b16(1)='0' then
            vout_B_b16 <= (others=>'Z');
          else
            vout_B_b16 <= vout_B_b16;
          end if;
        end if;
      
      -----------------
      --soft reset
      when "101" =>
        dac_reg_A_b16 <= (others=>'0');
        dac_reg_B_b16 <= (others=>'0');
        --full reset
        if data_bits_b16(0)='1' then
          --LDAC
          LDAC_reg_A  <= '0';
          LDAC_reg_B  <= '0';
          --power-down
          vout_A_b16      <= vout_A_b16;
          vout_B_b16      <= vout_B_b16;
        end if;
      
      -----------------
      --LDAC reg (softwsre LDAC) 0=determined by pin (default), 1=update DAC on 24thSCK
      when "110" => 
        LDAC_reg_A <= data_bits_b16(0);
        LDAC_reg_B <= data_bits_b16(1);
        
      -----------------
      --Reserved
      when "111" => 
        null;
      
      -----------------
      when others => 
        null;
          
    end case;
  end if;
end process;

--analog voltage (DAC outputs)
vout_A_ob16 <= dac_reg_A_b16 when (ldac_in='0' or ldac_in='L') and clr_in='1'
          else vout_A_b16;
vout_B_ob16 <= dac_reg_B_b16 when (ldac_in='0' or ldac_in='L') and clr_in='1'
          else vout_B_b16;
         
end architecture;
