------------------------------------------------------------------------------------------
--                                                                                      --
-- eeprom_m24aa64.v - Microchip 24AA64 64K-BIT I2C SERIAL EEPROM (VCC = +1.7V TO +5.5V) --
--                                                                                      --
------------------------------------------------------------------------------------------


-----------------------------------------------------------------------
-- Libraries
----------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--===================================================================
-- Entity
--===================================================================
entity eeprom_at24c32 is
  port (
    --SPI bus
    A0    : in    std_logic;   -- chip select bit
    A1    : in    std_logic;   -- chip select bit
    A2    : in    std_logic;   -- chip select bit
                                
    WP    : in    std_logic;   -- write protect pin
                                
    SDA   : inout std_logic;   -- serial data I/O
    SCL   : in    std_logic;   -- serial data clock
                                
    RESET : in    std_logic    -- system reset
  );
end entity;

--============================================================================
-- architecture 
--============================================================================

architecture behavioral of eeprom_at24c32 is

  signal internal_reset    : std_logic;                     -- component startup 
  
  signal scl_io            : std_logic;                     -- internal signals
  signal sda_io            : std_logic;                     -- internal signals
  
  signal sda_q             : std_logic;                     -- internal signals
  
  signal Rd_Wrn           : std_logic;                      -- 
                                                            
  signal ChipAddress       : std_logic_vector(6 downto 0);  -- hardwired chip address
  signal addr0             : std_logic;                     -- 
  signal addr1             : std_logic;                     -- 
  signal addr2             : std_logic;                     -- 

  signal ShCount           : integer;                       -- 
  signal AddByte           : integer;                       -- 
  signal State             : std_logic_vector(3 downto 0);  -- 
     
  constant c_addr_width    : integer := 13;  
  
  type data_byte is array (integer range <>) of std_logic_vector(7 downto 0);
  signal Mem_m             : data_byte(2**c_addr_width-1 downto 0) := (others=>(others=>'1')); -- EEPROM data memory array
                           
  signal ByteAddr_b        : std_logic_vector(c_addr_width-1 downto 0);
                           
  signal ShReg             : std_logic_vector(7 downto 0);
                           
  constant s_Idle          : std_logic_vector(3 downto 0) := x"0";
  constant s_Addressing    : std_logic_vector(3 downto 0) := x"1";
  -- constant s_WrRdSel       : std_logic_vector(3 downto 0) := x"";
  constant s_AckAddress    : std_logic_vector(3 downto 0) := x"2";
  constant s_ByteAddr      : std_logic_vector(3 downto 0) := x"3";
  constant s_AckByteAddr   : std_logic_vector(3 downto 0) := x"4";
  constant s_GetByte       : std_logic_vector(3 downto 0) := x"5";
  constant s_SendAckData   : std_logic_vector(3 downto 0) := x"6";
  constant s_SendByte      : std_logic_vector(3 downto 0) := x"7";
  constant s_GetAckByte    : std_logic_vector(3 downto 0) := x"8";
  
  constant tAA                : time := 1 ns;                  -- SCL to SDA output delay (short for simu)
  constant tWC                : time := 500 ns;                -- memory write cycle time (short for simu)
  -- VCC_1_7V_TO_2_5V                  -- VCC_2_5V_TO_5_5V
  -- tAA = 3500 ns;     --3.5us        -- tAA = 900 ns;      --9us
  -- tWC = 5000000 ns;  --5ms          -- tWC = 5000000 ns;  --5ms
  
begin

  
  --------------------
  --     Init       --
  --------------------
  p_startup: process
  begin
    internal_reset <= '1';
    wait for 11 ns; -- 3 us reduced for simu
    internal_reset <= '0';
    wait;
  end process;
  
    
  --------------------
  --   Input I2C    --
  --------------------
  scl_io <= '0' when SCL='0' or SCL='L' else '1';
  sda_io <= '0' when SDA='0' or SDA='L' else '1';
  
    
  
  --------------------
  -- Chip address   --
  --------------------
  addr2 <= '0' when A2='0' OR A2='L' else '1';
  addr1 <= '0' when A1='0' OR A1='L' else '1';
  addr0 <= '0' when A0='0' OR A0='L' else '1';
  ChipAddress <= x"A" & addr2 & addr1 & addr0;


  --------------------
  -- FSM     --
  --------------------
  p_State: process(internal_reset, scl_io, sda_io)
  begin
    if internal_reset = '1' then
      State      <= s_Idle;
      ShReg      <= (others=>'0');
      ByteAddr_b <= (others=>'0');
      ShCount    <= 0;
      AddByte    <= (c_addr_width-1)/8;
      Rd_Wrn     <= '1'; --read by default
      
    else
   
      --start
      if falling_edge(sda_io) then
        if scl_io='1' then
          State   <= s_Addressing; 
          ShCount <= 0;
          AddByte <= (c_addr_width-1)/8;
        end if;
      
      --stop    
      elsif rising_edge(sda_io) then
        if scl_io='1' then
          State <= s_Idle; 
        end if;
        
      --main decoding    
      elsif rising_edge(scl_io) then

        case State is 
        
          when s_Addressing =>
            ShReg   <= ShReg(6 downto 0) & sda_io;
            ShCount <= ShCount + 1;
            if ShCount=7 then
              if ShReg(6 downto 0)=ChipAddress then
                State  <= s_AckAddress;
                Rd_Wrn <= sda_io;
              else
                State <= s_Idle;
              end if;
            end if;
            
          -- when s_WrRdSel =>
            -- Rd_Wrn <= ShReg(0);
            -- State  <= s_AckAddress;

          when s_AckAddress =>
            ShCount <= 0;
            AddByte <= (c_addr_width-1)/8;
            if Rd_Wrn='1' then
              State <= s_SendByte;
              ShReg <= Mem_m(to_integer(unsigned(ByteAddr_b)));
            else 
              State <= s_ByteAddr;
            end if;
            
          when s_ByteAddr =>
            ByteAddr_b <= ByteAddr_b(c_addr_width-2 downto 0) & sda_io;   
            ShCount    <= ShCount + 1;
            if ShCount=7 then
              State    <= s_AckByteAddr;
            end if;
            
          when s_AckByteAddr =>
            if AddByte=0 then
              State   <= s_GetByte;
              ShCount <= 0;
              AddByte <= c_addr_width/8-1;
            else 
              State   <= s_ByteAddr;
              AddByte <= AddByte-1;
              ShCount <= 0;
            end if;

          when s_GetByte =>
            ShReg   <= ShReg(6 downto 0) & sda_io;
            ShCount <= ShCount + 1;
            if ShCount=7 and (WP='0' or WP='L') then
              State <= s_SendAckData;         
            end if;
            
          when s_SendAckData =>
            ShCount    <= 0;
            State      <= s_GetByte;
            ByteAddr_b <= std_logic_vector(unsigned(ByteAddr_b) + 1);
            Mem_m(to_integer(unsigned(ByteAddr_b))) <= ShReg; --should use tWC (but 0 for simu)

          when s_SendByte =>
            ShReg   <= ShReg(6 downto 0) & '0';
            ShCount <= ShCount + 1;
            if ShCount=6 then
              ByteAddr_b <= std_logic_vector(unsigned(ByteAddr_b) + 1);
            elsif ShCount=7 then
              State      <= s_GetAckByte;
              -- ByteAddr_b <= std_logic_vector(unsigned(ByteAddr_b) + 1);
              ShReg      <= Mem_m(to_integer(unsigned(ByteAddr_b)));
            end if;

          when s_GetAckByte =>       
            ShCount <= 0;
            if sda_io='1' then
              State <= s_Idle;
            else
              State <= s_SendByte;
            end if;
          
          when others =>
            State   <= s_Idle;
            ShCount <= 0;
            AddByte <= (c_addr_width-1)/8;
            
        end case;
        
      end if;
    end if;
  end process;


    
  --------------------
  -- Timing Checks  --
  --------------------
  p_Timing_Checks: process(internal_reset, scl_io)
  begin
    if internal_reset = '1' then
      sda_q <= 'Z';
        
    elsif falling_edge(scl_io) then
   
      case State is 
        
        when s_Addressing =>
          sda_q <= 'Z'; 
          
        -- when s_WrRdSel =>
          -- sda_q <= 'Z'; 
          
        when s_AckAddress =>
          sda_q <= '0'; 
          
        when s_ByteAddr =>
          sda_q <= 'Z'; 
          
        when s_AckByteAddr =>
          sda_q <= '0'; 
          
        when s_GetByte =>
          sda_q <= 'Z'; 
          
        when s_SendAckData =>
          sda_q <= '0'; 
          
        when s_SendByte =>
          if ShReg(7)='1' then
            sda_q <= 'Z'; 
          else
            sda_q <= '0'; 
          end if;
          
        when s_GetAckByte =>
          sda_q <= 'Z'; 
          
        when others =>
          sda_q <= 'Z'; 
  
      end case;
    end if;
  end process;
  
  SDA <= transport sda_q after tAA; --reduced time for simu
  
 end architecture;
