--------------------------------------------------------------------------------
-- CERN (SY-BI-BL)
-- FMC-HV-2CH test bench
-- https://ohwr.org/project/fmc-hv-2ch
--------------------------------------------------------------------------------
--
-- unit name:     top_tb
--
-- author:        Mathieu Saccani <mathieu.saccani@cern.ch>
--
-- description:
--
--   Implement a master and test various scenarios. Set the g_SCENARIO
--   generic to select the test. See descriptions in the file.
--
-- dependencies:
--
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html

entity top_tb is
  generic (
    g_SCENARIO : natural range 0 to 11 := 0
  );
end top_tb;

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;

architecture behaviour of top_tb is
 
  procedure wait_sig_time 
  (
    signal sig              : in std_logic; 
    constant sig_state      : in std_logic ; 
    constant time_out_value : in time; 
    constant msg            : in string
  ) is
  
    variable start_time_out    : time := 0 ns;
    variable stop_time_out     : time := 0 ns;
    variable signal_reached    : boolean := false;
  
  begin
    start_time_out := NOW ;
    stop_time_out  := 0 ns ;
    signal_reached := false;
    
    while ((signal_reached = false) and (stop_time_out < time_out_value)) loop
        if (sig = sig_state) then signal_reached := true;
        end if ;
        wait for 1 ns;
        stop_time_out := NOW - start_time_out;
    end loop;
    if ((stop_time_out > time_out_value) or (stop_time_out = time_out_value)) then
       report ( "*** ERROR TIME_OUT : Signal => " & msg);
    end if ;
  end procedure wait_sig_time;
 
  --Clock
  constant g_CLOCK_PERIOD          : natural := 10;  -- in ns
  signal clk                       : std_logic;
  
  --Reset
  signal rst                       : std_logic;

  --end of simulation
  signal end_tb                    : std_logic;
  
  --FPGA logic side
  signal fmc_enable_ctrl           : std_logic; 
  signal fmc_enable_stat           : std_logic; 

  signal fmc_HV1_pwm_ena           : std_logic; 
  signal fmc_HV1_pwm_duty_cycle_b8 : std_logic_vector(7 downto 0);
  signal fmc_HV1_pwm_clk_div_b32   : std_logic_vector(31 downto 0);
  signal fmc_HV1_oe                : std_logic; 

  signal fmc_HV2_pwm_ena           : std_logic; 
  signal fmc_HV2_pwm_duty_cycle_b8 : std_logic_vector(7 downto 0);
  signal fmc_HV2_pwm_clk_div_b32   : std_logic_vector(31 downto 0);
  signal fmc_HV2_oe                : std_logic; 

  signal fmc_LV_pwm_ena            : std_logic; 
  signal fmc_LV_pwm_duty_cycle_b8  : std_logic_vector(7 downto 0);   
  signal fmc_LV_pwm_clk_div_b32    : std_logic_vector(31 downto 0);    
  signal fmc_LV_oe                 : std_logic; 

  signal fmc_power_good_status     : std_logic;

  signal DAC1_HV1_wr               : std_logic; 
  signal DAC1_HV1_b16              : std_logic_vector(15 downto 0); 
  signal DAC1_HV1_busy             : std_logic; 
  signal DAC1_TP3_wr               : std_logic; 
  signal DAC1_TP3_b16              : std_logic_vector(15 downto 0); 
  signal DAC1_TP3_busy             : std_logic; 

  signal DAC2_HV2_wr               : std_logic; 
  signal DAC2_HV2_b16              : std_logic_vector(15 downto 0); 
  signal DAC2_HV2_busy             : std_logic; 
  signal DAC2_LV_wr                : std_logic; 
  signal DAC2_LV_b16               : std_logic_vector(15 downto 0); 
  signal DAC2_LV_busy              : std_logic; 

  signal ADC1_ch_b25               : std_logic_vector(24 downto 0); 
  signal ADC1_ch_valid             : std_logic; 
  signal ADC1_temp_rd              : std_logic;
  signal ADC1_temp_b25             : std_logic_vector(24 downto 0); 
  signal ADC1_temp_rd_valid        : std_logic; 
  signal ADC1_temp_busy            : std_logic; 

  signal ADC2_ch_b25               : std_logic_vector(24 downto 0); 
  signal ADC2_ch_valid             : std_logic; 
  signal ADC2_temp_rd              : std_logic;
  signal ADC2_temp_b25             : std_logic_vector(24 downto 0); 
  signal ADC2_temp_rd_valid        : std_logic; 
  signal ADC2_temp_busy            : std_logic; 
 
  signal ADC3_LV_volt_valid        : std_logic;
  signal ADC3_LV_volt_b16          : std_logic_vector(15 downto 0);
  signal ADC3_HV1_curr_valid       : std_logic;
  signal ADC3_HV1_curr_b16         : std_logic_vector(15 downto 0); 
  signal ADC3_HV2_curr_valid       : std_logic;
  signal ADC3_HV2_curr_b16         : std_logic_vector(15 downto 0); 
  signal ADC3_LV_curr_valid        : std_logic;
  signal ADC3_LV_curr_b16          : std_logic_vector(15 downto 0); 
  signal ADC3_4ch_temp_valid       : std_logic;
  signal ADC3_4ch_temp_b16         : std_logic_vector(15 downto 0); 

  --WB to EEPROM
  signal eeprom_wb_cyc             : std_logic;
  signal eeprom_wb_stb             : std_logic;
  signal eeprom_wb_adr_b2          : std_logic_vector(1 downto 0);
  signal eeprom_wb_sel_b4          : std_logic_vector(3 downto 0);
  signal eeprom_wb_we              : std_logic;
  signal eeprom_wb_dat_mosi_b32    : std_logic_vector(31 downto 0);
  signal eeprom_wb_dat_miso_b32    : std_logic_vector(31 downto 0);
  signal eeprom_wb_ack             : std_logic;
  signal eeprom_wb_stall           : std_logic;
  
  constant c_TEST_DATA_BYTE_1      : std_logic_vector(7 downto 0) := x"55";  -- EEPROM test data byte 1
  constant c_TEST_DATA_BYTE_2      : std_logic_vector(7 downto 0) := x"77";  -- EEPROM test data byte 2
  
  --FMC I/F  
  signal fmc_present               : std_logic; 
  signal fmc_power_good            : std_logic;  
  signal FmcPrsntM2c               : std_logic; 
  signal FmcPgM2c                  : std_logic;
  signal fmc_volt_ena              : std_logic;
  signal fmc_volt_fb               : std_logic;  
  signal fmc_HV1_outena            : std_logic;  --tri-state output buffer
  signal fmc_HV2_outena            : std_logic;  --tri-state output buffer
  signal fmc_LV_outena             : std_logic;  --tri-state output buffer
  
  --HV1 DAC1
  signal fmc_hv1_DAC1_sync         : std_logic;
  signal fmc_hv1_DAC1_clk          : std_logic; 
  signal fmc_hv1_DAC1_din          : std_logic; 
                                   
  --HV2 DAC2                       
  signal fmc_hv2_DAC2_sync         : std_logic; 
  signal fmc_hv2_DAC2_clk          : std_logic; 
  signal fmc_hv2_DAC2_din          : std_logic; 
                                   
  --HV1 ADC1                       
  signal fmc_hv1_ADC1_Vfb_clk      : std_logic; 
  signal fmc_hv1_ADC1_Vfb_cs       : std_logic;  
  signal fmc_hv1_ADC1_Vfb_mosi     : std_logic; 
  signal fmc_hv1_ADC1_Vfb_miso     : std_logic; 
                                   
  --HV2 ADC2                       
  signal fmc_hv2_ADC2_Vfb_clk      : std_logic; 
  signal fmc_hv2_ADC2_Vfb_cs       : std_logic;  
  signal fmc_hv2_ADC2_Vfb_mosi     : std_logic; 
  signal fmc_hv2_ADC2_Vfb_miso     : std_logic; 
                                   
  --LV ADC3                        
  signal fmc_lv_ADC3_sck           : std_logic; 
  signal fmc_lv_ADC3_cnv           : std_logic; 
  signal fmc_lv_ADC3_din           : std_logic; 
  signal fmc_lv_ADC3_sdo           : std_logic;
                                   
  --EEPROM                         
  signal FmcEepromScl              : std_logic; 
  signal FmcEepromSda              : std_logic;

  --DAC outputs
  signal hv1_dac_b16               : std_logic_vector(15 downto 0); 
  signal tp3_dac_b16               : std_logic_vector(15 downto 0); 
  signal hv2_dac_b16               : std_logic_vector(15 downto 0); 
  signal lv_dac_b16                : std_logic_vector(15 downto 0); 


  
begin
  
  ----------------------
  -- FMC control (DAC + PWM)
  ----------------------
  u_fmc_ctrl :  process 
  begin
    --Initialisation
    end_tb          <= '0';
    fmc_enable_ctrl <= '0';
    
    --DAC1
    fmc_HV1_pwm_ena           <= '0'; 
    fmc_HV1_pwm_duty_cycle_b8 <= (others=>'0'); 
    fmc_HV1_pwm_clk_div_b32   <= (others=>'0'); 
    fmc_HV1_oe                <= '0';
    DAC1_HV1_wr               <= '0'; 
    DAC1_HV1_b16              <= x"0000";  
    DAC1_TP3_wr               <= '0'; 
    DAC1_TP3_b16              <= x"0000";  
    
    --DAC2
    fmc_HV2_pwm_ena           <= '0'; 
    fmc_HV2_pwm_duty_cycle_b8 <= (others=>'0'); 
    fmc_HV2_pwm_clk_div_b32   <= (others=>'0'); 
    fmc_HV2_oe                <= '0';
    DAC2_HV2_wr               <= '0'; 
    DAC2_HV2_b16              <= x"0000";  
    fmc_LV_pwm_ena            <= '0'; 
    fmc_LV_pwm_duty_cycle_b8  <= (others=>'0'); 
    fmc_LV_pwm_clk_div_b32    <= (others=>'0'); 
    fmc_LV_oe                 <= '0';
    DAC2_LV_wr                <= '0'; 
    DAC2_LV_b16               <= x"0000";  
    
    wait for 1 us; 
    
    --NOT TESTED
    --fmc_present
    --fmc_power_good
    
    --Start the FMC
    fmc_enable_ctrl <= '1'; 
    wait until rising_edge(clk);
    assert fmc_volt_ena='1' report "fmc_volt_ena should be '1'" severity warning;
    assert fmc_volt_fb='1' report "fmc_volt_fb should be '1'" severity warning;
    wait until rising_edge(clk);
    wait for 5000 ns; --settling time
    wait until rising_edge(clk);
    assert fmc_enable_stat='1' report "fmc_enable_stat should be '1'" severity warning;

    
    --Wait for the first SPI programming at startup
    wait for 3 us; 
    wait until rising_edge(clk);
    
    --Enable HV1 PWM
    DAC1_HV1_wr               <= '1'; 
    DAC1_HV1_b16              <= x"8000"; 
    wait until rising_edge(clk) and DAC1_HV1_wr = '1' and DAC1_HV1_busy='0';
    -- wait for 5 ns;
    DAC1_HV1_wr               <= '0'; 
    wait until DAC1_HV1_busy='0';    
    wait for 600 ns; 
    fmc_HV1_pwm_duty_cycle_b8 <= x"80"; 
    fmc_HV1_pwm_clk_div_b32   <= x"00000002"; 
    wait for 10 ns; 
    fmc_HV1_pwm_ena           <= '1'; 
    assert fmc_HV1_outena='Z' report "fmc_HV1_outena should be 'Z'" severity warning;
    wait for 10 ns; 
    fmc_HV1_oe                <= '1';
    wait for 10 ns;
    assert fmc_HV1_outena='1' report "fmc_HV1_outena should be '1'" severity warning;
    wait for 600 ns; 
    wait until rising_edge(clk);
    
    --Enable TP3
    DAC1_TP3_wr               <= '1'; 
    DAC1_TP3_b16              <= x"6000";  
    wait until DAC1_TP3_busy='1';
    DAC1_TP3_wr               <= '0';     
    wait for 600 ns; 
    wait until rising_edge(clk);
    
    --Enable HV2 PWM
    DAC2_HV2_wr               <= '1'; 
    DAC2_HV2_b16              <= x"9000";  
    -- wait for 5 ns;
    wait until rising_edge(clk) and DAC2_HV2_wr = '1' and DAC2_HV2_busy='0';
    DAC2_HV2_wr               <= '0';   
    wait until DAC2_HV2_busy='0';    
    -- wait for 600 ns; 
    fmc_HV2_pwm_duty_cycle_b8 <= x"80"; 
    fmc_HV2_pwm_clk_div_b32   <= x"00000002"; 
    wait for 10 ns; 
    fmc_HV2_pwm_ena           <= '1'; 
    assert fmc_HV2_outena='Z' report "fmc_HV2_outena should be 'Z'" severity warning;
    wait for 10 ns; 
    fmc_HV2_oe                <= '1';
    wait for 10 ns;
    assert fmc_HV2_outena='1' report "fmc_HV2_outena should be '1'" severity warning;
    wait for 600 ns; 
    wait until rising_edge(clk);
    
    --Enable LV PWM
    DAC2_LV_wr               <= '1'; 
    DAC2_LV_b16              <= x"2000";  
    -- wait for 5 ns;
    wait until rising_edge(clk) and DAC2_LV_wr = '1' and DAC2_LV_busy='0';
    DAC2_LV_wr               <= '0';   
    wait until DAC2_LV_busy='0';    
    -- wait for 600 ns; 
    fmc_LV_pwm_duty_cycle_b8 <= x"80"; 
    fmc_LV_pwm_clk_div_b32   <= x"00000002"; 
    wait for 10 ns; 
    fmc_LV_pwm_ena           <= '1'; 
    assert fmc_LV_outena='Z' report "fmc_HV2_outena should be 'Z'" severity warning;
    wait for 10 ns; 
    fmc_LV_oe                <= '1';
    wait for 10 ns;
    assert fmc_LV_outena='1' report "fmc_HV2_outena should be '1'" severity warning;
    wait for 600 ns; 
    wait until rising_edge(clk);
    
    wait for 100 us;
    end_tb <= '1';
    wait;
  end process;



  ----------------------
  -- AD1 status
  ----------------------
  u_adc1_status :  process 
  begin
    
    -- init
    ADC1_temp_rd              <= '0'; 
    
    --time to init the FMC and read once all ADC values
    wait for 50 us;
    
    wait until rising_edge(ADC1_ch_valid);
    assert ADC1_ch_b25='1'& x"110111" report "ADC1_ch_b25 HV1 value is not correct" severity warning; 
    wait until rising_edge(ADC1_ch_valid);
    assert ADC1_ch_b25='1'& x"110111" report "ADC1_ch_b25 HV1 value is not correct" severity warning; 
    wait until rising_edge(ADC1_ch_valid);
    assert ADC1_ch_b25='1'& x"110111" report "ADC1_ch_b25 HV1 value is not correct" severity warning; 
    
    --Read ADC1 temperature value
    wait until rising_edge(clk) and ADC1_temp_rd='0' and ADC1_temp_busy='0';
    ADC1_temp_rd              <= '1'; 
    wait until rising_edge(clk) and ADC1_temp_rd='1' and ADC1_temp_busy='0';
    ADC1_temp_rd              <= '0'; 
    wait until rising_edge(ADC1_temp_rd_valid);
    assert ADC1_temp_b25='0'& x"220222" report "ADC1_temp_b25 temperature value is not correct" severity warning;      

    wait;    
  end process;
  
  
  
  ----------------------
  -- ADC2 status
  ----------------------
  u_adc2_status :  process 
  begin
    
    --init
    ADC2_temp_rd              <= '0'; 
    
    --time to init the FMC and read once all ADC values
    wait for 50 us;
    
    wait until rising_edge(ADC1_ch_valid);
    assert ADC2_ch_b25='0'& x"330333" report "ADC2_ch_b25 HV2 value is not correct" severity warning; 
    wait until rising_edge(ADC1_ch_valid);
    assert ADC2_ch_b25='0'& x"330333" report "ADC2_ch_b25 HV2 value is not correct" severity warning; 
    wait until rising_edge(ADC1_ch_valid);
    assert ADC2_ch_b25='0'& x"330333" report "ADC2_ch_b25 HV2 value is not correct" severity warning; 
    
    -- Read ADC2 temperature value
    wait until rising_edge(clk) and ADC2_temp_rd='0' and ADC2_temp_busy='0';
    ADC2_temp_rd              <= '1'; 
    wait until rising_edge(clk) and ADC2_temp_rd='1' and ADC2_temp_busy='0';
    ADC2_temp_rd              <= '0'; 
    wait until rising_edge(ADC2_temp_rd_valid);
    assert ADC2_temp_b25='0'& x"440444" report "ADC2_temp_b25 temperature value is not correct" severity warning;      

    wait;    
  end process;
  
    
  ----------------------
  -- ADC3 status
  ----------------------
  u_adc3_LV_V_status :  process 
  begin
    -- time to init the FMC and read once all ADC3 values
    wait for 6 us;
    
    wait until rising_edge(ADC3_LV_volt_valid);
    assert ADC3_LV_volt_b16=x"1111" report "ADC3_LV_volt_b16 ADC3 LV value is not correct" severity warning;
    wait;    
  end process;
  
  u_adc3_HV1_I_status :  process 
  begin
    -- time to init the FMC and read once all ADC3 values
    wait for 6 us;
    
    wait until rising_edge(ADC3_HV1_curr_valid);
    assert ADC3_HV1_curr_b16=x"2222" report "ADC3_HV1_curr_b16 ADC3 HV1 current value is not correct" severity warning;
    wait;    
  end process;
  
  u_adc3_HV2_I_status :  process 
  begin
    -- time to init the FMC and read once all ADC3 values
    wait for 6 us;
    wait until rising_edge(ADC3_HV2_curr_valid); --ignore the first value
    
    loop
      wait until rising_edge(ADC3_HV2_curr_valid);
      assert ADC3_HV2_curr_b16=x"3333" report "ADC3_HV2_curr_b16 ADC3 HV2 current value is not correct" severity warning;
    end loop;
    
    wait;    
  end process;
  
  u_adc3_LV_I_status :  process 
  begin
    -- time to init the FMC and read once all ADC3 values
    wait for 6 us;
    wait until rising_edge(ADC3_LV_curr_valid); --ignore the first value
    
    loop
      wait until rising_edge(ADC3_LV_curr_valid);
      assert ADC3_LV_curr_b16=x"4444" report "ADC3_LV_curr_b16 ADC3 LV current value is not correct" severity warning;
    end loop;
    
    wait;    
  end process;
  
  u_adc3_temp_status :  process 
  begin
    -- time to init the FMC and read once all ADC3 values
    wait for 6 us;
    wait until rising_edge(ADC3_4ch_temp_valid); --ignore the first value
    
    loop
        wait until rising_edge(ADC3_4ch_temp_valid);
    assert ADC3_4ch_temp_b16=x"7777" report "ADC3_4ch_temp_b16 ADC3 temperature value is not correct" severity warning;
    end loop;
    
    wait;    
  end process;


  ----------------------
  -- FMC EEPROM (WB)
  ----------------------
  --
  -- WR
  -- w @0x01 0x01000000   #start bit
  -- w @0x01 0x020000A0   #select i2c slave for write x50 + write
  -- w @0x01 0x02AddrMSB  #write reg address MSB
  -- w @0x01 0x02AddrLSB  #write reg address LSB
  -- for loop (max 32)
  --   w @0x01 0x020000DB" #write data byte
  -- end loop  
  -- sleep 0.005          #5ms per page
  -- w @0x01 0x04000000   #stop bit
  -- 
  -- RD
  -- w @0x01 0x01000000   #start bit
  -- w @0x01 0x020000A0   #select i2c slave for read 0x50 + WR
  -- w @0x01 0x02AddrMSB  #address MSB
  -- w @0x01 0x02AddrLSB  #address LSB
  -- w @0x01 0x01000000   #start bit
  -- w @0x01 0x020000A1   #select i2c slave for read 0x50 + RD
  -- for loop (max 32)
  --   w @0x01 0x03000000 #read data and acknowledge (lsb=0)
  --   r @0x00
  -- end loop  
  -- w @0x01 0x03000001   #read last data and no acknowledge (lsb=1)
  -- r @0x00 1
  -- w @0x01 0x04000000   #stop bit
  --
  u_fmc_eeprom :  process 
  begin
    --Initialisation
    eeprom_wb_cyc          <= '0';
    eeprom_wb_stb          <= '0';
    eeprom_wb_adr_b2       <= "00";
    eeprom_wb_sel_b4       <= "1111";
    eeprom_wb_we           <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait for 2 us;
    wait until rising_edge(clk);
    
    -----------------------
    -- WR access
    --w @0x01 0x01000000   #start bit
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"01000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x020000A0   #select i2c slave for write x50 + write
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"020000A0";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x02AddrMSB  #write reg address MSB
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"02000001";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x02AddrLSB  #write reg address LSB
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"02000010";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x020000DB   #write data byte
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"020000" & c_TEST_DATA_BYTE_1;
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x020000DB   #write data byte
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"020000" & c_TEST_DATA_BYTE_2;
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x04000000   #stop bit
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"04000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    wait for 600 ns; --write time in E2PROM (500ns in simu, 5ms in reality)
    
    
    wait for 3 us;
    wait until rising_edge(clk);
     
    ---------------------
    -- RD access
    --w @0x01 0x01000000   #start bit
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"01000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x020000A0   #select i2c slave for write x50 + write
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"020000A0";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x02AddrMSB  #write reg address MSB
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"02000001";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x02AddrLSB  #write reg address LSB
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"02000010";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x01000000   #start bit
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"01000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x020000A1   #select i2c slave for write x50 + read
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"020000A1";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    -- w @0x01 0x03000000   #read data with acknowledge (lsb=0)
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"03000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    -- r @0x00 1
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    assert eeprom_wb_dat_miso_b32(7 downto 0)=c_TEST_DATA_BYTE_1 report "EEPROM read data byte is not the one expected" severity warning;
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    -- w @0x01 0x03000001   #read last data and no acknowledge (lsb=1)
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"03000001";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    -- r @0x00 1
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    assert eeprom_wb_dat_miso_b32(7 downto 0)=c_TEST_DATA_BYTE_2 report "EEPROM read data byte is not the one expected" severity warning;
    wait until rising_edge(clk) and eeprom_wb_stall='0';
    --w @0x01 0x04000000   #stop bit
    eeprom_wb_cyc     <= '1';
    eeprom_wb_stb     <= '1';
    eeprom_wb_adr_b2  <= "01";
    eeprom_wb_we      <= '1';
    eeprom_wb_dat_mosi_b32 <= x"04000000";
    wait until rising_edge(clk) and eeprom_wb_ack='1';
    eeprom_wb_cyc     <= '0';
    eeprom_wb_stb     <= '0';
    eeprom_wb_adr_b2  <= "00";
    eeprom_wb_we      <= '0';
    eeprom_wb_dat_mosi_b32 <= x"00000000";
    wait until rising_edge(clk) and eeprom_wb_stall='0';

    wait;
  end process;
  
  

 
 
  ----------------------
  -- DUT
  i_fmc_hv_2ch_if: entity work.fmc_hv_2ch_if
  generic map(
    --synthesis or simuation mode
    g_SYNTHESIS                => 0,                         -- integer   := 1; -->1=true / 0=false
    --main clock frequency
    g_CLOCK_FREQ               => 100E6                      -- positive := 100E6
  )
  port map(
    --CLK/RST
    rst_ir                     => rst,                       -- in    std_logic; 
    clk_ik                     => clk,                       -- in    std_logic; 
    cen_ie                     => '1',                       -- in    std_logic; 
    --CTRL/STATUS
    present_o                  => open,                      -- out   std_logic; 
    enable_ctrl_i              => fmc_enable_ctrl,           -- in    std_logic; 
    enable_stat_o              => fmc_enable_stat,           -- out   std_logic; 
    power_good_o               => fmc_power_good_status,     -- out   std_logic;
    --PWM HV1
    fmc_HV1_pwm_ena_i          => fmc_HV1_pwm_ena,           -- in    std_logic; 
    fmc_HV1_pwm_duty_cycle_ib8 => fmc_HV1_pwm_duty_cycle_b8, -- in    std_logic_vector(7 downto 0);
    fmc_HV1_pwm_clk_div_ib32   => fmc_HV1_pwm_clk_div_b32,   -- in    std_logic_vector(31 downto 0);
    fmc_HV1_oe_i               => fmc_HV1_oe,                -- in    std_logic; 
    --PWM HV2
    fmc_HV2_pwm_ena_i          => fmc_HV2_pwm_ena,           -- in    std_logic; 
    fmc_HV2_pwm_duty_cycle_ib8 => fmc_HV2_pwm_duty_cycle_b8, -- in    std_logic_vector(7 downto 0);
    fmc_HV2_pwm_clk_div_ib32   => fmc_HV2_pwm_clk_div_b32,   -- in    std_logic_vector(31 downto 0);
    fmc_HV2_oe_i               => fmc_HV2_oe,                -- in    std_logic; 
    --PWM LV
    fmc_LV_pwm_ena_i           => fmc_LV_pwm_ena,            -- in    std_logic; 
    fmc_LV_pwm_duty_cycle_ib8  => fmc_LV_pwm_duty_cycle_b8,  -- in    std_logic_vector(7 downto 0);   
    fmc_LV_pwm_clk_div_ib32    => fmc_LV_pwm_clk_div_b32,    -- in    std_logic_vector(31 downto 0);    
    fmc_LV_oe_i                => fmc_LV_oe,                 -- in    std_logic; 
    --DAC1
    DAC1_HV1_wr_i              => DAC1_HV1_wr,               -- in    std_logic; 
    DAC1_HV1_ib16              => DAC1_HV1_b16,              -- in    std_logic_vector(15 downto 0); 
    DAC1_HV1_busy_o            => DAC1_HV1_busy,             -- out   std_logic; 
    DAC1_TP3_wr_i              => DAC1_TP3_wr,               -- in    std_logic; 
    DAC1_TP3_ib16              => DAC1_TP3_b16,              -- in    std_logic_vector(15 downto 0); 
    DAC1_TP3_busy_o            => DAC1_TP3_busy,             -- out   std_logic; 
    --DAC2
    DAC2_HV2_wr_i              => DAC2_HV2_wr,               -- in    std_logic; 
    DAC2_HV2_ib16              => DAC2_HV2_b16,              -- in    std_logic_vector(15 downto 0); 
    DAC2_HV2_busy_o            => DAC2_HV2_busy,             -- out   std_logic; 
    DAC2_LV_wr_i               => DAC2_LV_wr,                -- in    std_logic; 
    DAC2_LV_ib16               => DAC2_LV_b16,               -- in    std_logic_vector(15 downto 0); 
    DAC2_LV_busy_o             => DAC2_LV_busy,              -- out   std_logic; 
    --ADC1
    ADC1_ch_ob25               => ADC1_ch_b25,               -- out   std_logic_vector(24 downto 0); 
    ADC1_ch_valid_o            => ADC1_ch_valid,             -- out   std_logic; 
    ADC1_temp_rd_i             => ADC1_temp_rd,              -- in    std_logic;
    ADC1_temp_ob25             => ADC1_temp_b25,             -- out   std_logic_vector(24 downto 0); 
    ADC1_temp_rd_valid_o       => ADC1_temp_rd_valid,        -- out   std_logic; 
    ADC1_temp_busy_o           => ADC1_temp_busy,            -- out   std_logic; 
    --ADC2
    ADC2_ch_ob25               => ADC2_ch_b25,               -- out   std_logic_vector(24 downto 0); 
    ADC2_ch_valid_o            => ADC2_ch_valid,             -- out   std_logic; 
    ADC2_temp_rd_i             => ADC2_temp_rd,              -- in    std_logic;
    ADC2_temp_ob25             => ADC2_temp_b25,             -- out   std_logic_vector(24 downto 0); 
    ADC2_temp_rd_valid_o       => ADC2_temp_rd_valid,        -- out   std_logic; 
    ADC2_temp_busy_o           => ADC2_temp_busy,            -- out   std_logic; 
    --ADC3
    ADC3_LV_volt_valid_o       => ADC3_LV_volt_valid,        -- out   std_logic;
    ADC3_LV_volt_ob16          => ADC3_LV_volt_b16,          -- out   std_logic_vector(15 downto 0);
    ADC3_HV1_curr_valid_o      => ADC3_HV1_curr_valid,       -- out   std_logic;
    ADC3_HV1_curr_ob16         => ADC3_HV1_curr_b16,         -- out   std_logic_vector(15 downto 0); 
    ADC3_HV2_curr_valid_o      => ADC3_HV2_curr_valid,       -- out   std_logic;
    ADC3_HV2_curr_ob16         => ADC3_HV2_curr_b16,         -- out   std_logic_vector(15 downto 0); 
    ADC3_LV_curr_valid_o       => ADC3_LV_curr_valid,        -- out   std_logic;
    ADC3_LV_curr_ob16          => ADC3_LV_curr_b16,          -- out   std_logic_vector(15 downto 0); 
    ADC3_4ch_temp_valid_o      => ADC3_4ch_temp_valid,       -- out   std_logic;
    ADC3_4ch_temp_ob16         => ADC3_4ch_temp_b16,         -- out   std_logic_vector(15 downto 0); 
    -- EEPROM WB
    eeprom_wb_cyc_i            => eeprom_wb_cyc,             -- in    std_logic;
    eeprom_wb_stb_i            => eeprom_wb_stb,             -- in    std_logic;
    eeprom_wb_adr_ib2          => eeprom_wb_adr_b2,          -- in    std_logic_vector(1 downto 0);
    eeprom_wb_sel_ib4          => eeprom_wb_sel_b4,          -- in    std_logic_vector(3 downto 0);
    eeprom_wb_we_i             => eeprom_wb_we,              -- in    std_logic;
    eeprom_wb_dat_mosi_ib32    => eeprom_wb_dat_mosi_b32,    -- in    std_logic_vector(31 downto 0);
    eeprom_wb_dat_miso_ob32    => eeprom_wb_dat_miso_b32,    -- out   std_logic_vector(31 downto 0);
    eeprom_wb_ack_o            => eeprom_wb_ack,             -- out   std_logic;
    eeprom_wb_stall_o          => eeprom_wb_stall,           -- out   std_logic;
    ---------------
    --FMC I/F                                                   
    fmc_present_i              => fmc_present,               -- in    std_logic; 
    fmc_power_good_i           => fmc_power_good,            -- in    std_logic;
    fmc_enable_ctrl_o          => fmc_volt_ena,              -- out   std_logic;
    fmc_enable_stat_i          => fmc_volt_fb,               -- in    std_logic;  
    --
    fmc_HV1_outena_o           => fmc_HV1_outena,            -- out   std_logic;  --tri-state output buffer
    fmc_HV2_outena_o           => fmc_HV2_outena,            -- out   std_logic;  --tri-state output buffer
    fmc_LV_outena_o            => fmc_LV_outena,             -- out   std_logic;  --tri-state output buffer
    --                                                       
    fmc_DAC1_sync_no           => fmc_hv1_DAC1_sync,         -- out   std_logic;
    fmc_DAC1_clk_o             => fmc_hv1_DAC1_clk,          -- out   std_logic; 
    fmc_DAC1_din_o             => fmc_hv1_DAC1_din,          -- out   std_logic; 
    --                                                       
    fmc_DAC2_sync_no           => fmc_hv2_DAC2_sync,         -- out   std_logic; 
    fmc_DAC2_clk_o             => fmc_hv2_DAC2_clk,          -- out   std_logic; 
    fmc_DAC2_din_o             => fmc_hv2_DAC2_din,          -- out   std_logic; 
    --                                                       
    fmc_ADC1_Vfb_clk_o         => fmc_hv1_ADC1_Vfb_clk,      -- out   std_logic; 
    fmc_ADC1_Vfb_cs_o          => fmc_hv1_ADC1_Vfb_cs,       -- out   std_logic;  
    fmc_ADC1_Vfb_mosi_o        => fmc_hv1_ADC1_Vfb_mosi,     -- out   std_logic; 
    fmc_ADC1_Vfb_miso_i        => fmc_hv1_ADC1_Vfb_miso,     -- in    std_logic; 
    --                                                       
    fmc_ADC2_Vfb_clk_o         => fmc_hv2_ADC2_Vfb_clk,      -- out   std_logic; 
    fmc_ADC2_Vfb_cs_o          => fmc_hv2_ADC2_Vfb_cs,       -- out   std_logic;  
    fmc_ADC2_Vfb_mosi_o        => fmc_hv2_ADC2_Vfb_mosi,     -- out   std_logic; 
    fmc_ADC2_Vfb_miso_i        => fmc_hv2_ADC2_Vfb_miso,     -- in    std_logic; 
    --                                                       
    fmc_ADC_sck_o              => fmc_lv_ADC3_sck,           -- out   std_logic; 
    fmc_ADC_cnv_o              => fmc_lv_ADC3_cnv,           -- out   std_logic; 
    fmc_ADC_din_o              => fmc_lv_ADC3_din,           -- out   std_logic; 
    fmc_ADC_sdo_i              => fmc_lv_ADC3_sdo,           -- in    std_logic;
    fmc_i2c_prom_scl_iok       => FmcEepromScl,              -- inout std_logic; 
    fmc_i2c_prom_sda_io        => FmcEepromSda               -- inout std_logic 
  );
  

  ----------------------
  --Clock generator
  clk_gen: process
  begin
    clk <= '0';
    wait for (g_CLOCK_PERIOD / 2) * 1 ns;
    clk <= '1';
    wait for (g_CLOCK_PERIOD / 2) * 1 ns;
  end process;
  
  ----------------------
  --Reset generator
  rst_gen: process
  begin
    rst <= '1';
    wait for 51 ns;
    rst <= '0';
    wait;
  end process;
  
  ----------------------
  -- /!\/!\ FMC Status /!\/!\
  FmcPrsntM2c    <= '1';  -- 11k pull-up, should be always '0' on the mezzanine (but ERROR, not connected on the FMC HV 2CH, pull-up on the VFC)
  FmcPgM2c       <= '1';  -- 11k pull-up, (not connected on the FMC HV 2CH, as it uses a LPC connector)
  fmc_present    <= NOT(FmcPrsntM2c);
  fmc_power_good <= NOT(FmcPgM2c); 
  
  ----------------------
  --FMC Control                                             
  fmc_volt_fb <= fmc_volt_ena;
  --fmc_HV1_outena 
  --fmc_HV2_outena 
  --fmc_LV_outena  
  
  ----------------------
  -- HV1 DAC SPI (AD5663) 
  i_fmc_dac1_ad5663_hv1_tp3: entity work.dac_ad5663
  port map(
    --DAC AD5660
    sclk_ik     => fmc_hv1_DAC1_clk,  -- in  std_logic; --fmc_DAC1_sync;
    din_i       => fmc_hv1_DAC1_din,  -- in  std_logic; --fmc_DAC1_clk;
    sync_in     => fmc_hv1_DAC1_sync, -- in  std_logic; --fmc_DAC1_din;
    --triggers        
    ldac_in     => '0',               -- in  std_logic; 
    clr_in      => '1',               -- in  std_logic; 
    -- analog voltage outputs
    vout_A_ob16 => hv1_dac_b16,       --  out std_logic_vector(15 downto 0);  -- : 0xFFFF=3.3V and 0x0000=0V
    vout_B_ob16 => tp3_dac_b16        --  out std_logic_vector(15 downto 0)   -- : 0xFFFF=3.3V and 0x0000=0V
  );
  
  -- HV2 DAC SPI (AD5663) 
  i_fmc_dac2_ad5663_hv2_lv: entity work.dac_ad5663
  port map(
    --DAC AD5660
    sclk_ik     => fmc_hv2_DAC2_clk, -- in  std_logic; --fmc_DAC1_sync;
    din_i       => fmc_hv2_DAC2_din,  -- in  std_logic; --fmc_DAC1_clk;
    sync_in     => fmc_hv2_DAC2_sync, -- in  std_logic; --fmc_DAC1_din;
    --triggers        
    ldac_in     => '0',               -- in  std_logic; 
    clr_in      => '1',               -- in  std_logic; 
    -- analog voltage outputs     
    vout_A_ob16 => hv2_dac_b16,       --  out std_logic_vector(15 downto 0);  -- : 0xFFFF=3.3V and 0x0000=0V
    vout_B_ob16 => lv_dac_b16         --  out std_logic_vector(15 downto 0)   -- : 0xFFFF=3.3V and 0x0000=0V
  );


  ----------------------
  -- HV1 ADC1 SPI (LTC2484)
  i_fmc_adc1_hv1_ltc2484: entity work.adc_ltc2484
  generic map(
    G_TIMING_CHECK_EN => '0'            -- std_logic := '0' 
  )
  port map(
    --Analog input
    Vin_ib25  => '1'&x"110111",         -- in  std_logic_vector(23 downto 0); --
    Temp_ib24 => x"220222",             -- in  std_logic_vector(23 downto 0); --
    --SPI bus     
    SCK       => fmc_hv1_ADC1_Vfb_clk,  -- in  std_logic;   -- Serial clock input
    SDI       => fmc_hv1_ADC1_Vfb_mosi, -- in  std_logic;   -- Serial data input
    SDO       => fmc_hv1_ADC1_Vfb_miso, -- out std_logic;   -- Serial data output
    CS_N      => fmc_hv1_ADC1_Vfb_cs    -- in  std_logic    -- Chip select input
  );
  
  fmc_hv1_ADC1_Vfb_cs   <= 'L'; --1k pull-down on FMC

  
  ----------------------
  -- HV2 ADC2 SPI (LTC2484)
  i_fmc_adc2_hv2_ltc2484: entity work.adc_ltc2484
  generic map(
    G_TIMING_CHECK_EN => '0'            -- std_logic := '0' 
  )
  port map(
    --Analog input
    Vin_ib25  => '0'& x"330333",        -- in  std_logic_vector(23 downto 0); --
    Temp_ib24 => x"440444",             -- in  std_logic_vector(23 downto 0); --
    --SPI bus     
    SCK       => fmc_hv2_ADC2_Vfb_clk,  -- in  std_logic;   -- Serial clock input
    SDI       => fmc_hv2_ADC2_Vfb_mosi, -- in  std_logic;   -- Serial data input
    SDO       => fmc_hv2_ADC2_Vfb_miso, -- out std_logic;   -- Serial data output
    CS_N      => fmc_hv2_ADC2_Vfb_cs    -- in  std_logic    -- Chip select input
  );

  fmc_hv2_ADC2_Vfb_cs   <= 'L'; --1k pull-down on FMC
  
  ----------------------
  -- LV ADC3 SPI (AD7682)
  i_fmc_adc3_lv_ad7682: entity work.adc_ad7682
  generic map(
    G_TIMING_CHECK_EN => '0'              -- std_logic := '0' 
  )
  port map(
    --Analog input
    Vin0_ib16         => x"1111",         -- std_logic_vector(15 downto 0); -- ADC3 LV  
    Vin1_ib16         => x"2222",         -- std_logic_vector(15 downto 0); -- ADC3 HV1 current 
    Vin2_ib16         => x"3333",         -- std_logic_vector(15 downto 0); -- ADC3 HV2 current 
    Vin3_ib16         => x"4444",         -- std_logic_vector(15 downto 0); -- ADC3 LV current
    Temp_ib16         => x"7777",         -- std_logic_vector(15 downto 0); -- ADC3 temperature
    --SPI bus     
    SCK               => fmc_lv_ADC3_sck, -- in  std_logic;   -- Serial clock input
    DIN               => fmc_lv_ADC3_din, -- in  std_logic;   -- Serial data input
    SDO               => fmc_lv_ADC3_sdo, -- out std_logic;   -- Serial data output
    CVN               => fmc_lv_ADC3_cnv  -- in  std_logic    -- Chip select input
  );

  ----------------------
  --EEPROM
  i_fmc_prom_i2c: entity work.eeprom_at24c32
  port map(
   A0    => 'L',    -- 10k pull down 
   A1    => 'L',    -- 10k pull down 
   A2    => '0', 
   WP    => '0',
   SDA   => FmcEepromSda, 
   SCL   => FmcEepromScl, 
   RESET => '0'
  );
  --I2C
  FmcEepromScl <= 'H' ; -- 1k pull up 
  FmcEepromSda <= 'H' ; -- 1k pull up 

  -----------------------------------------------------------------------
  -- Testbench Main Timer
  -----------------------------------------------------------------------
  u_scenario :  process 
    begin
      wait_sig_time(end_tb, '1', 1 ms, "End of test");
      report "End of test (wait for GHDL stop-time)";
      -- finish;
      -- stop;
      wait;
  end process;

end behaviour;
